/**
	$Id: ExportedProperty.java 7650 2019-03-14 08:31:57Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.plugins;

import net.sf.basedb.clients.web.formatter.FormatterFactory;
import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.FileSetMember;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Metadata;
import net.sf.basedb.core.Nameable;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.Unit;
import net.sf.basedb.core.Metadata.PropertyPath;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.excel.ExcelFormatter;
import net.sf.basedb.util.excel.WrappedFormatter;
import net.sf.basedb.util.formatter.FilePathFormatter;
import net.sf.basedb.util.formatter.Formatter;
import net.sf.basedb.util.formatter.ToStringFormatter;

/**

 @author nicklas
 @version 2.2
 @base.modified $Date: 2019-03-14 09:31:57 +0100 (tors, 14 mars 2019) $
 */
public class ExportedProperty
{
	public final String name;
	public final String title;
	public final AnnotationType annotationType;
	public final boolean loadInherited; 
	public final Unit unit;
	@SuppressWarnings("rawtypes")
	public final Formatter formatter;
	@SuppressWarnings("rawtypes")
	public final ExcelFormatter excelFormatter;
	@SuppressWarnings("rawtypes")
	public final DataLoader dataloader;
	@SuppressWarnings("rawtypes")
	public final PropertyPath propertyPath;

	@SuppressWarnings("rawtypes")
	public static ExportedProperty createExportedProperty(String path, String title, Metadata metadata, Formatter formatter, ExcelFormatter excelFormatter)
	{
		PropertyPath propertyPath = metadata.getPropertyPath(path, false);
		return new ExportedProperty(path, title, propertyPath, null, false, null, formatter, excelFormatter, null);
	}

	
	/**
	 	@since 2.15
	*/
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public static ExportedProperty parse(DbControl dc, Metadata metadata, String column, String columnPrefix, 
			boolean annotatable, Formatter formatter, ExcelFormatter excelFormatter, boolean sameUnits, DataLoader dataloader)
	{
		String[] p = column.split(":");
		String name = p[0];
		Type valueType = Type.fromValue(p[1]);
		String title = p.length > 2 ? p[2] : p[0];
		AnnotationType at = null;
		boolean loadInherited = false;
		Unit unit = null;
		String columnSuffix = null;
		PropertyPath propertyPath = null;
		if (name.startsWith("#") && annotatable) // && itemType != Item.SPOTDATA)
		{
			if (name.contains("("))
			{
				int joinStart = name.indexOf('(');
				int joinEnd = name.indexOf(')');
				propertyPath = metadata.getPropertyPath(name.substring(joinStart+1, joinEnd), false);
				name = name.substring(0, joinStart) + name.substring(joinEnd+1); // Should give us: #id
			}
			
			loadInherited = name.startsWith("##");
			// Annotation: properties[i] = #annotationTypeId
			int annotationTypeId = Values.getInt(loadInherited ? name.substring(2) : name.substring(1));
			at = AnnotationType.getById(dc, annotationTypeId);
			if (at.getDisableInheritance() && loadInherited) return null;
			name = "annotationtype_"+annotationTypeId;
			if (sameUnits && at.supportUnits()) 
			{
				unit = at.getDefaultUnit();
				if (unit != null) columnSuffix = " (" + unit.getDisplaySymbol() + ")";
			}
		}
		else if (name.startsWith("¤"))
		{
			// Data files name=¤<id>(file-filter)
			int i = name.indexOf('(');
			int fileTypeId = Values.getInt(name.substring(1, i));
			String fileFilter = name.substring(i+1, name.length()-1);
			DataFileType fileType = DataFileType.getById(dc, fileTypeId);
			if ("file.path".equals(fileFilter))
			{
				// Special case handled by loading the FILE object and formatting using the file path formatter
				formatter = new FilePathFormatter();
				fileFilter = "file";
			}
			// We just use whatever that is specified in the file filter part of the property 
			propertyPath = Metadata.getInstance(FileSetMember.class).getPropertyPath(fileFilter, false);
			dataloader = new FileSetMemberLoader(fileType, propertyPath);
		}
		else if (name.startsWith("|") && dataloader == null)
		{
			// Any-to-any=|link|type|property
			String[] any = name.split("\\|");
			String linkName = any[1];
			Item linkType = any.length > 2 ? Item.valueOf(any[2]) : null;
			String linkProperty = any.length > 3 ? any[3] : null;
			
			if (linkProperty == null)
			{
				// Use 'name' if possible when no link property has been specified
				if (linkType != null && Nameable.class.isAssignableFrom(linkType.getItemClass()))
				{
					linkProperty = "name";
					if (formatter == null)
					{
						formatter = FormatterFactory.getTypeFormatter(dc.getSessionControl(), Type.STRING);
					}
				}
				else
				{
					if (formatter == null)
					{
						formatter = FormatterFactory.getTypeFormatter(dc.getSessionControl(), Type.INT);
					}
				}
			}
			
			if (linkProperty != null)
			{
				propertyPath = Metadata.getInstance(linkType.getItemClass()).getPropertyPath(linkProperty, false);
			}
			dataloader = new AnyToAnyDataLoader(linkName, linkType, propertyPath);
		}
		else if (name.startsWith("@") && name.endsWith(")") && dataloader == null)
		{
			// Map collection: @collection-name(key) 
			int i = name.indexOf('(');
			String collectionName = name.substring(1, i);
			propertyPath = metadata.getPropertyPath(collectionName, false);
			
			String key = name.substring(i+1, name.length()-1);
			// If 'key' is quoted it is a string, otherwise a number
			Object collectionKey = key.startsWith("'") ?
					key.substring(1, key.length()-1) :
					Values.getInt(key);
			dataloader = new MapValueLoader(propertyPath, collectionKey);
		}
		else if (dataloader == null && metadata != null)
		{
			propertyPath = metadata.getPropertyPath(name, false);
		}
		if (formatter == null) formatter = FormatterFactory.getTypeFormatter(dc.getSessionControl(), valueType);
		if (columnPrefix != null) title = columnPrefix + title;
		if (columnSuffix != null) title += columnSuffix;
		return new ExportedProperty(name, title, propertyPath, at, loadInherited, unit, formatter, excelFormatter, dataloader);
	}

	/**
		@since 3.15
	*/
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private ExportedProperty(String name, String title, PropertyPath propertyPath,
			AnnotationType annotationType, boolean loadInherited, Unit unit, Formatter formatter, ExcelFormatter excelFormatter, DataLoader dataloader)
	{
		this.name = name;
		this.title = title;
		this.propertyPath = propertyPath;
		this.annotationType = annotationType;
		this.loadInherited = loadInherited;
		if (formatter == null) formatter = new ToStringFormatter();
		this.formatter = formatter;
		if (excelFormatter == null)
		{
			if (formatter instanceof ExcelFormatter)
			{
				excelFormatter = (ExcelFormatter)formatter;
			}
			else
			{
				excelFormatter = new WrappedFormatter(formatter);
			}
		}
		this.excelFormatter = excelFormatter;
		this.unit = unit;
		this.dataloader = dataloader;
	}
}
