/*
	$Id: HideableSection.java 7690 2019-04-05 12:19:47Z nicklas $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;

import net.sf.basedb.core.ItemContext;

/**
	This tag is used to display a section on a web page that
	can be shown/hidden by the user by clicking on an icon
	on it's title.
	<p>
	<b>Example:</b>
	<pre>
   &lt;%@ taglib prefix="base" uri="/WEB-INF/base.tld" %&gt;
   &lt;base:section id="my.section" title="Click to hide/show"&gt;
   This is the contents of this section...
   &lt;/base:section&gt;
</pre>

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;base:section
      id=...
      clazz=...
      showclazz=...
      hideclazz=...
      style=...
      contentstyle=...
      title=...
      visible=...
      ontoggle=...
      initial=show|hide
      context=...
   &gt;
</pre>

	<table class="as-grid">
	<caption>Tag attributes</caption>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>id</td>
		<td>-</td>
		<td>no</td>
		<td>The ID of the section. The value of this attribute
		goes directly into the standard <code>id</code> attribute.
		This means that if you need a reference to the scetion object in your own
		JavaScript code, you may for example use the following code:
		<p>
		<code>
		section = document.getElementById('&lt;id&gt;');
		</code>
		</td>
	</tr>
	<tr>
		<td>clazz</td>
		<td>note</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>class</code> attribute both when the section contents is hidden or
			visible.
		</td>
	</tr>
	<tr>
		<td>showclazz</td>
		<td>note</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>class</code> attribute when the section contents is visible.
		</td>
	</tr>
	<tr>
		<td>hideclazz</td>
		<td>note</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>class</code> attribute when the section contents is hidden.
		</td>
	</tr>
	<tr>
		<td>style</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute. This allows you to apply different styles
			to different sections.
		</td>
	</tr>
	<tr>
		<td>contentstyle</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute of the content part. This allows you to apply different styles
			to different sections.
		</td>
	</tr>
	<tr>
		<td>title</td>
		<td></td>
		<td>no</td>
		<td>The title of the section.</td>
	</tr>
	<tr>
		<td>visible</td>
		<td>true</td>
		<td>no</td>
		<td>
			If the section should be visible or not.
		</td>
	</tr>
	<tr>
		<td>initial</td>
		<td>show</td>
		<td>no</td>
		<td>
			If the value is <code>hide</code> the section is hidden, otherwise it
			is visible. This setting is ignored if a <code>context</code> is given.  
		</td>
	</tr>
	<tr>
		<td>context</td>
		<td>-</td>
		<td>no</td>
		<td>
			An optional {@link ItemContext} object. If given the <code>initial</code>
			atribute is ignored. The valus is instead taken from the context setting
			<code>section.id</code> where id is replaced with the value of the 
			<code>id</code> attribute. This should have the value "1" to show the
			section or "0" to hide the section. If a context is given the default
			javascript method will also send an Ajax request to the web server which
			will update the setting in the context with a new value for the
			show/hide setting. Thus, all settings are remembered during a session 
			and also when the user returns to the next session.
		</td>
	</tr>
	</table>

	@author Nicklas
	@version 2.4
*/
public class HideableSection
	extends StylableTag
{

	private static final long serialVersionUID = 4750788983708350436L;

	public HideableSection()
	{
		super("hideablesection");
	}
	
	/**
		The parent &lt;base:page&gt; tag.
	*/
	private Page page;

	private String showClass = null;
	private String hideClass = null;
	
	/**
		Optional <code>style</code> attribute for the content section
	*/
	private String contentStyle = null;

	/**
		The title of the section.
	*/
	private String title = null;

	/**
		If the section should be visible or not.
	*/
	private boolean visible = true;
	
	/**
		If the section should be shown or hidden to start with.
	*/
	private boolean showInitially = true;
	
	private transient ItemContext context = null;
	
	/*
		Taglib initialization methods
	*/
	public void setShowclazz(String clazz)
	{
		this.showClass = clazz;
	}
	public String getShowclazz()
	{
		return showClass;
	}	
	public void setHideclazz(String clazz)
	{
		this.hideClass = clazz;
	}
	public String getHideclazz()
	{
		return hideClass;
	}	
	
	public void setContentstyle(String contentStyle)
	{
		this.contentStyle = contentStyle;
	}
	public String getContentstyle()
	{
		return contentStyle;
	}

	public void setTitle(String title)
	{
		this.title = title;
	}
	public String getTitle()
	{
		return title;
	}

	public void setVisible(boolean visible)
	{
		this.visible = visible;
	}
	public boolean isVisible()
	{
		return visible;
	}

	public void setInitial(String initial)
	{
		this.showInitially = !"hide".equals(initial);
	}
	public String getInitial()
	{
		return showInitially ? "show" : "hide";
	}

	public void setContext(ItemContext context)
	{
		this.context = context;
	}
	public ItemContext getContext()
	{
		return context;
	}

	@Override
	public int doStartTag()
		throws JspException
	{
		page = (Page)findAncestorWithClass(this, Page.class);
		if (page == null) throw new JspTagException("Tag <base:section> must be inside a <base:page> tag");
		String id = getId();
		if (id == null) throw new JspTagException("id attribute must have a value");
		if (!isVisible()) return SKIP_BODY;

		boolean showContent = context == null ? showInitially : !"0".equals(context.getSetting("section."+id));
		String clazz = getClazz();
		String contentStyle = getContentstyle();
		String icon = "";
		if (showContent)
		{
			icon = "hide_section.png";
		}
		else
		{
			contentStyle = "display: none; " + (contentStyle != null ? contentStyle : "");
			icon = "show_section.png";
		}
		
		StringBuilder sb = new StringBuilder();
		sb.append("<div");
		addIdAndStyles(sb, showContent ? getShowclazz() : getHideclazz());
		if (getShowclazz() != null) sb.append(" data-showclass=\"").append(getShowclazz()).append("\"");
		if (getHideclazz() != null) sb.append(" data-hideclass=\"").append(getHideclazz()).append("\"");
		if (context != null) 
		{
			sb.append(" data-context=\"").append(context.getItemType().name()).append("\"");
			sb.append(" data-subcontext=\"").append(context.getSubContext()).append("\"");
		}
		addDynamicAttributes(sb);
		sb.append(">\n");
		sb.append("<h2 id=\"").append(id).append(".title\""); 
		sb.append(" class=\"link auto-init\" data-auto-init=\"hideable-section\"");
		sb.append(" title=\"Click to show/hide the contents of this section\"");
		sb.append(">");
		sb.append("<img id=\"").append(id).append(".icon\" src=\"");
		sb.append(page.getRoot()).append("images/").append(icon).append("\"");
		sb.append(" alt=\"►\">");
		sb.append(getTitle()).append("</h2>\n");
		sb.append("<div id=\"").append(id).append(".content\"");
		if (contentStyle != null) sb.append(" style=\"").append(contentStyle).append("\"");
		sb.append(">");

		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_BODY_INCLUDE;
	}

	@Override
	public int doEndTag()
		throws JspException
	{
		if (!isVisible()) return EVAL_PAGE;
		StringBuilder sb = new StringBuilder();
		sb.append("</div>\n</div>\n");
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_PAGE;
	}
}




