/*
	$Id: Help.java 6160 2012-10-10 09:32:57Z nicklas $

	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.clients.web.taglib;


import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;

/**
  This tag is used to make it easier to add zoom features on a BASE web page.
  <p>
  <b>Example:</b>
  <pre>
  	&lt;%@ taglib prefix="base" uri="/WEB-INF/base.tld" %&gt;
  	&lt;base:zoom textarea="required_textarea_ID" title="dialog title" /&gt;
  </pre>
  
  <p>
  <b>Syntax:</b>
  <pre>
  &lt;base:zoom
  	clazz=...
  	subclass=...
  	style=...
  	image=...
  	textarea=...
  	title=...
  	tooltip=...
  	visible=...
  	tabindex=...
  &gt;
  </pre>

  	<table class="as-grid">
 	<caption>Tag attributes</caption>
 	<tr>
	  	<td><b>Attribute</b></td>
	  	<td><b>Default value</b></td>
	  	<td><b>Required</b></td>
	  	<td><b>Description</b></td>
  	</tr>
  	<tr>
  		<td>clazz</td>
  		<td>zoom</td>
  		<td>no</td>
  		<td>
  			The value if this attribute goes directly into the standard HTML 
			<code>class</code> attribute. This allows you to apply different styles
			to different zoom icons.  		
  		</td>
  	</tr>
	<tr>
		<td>subclass</td>
		<td>-</td>
		<td>no</td>
		<td>
			Additional classes to put in the HTML class attribute. Doesn't
			replace the main class attribute.
		</td>
	</tr>
  	<tr>
  		<td>style</td>
  		<td>-</td>
  		<td>no</td>
  		<td>
  			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute. This allows you to apply different styles
			to different icons.
  		</td>
  	</tr>
  	<tr>
  		<td>image</td>
  		<td>zoom.png</td>
  		<td>no</td>
  		<td>
  			The name of the image to display.
		</td>
	</tr>
  	<tr>
  		<td>textarea</td>
  		<td>-</td>
  		<td>yes</td>
  		<td>
  			The ID attribute of a textarea field on the page.
		</td>
	</tr>
  	<tr>
  		<td>title</td>
  		<td>-</td>
  		<td>no</td>
  		<td>
  			The title of the zoom popup dialog.
		</td>
	</tr>
  	<tr>
  		<td>tooltip</td>
  		<td>Edit in larger window</td>
  		<td>no</td>
  		<td>
  			A short explanation that will show up as a tooltip.
  		</td>
  	</tr>
  	<tr>
  		<td>visible</td>
  		<td>true</td>
  		<td>no</td>
  		<td>
  			If the zoom icon should be visible or not.
  		</td>
  	</tr>
 	<tr>
		<td>tabindex</td>
		<td>0</td>
		<td>no</td>
		<td>
			Tab index value to control the focus order. Use -1 to disable tabbing to the icon.
		</td>
	</tr>
  	</table>
  	
  
  @author nicklas
  @since 3.3
*/
public class Zoom 
	extends StylableTag
{

	private static final long serialVersionUID = 5226627932346514174L;

	/**
	 	The parent &lt;base:page&gt; tag.	 	
	*/
	private Page page;
		
	/**
	  	The path to the help icon
	*/
	private String image = "zoom.png";
	
	/**
	 	Optional tooltip.
	*/
	private String tooltip = "Edit in larger window";
	
	/**
	 * 	If the zoom icon should be visible or not.
	 */
	private boolean visible = true;
	
	private int tabIndex;
	
	private String textarea;
	private String title;
	
	
	public Zoom()
	{
		super("zoomicon");
	}
	
	/*
	 	Taglib initialization methods
	*/
	public void setImage(String image)
	{
		this.image = image;		
	}
	public String getImage()
	{
		return this.image;
	}

	public void setTextarea(String textarea)
	{
		this.textarea = textarea;		
	}
	public String getTextarea()
	{
		return this.textarea;
	}

	public void setTitle(String title)
	{
		this.title = title;		
	}
	public String getTitle()
	{
		return this.title;
	}

	public void setTooltip(String tooltip)
	{
		this.tooltip = tooltip;
	}
	public String getTooltip()
	{
		return this.tooltip;
	}
	
	public void setVisible(boolean visible)
	{
		this.visible = visible;
	}
	public boolean isVisible()
	{
		return this.visible;
	}
	
	public void setTabindex(int tabIndex)
	{
		this.tabIndex = tabIndex;
	}
	public int getTabindex()
	{
		return tabIndex;
	}
	
	@Override
	public int doStartTag() 
		throws JspException
	{
		page = (Page)findAncestorWithClass(this, Page.class);
		if (page == null) throw new JspTagException("Tag <base:zoom> must be inside a <base:page> tag");
		if (!isVisible()) return SKIP_BODY;
		
		StringBuilder sb = new StringBuilder();
		sb.append("<img");
		addIdAndStyles(sb, "auto-init");
		sb.append(" src=\"").append(page.getRoot()).append("images/").append(getImage()).append("\"");
		sb.append(" data-auto-init=\"zoom\"");
		if (getTextarea() != null)
		{
			sb.append(" data-textarea-id=\"").append(getTextarea()).append("\"");
		}
		if (getTitle() != null)
		{
			sb.append(" data-title=\"").append(getTitle()).append("\"");
		}
		if (getTooltip() != null)
		{
			sb.append(" title=\"").append(getTooltip()).append("\"");
		}
		if (getTabindex() >= 0) 
		{
			sb.append(" tabindex=\"").append(getTabindex()).append("\"");
		}
		sb.append(" alt=\"+\"");
		addDynamicAttributes(sb);
		sb.append(">");
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}		
		return EVAL_BODY_INCLUDE;		
	}
}
