/*
	$Id: Menu.java 7690 2019-04-05 12:19:47Z nicklas $

	Copyright (C) 2005 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Gregory Vincic

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib.menu;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.http.HttpServletRequest;

import net.sf.basedb.clients.web.taglib.StylableTag;


/**
	This tag defines a drop-down menu that is is opened when the user moves the
	mouse over or clicks at the title of the menu.
	<p>
	This tag can only be used inside a {@link net.sf.basedb.clients.web.taglib.Body} tag and you should have at
	least one of these tags. Use {@link Menuitem} and/or {@link Menuseparator}
	tags inside this tag to define the contents of the drop-down menu.

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;m:menu
      id=...
      type=...
      clazz=...
      subclass=...
      style=...
      open=...
      filltext=...
   &gt;
</pre>


	<table class="as-grid">
	<caption>Tag attributes</caption>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>id</td>
		<td>-</td>
		<td>yes</td>
		<td>The ID of the menu. The value of this attribute
		goes directly into the standard HTML <code>id</code> attribute.
		This means that if you need a reference to the menu object in your own
		JavaScript code, you may for example use the following code:
		<p>
		<code>
		menu = document.getElementById('&lt;id&gt;');
		</code>
		</td>
	</tr>
	<tr>
		<td>type</td>
		<td>vertical</td>
		<td>no</td>
		<td>
			If the menu should be displayed vertically or horizontally. The 
			allowed values are <code>horizontal</code> and <code>vertical</code>
		</td>
	</tr>

	<tr>
		<td>clazz</td>
		<td>menu_vertical</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>class</code> attribute. This allows you to apply different styles
			to different menus. If you set the type attribute to horizontal you
			should also change this attribute to <code>menu_horizontal</code>.
		</td>
	</tr>
	<tr>
		<td>style</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute. This allows you to apply different styles
			to different menus. Note! If you want the menu to be hidden to begin with
			you can use: <code>style="display: none;"</code>.
		</td>
	</tr>
	<tr>
		<td>open</td>
		<td>enter</td>
		<td>no</td>
		<td>A value indicating if it requires a mouse click or not to
			open a menu. Valid values are:
			<ul>
			<li>enter: The menu will open as soon as the mouse enters
			<li>click: The menu will not open until the mouse is clicked
			</ul>
		</td>
	</tr>
	<tr>
		<td>filltext</td>
		<td>-</td>
		<td>no</td>
		<td>
			A short text that will be displayed to the far right of the menu if it
			is a horizontal menu. This value is ignored for vertical menus.
		</td>
	</tr>
	</table>

	@author Nicklas
	@version 2.0
*/
@SuppressWarnings("serial")
public class Menu 
	extends StylableTag
{

	/**
		The path to the root directory of this webserver.
	*/
	private static String ROOT = null;


	/**
		If it is a vertical or horizontal menu.
	*/
	private String type = "vertical";

	/**
		If submenus opens as soon as the mouse enters or a click is required.
	*/
	private String open = "enter";

	/**
		Optional text to display to the far right of a horizontal menu
	*/
	private String fillText = null;
	
	public Menu()
	{
		super("menu bg-filled-100");
	}
	
	/*
		Taglib initialization methods
	*/
	public void setType(String type)
	{
		this.type = type;
	}
	public String getType()
	{
		return type;
	}

	public void setOpen(String open)
	{
		this.open = open;
	}
	public String getOpen()
	{
		return open;
	}

	public void setFilltext(String fillText)
	{
		this.fillText = fillText;
	}
	public String getFilltext()
	{
		return fillText;
	}

	boolean isVertical()
	{
		return "vertical".equalsIgnoreCase(getType());
	}
	
	void addMenuitem(String html)
		throws Exception
	{
		if (isVertical())
		{
			pageContext.getOut().print("<tr><td>"+html+"</td></tr>");
		}
		else
		{
			pageContext.getOut().print("<td>"+html+"</td>");
		}
	}
	
	String getRoot()
	{
		return ROOT;
	}
	
	@Override
	public int doStartTag()
		throws JspException
	{
		if (ROOT == null) ROOT = ((HttpServletRequest)pageContext.getRequest()).getContextPath()+"/";

		StringBuilder sb = new StringBuilder();
		sb.append("<div");
		addIdAndStyles(sb, isVertical() ? "vertical" : "horizontal", "auto-init");
		sb.append(" data-auto-init=\"menu\"");
		sb.append(" data-vertical=\"").append(isVertical() ? "1" : "0").append("\"");
		sb.append(" data-open=\"").append(getOpen()).append("\"");
		addDynamicAttributes(sb);
		sb.append(">\n");
		sb.append("<table>\n");
		if (isVertical()) 
		{
			sb.append("<tr><td><div class=\"menustart\"></div></td></tr>\n");
		}
		else
		{
			sb.append("<tr>\n");
		}

		try
		{
			pageContext.getOut().print(sb.toString());
    	}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_BODY_INCLUDE;
	}

	@Override
	public int doEndTag() 
		throws JspException
	{
		StringBuilder sb = new StringBuilder();
		if (!isVertical()) 
		{
			sb.append("<td class=\"menufill\">");
			if (getFilltext() != null) sb.append(getFilltext());
			sb.append("</td></tr>\n");
		}
		sb.append("</table>\n");
		sb.append("</div>\n");
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_PAGE;
	}
}
