/*
	$Id: Header.java 7690 2019-04-05 12:19:47Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib.table;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.Tag;

import net.sf.basedb.clients.web.taglib.StylableTag;

/**
	This tag defines a header column among the colummns
	or on a row. It should be placed inside a {@link HeaderRow &lt;tbl:headers&gt;} 
	or {@link Row &lt;tbl:row&gt;} tag.

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;tbl:header
      clazz=...
      style=...
      visible=true|false
   &gt;
</pre>

	<table class="as-grid">
	<caption>Tag attributes</caption>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>clazz</td>
		<td>-</td>
		<td>header</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>class</code> attribute. This allows you to apply different styles
			to different tables.
		</td>
	</tr>
	<tr>
		<td>style</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute. This allows you to apply different styles
			to different tables.
		</td>
	</tr>
	<tr>
		<td>visible</td>
		<td>true</td>
		<td>no</td>
		<td>
			If the header should be visible or not.
		</td>
	</tr>
	</table>
	<p>
	The <code>&lt;tbl:header&gt;</code> doesn't have any subtags.

	@author Nicklas
	@version 2.0
*/
public class Header
	extends StylableTag
{

	/**
		
	*/
	private static final long serialVersionUID = -7541520513764712983L;

	public Header()
	{
		super(null);
	}
	
	private  int colspan = 1;
	
	/**
		If the header should be visible or not.
	*/
	private boolean visible = true;
	
	/*
		Taglib initialization methods
	*/

	public void setVisible(boolean visible)
	{
		this.visible = visible;
	}
	public boolean isVisible()
	{
		return visible;
	}

	public void setColspan(int colspan)
	{
		this.colspan = colspan;
	}
	
	public int getColspan()
	{
		return colspan;
	}
	
	/*
		Display methods
	*/
	@Override
	public int doStartTag()
		throws JspException
	{
		if (!isVisible()) return SKIP_BODY;
		
		Tag parent = getParent();
		if (parent instanceof HeaderRow)
		{
			HeaderRow hr = (HeaderRow)parent;
			hr.addHeaderColumns(getColspan());
		}

		StringBuilder sb = new StringBuilder();
		sb.append("<th");
		addIdAndStyles(sb);
		addDynamicAttributes(sb);
		if (getColspan() > 1) sb.append(" colspan=\"" + getColspan() + "\"");
		sb.append(">");
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_BODY_INCLUDE;
	}

	@Override
	public int doEndTag()
		throws JspException
	{
		if (!isVisible()) return EVAL_PAGE;
		StringBuilder sb = new StringBuilder();
		sb.append("</th>\n");
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_PAGE;
	}

}


