/*
	$Id: ArrayDesign.java 7381 2017-05-22 12:35:27Z nicklas $

	Copyright (C) 2005 Johan Enell, Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.ArrayDesignData;
import net.sf.basedb.core.data.ArrayDesignPlateData;
import net.sf.basedb.core.data.FeatureData;
import net.sf.basedb.core.data.ReporterData;
import net.sf.basedb.core.hibernate.TypeWrapper;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.PermissionRestriction;
import net.sf.basedb.core.query.Restrictions;

import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
	An array design is a description of the design of a microarray
	slide. An array design item is first created without any {@link Feature}
	information. A feature maps a coordinate on the array to a
	{@link Reporter}. Features can be added with links directly to
	reporters, or to {@link Well}s if the array design is associated
	with a set of {@link Plate}s. 
	<p>
	Once features has been added to the array design, they cannot be removed
	or modified.

	@see ArrayBatch
	@see ArraySlide
	@see Plate
	@see Feature
	@see ArrayDesignBlock
	@author Nicklas, Enell
	@version 2.0
*/
public class ArrayDesign 
	extends AnnotatedItem
	implements FileStoreEnabled, Registered
{
	
	/**
		The type of item represented by this class.

		@see Item#ARRAYDESIGN
		@see #getType()
	*/
	public static final Item TYPE = Item.ARRAYDESIGN;

	/**
		Create a new <code>ArrayDesign</code> item for a given platform.
	
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param platform The platform of the array design
		@return The new <code>ArrayDesign</code> item
		@throws BaseException If there is an error
		@since 2.5
	*/
	public static ArrayDesign getNew(DbControl dc, Platform platform)
	{
		ArrayDesign ad = dc.newItem(ArrayDesign.class);
		ad.setName("New array design");
		ad.setPlatform(platform);
		ad.setNumArrays(1);
		ad.getData().setFeatureIdentificationMethod(FeatureIdentificationMethod.NONE.getValue());
		ad.getData().setEntryDate(new Date());
		return ad;
	}

	/**
		Create a new <code>ArrayDesign</code> item for a given platform
		variant.
	
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param variant The platform variant of the array design
		@return The new <code>ArrayDesign</code> item
		@throws BaseException If there is an error
		@since 2.5
	*/
	public static ArrayDesign getNew(DbControl dc, PlatformVariant variant)
	{
		ArrayDesign ad = dc.newItem(ArrayDesign.class);
		ad.setName("New array design");
		ad.setVariant(variant);
		ad.setNumArrays(1);
		ad.getData().setFeatureIdentificationMethod(FeatureIdentificationMethod.NONE.getValue());
		ad.getData().setEntryDate(new Date());
		return ad;
	}
	
	/**
		Get an <code>ArrayDesign</code> item when you know the ID.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The ID of the item to load
		@return The <code>ArrayDesign</code> item
		@throws ItemNotFoundException If an item with the specified 
			ID is not found
		@throws PermissionDeniedException If the logged in user doesn't 
			have {@link Permission#READ} permission to the item
		@throws BaseException If there is another error
	*/
	public static ArrayDesign getById(DbControl dc, int id) 
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		ArrayDesign ad = dc.loadItem(ArrayDesign.class, id);
		if (ad == null) throw new ItemNotFoundException("ArrayDesign[id=" + id + "]");
		return ad;
	}

	/**
		Get a {@link ItemQuery} object configured
		to retrieve array designs.
		
		@return An {@link ItemQuery} object
	*/
	public static ItemQuery<ArrayDesign> getQuery() 
	{
		return new ItemQuery<ArrayDesign>(ArrayDesign.class);
	}

	private FeatureBatcher featureBatcher;
	private Map<BlockInfo, Integer> blocks = null;

	ArrayDesign(ArrayDesignData arrayDesignData)
	{
		super(arrayDesignData);
	}
	
	@Override
	ArrayDesignData getData()
	{
		return (ArrayDesignData)getBasicData();
	}

	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	@Override
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------
	/*
		From the Annotatable interface
		-------------------------------------------
	*/
	/**
		Get the plates, if any.
	*/
	@Override
	public Set<Annotatable> getAnnotatableParents()
		throws BaseException
	{
		Set<Annotatable> annotatable = new HashSet<Annotatable>();
		ItemQuery<Plate> plates = getPlates();
		plates.include(Include.ALL);
		annotatable.addAll(plates.list(getDbControl()));
		return annotatable;
	}
	// -------------------------------------------
	/*
		From the FileStoreEnabled interface
		-------------------------------------------
	*/
	/**
		@since 2.5
	*/
	@Override
	public FileSet getFileSet()
	{
		DbControl dc = getDbControl();
		FileSet fs = dc.getItem(FileSet.class, getData().getFileSet(), this);
		if (fs == null)
		{
			fs = FileSet.getNew(dc, this);
			getData().setFileSet(fs.getData());
			dc.saveItemIf(this, fs, false);
		}
		return fs;
	}
	/**
		@since 2.5
	*/
	@Override
	public boolean hasFileSet()
	{
		return getData().getFileSet() != null;
	}
	/**
		@since 2.5
	*/
	@Override
	public Platform getPlatform()
	{
		return getDbControl().getItem(Platform.class, getData().getPlatform());
	}
	/**
		@since 2.5
	*/
	@Override
	public PlatformVariant getVariant()
	{
		return getDbControl().getItem(PlatformVariant.class, getData().getVariant());
	}
	/**
		There are no parent file sets.
		@return Always null
		@since 2.5
	*/
	@Override
	public Collection<FileSet> getParentFileSets()
	{
		return null;
	}
	// -------------------------------------------
	/*
		From the Registered interface
		-------------------------------------------
	*/
	@Override
	public Date getEntryDate()
	{
		return DateUtil.copy(getData().getEntryDate());
	}
	@Override
	public void setEntryDate(Date entryDate)
	{
		if (isInDatabase()) 
		{
			throw new PermissionDeniedException("Not allowed to change entryDate of " + this);
		}
		getData().setEntryDate(DateUtil.setDateOrToday(entryDate));
	}
	// -------------------------------------------
	/*
		From the BasicItem class
		-------------------------------------------
	*/
	@Override
	public boolean isUsed() 
		throws BaseException
	{
		org.hibernate.Session session = getDbControl().getHibernateSession();
		org.hibernate.query.Query<Long> query = HibernateUtil.getPredefinedQuery(session, 
			"GET_ARRAYBATCHES_FOR_ARRAYDESIGN", Long.class, "count(*)");
		/*
			SELECT {1}
			FROM ArrayBatchData a
			WHERE a.arrayDesign = :arrayDesign
		*/
		query.setParameter("arrayDesign", getId(), TypeWrapper.H_INTEGER);
		long count = HibernateUtil.loadData(query);
		if (count == 0)
		{
			query = HibernateUtil.getPredefinedQuery(session, 
				"GET_RAWBIOASSAYS_FOR_ARRAYDESIGN", Long.class, "count(*)");
			/*
				SELECT {1}
				FROM RawBioAssayData rba
				WHERE rba.arrayDesign = :arrayDesign
			*/
			query.setParameter("arrayDesign", getId(), TypeWrapper.H_INTEGER);
			count = HibernateUtil.loadData(query);
		}
		return count > 0 || super.isUsed();
	}
	/**
	Get all:
		<ul>
		<li>{@link ArrayBatch}:s of this design
		<li>{@link RawBioAssay}:s of this design
		</ul>
		@since 2.2
	*/
	@Override
	public Set<ItemProxy> getUsingItems()
	{
		Set<ItemProxy> using = super.getUsingItems();
		org.hibernate.Session session = getDbControl().getHibernateSession();
		
		// Array batches
		org.hibernate.query.Query<Integer> query = HibernateUtil.getPredefinedQuery(session, 
			"GET_ARRAYBATCHES_FOR_ARRAYDESIGN", Integer.class, "a.id");
			/*
				SELECT {1}
				FROM ArrayBatchData a
				WHERE a.arrayDesign = :arrayDesign
			*/
		query.setParameter("arrayDesign", getId(), TypeWrapper.H_INTEGER);
		addUsingItems(using, Item.ARRAYBATCH, query);
		
		// Raw bioassays
		query = HibernateUtil.getPredefinedQuery(session, 
			"GET_RAWBIOASSAYS_FOR_ARRAYDESIGN", Integer.class, "rba.id");
			/*
				SELECT {1}
				FROM RawBioAssayData rba
				WHERE rba.arrayDesign = :arrayDesign
			*/
		query.setParameter("arrayDesign", getId(), TypeWrapper.H_INTEGER);
		addUsingItems(using, Item.RAWBIOASSAY, query);
		
		return using;
	}
	// -------------------------------------------
	
	/**
		Set the platform of the array design. This method will set the variant
		to null. Use {@link #setVariant(PlatformVariant)} if you want to
		set a specific variant.
		@param platform The new platform
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission
		@throws InvalidDataException If platform is null
		@since 2.5
	*/
	public void setPlatform(Platform platform)
	{
		checkPermission(Permission.WRITE);
		if (platform == null) throw new InvalidUseOfNullException("platform");
		getData().setPlatform(platform.getData());
		getData().setVariant(null);
	}
	
	/**
		Set the platform and variant of the array design. This method will 
		automatically set the platform to {@link PlatformVariant#getPlatform()}.

		@param variant The new platform variant, or null to remove
			the variant and keep the platform as it is
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission
		@since 2.5
	*/
	public void setVariant(PlatformVariant variant)
	{
		checkPermission(Permission.WRITE);
		if (variant != null)
		{
			getData().setVariant(variant.getData());
			getData().setPlatform(variant.getData().getPlatform());
		}
		else
		{
			getData().setVariant(null);
		}
	}
	
	/**
		Check if the platform/variant of this array design has the
		given external ID. 
		@param externalId The external ID to match
		@return TRUE if either the variant of platform matches the
			external ID
		@since 2.5
	*/
	public boolean isPlatform(String externalId)
	{
		if (externalId == null) return false;
		if (externalId.equals(getData().getPlatform().getExternalId())) return true;
		if (getData().getVariant() != null)
		{
			if (externalId.equals(getData().getVariant().getExternalId())) return true;
		}
		return false;
	}
	
	/**
		Check if this array design uses a file-only platform or variant.
		If the array design has a platform variant, the setting from the 
		variant is returned, otherwise the setting from the platform.
		@return TRUE if this array design use a file-only variant / platform
		@since 2.5
	*/
	public boolean isFileOnlyPlatform()
	{
		if (getData().getVariant() != null)
		{
			return getData().getVariant().isFileOnly();
		}
		else
		{
			return getData().getPlatform().isFileOnly();
		}
	}

	/**
		Get the job that imported features to this array design.
		@return A <code>Job</code> object, ot null if this information is
			not available
		@throws PermissionDeniedException If the logged in user doesn't have 
			read permission to the job
		@throws BaseException If there is another error
		@since 2.7
	*/
	public Job getJob()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(Job.class, getData().getJob());
	}
	
	/**
		Get the number of arrays on a single slide. The default
		value is 1.
		@since 2.6
	*/
	public int getNumArrays()
	{
		return getData().getNumArrays();
	}
	
	/**
		Set the number of arrays on a single slide.
		@param numArrays A value &gt; 0
		@throws NumberOutOfRangeException If the value is &lt;=0
		@since 2.6
	 */
	public void setNumArrays(int numArrays)
	{
		checkPermission(Permission.WRITE);
		if (numArrays <= 0) throw new NumberOutOfRangeException("numArrays", numArrays, 0, false);
		getData().setNumArrays(numArrays);
	}	
	
	/**
		Check if this array design has information about features, either
		in the database or in files.
	  	@return TRUE if this object has features, FALSE otherwise
	*/
	public boolean hasFeatures()
	{
		return getData().getHasFeatures();
	}

	/**
		Get the method used to identify features on this array design.
		@return A <code>FeatureIdentificationMethod</code>
		@since 2.6
	*/
	public FeatureIdentificationMethod getFeatureIdentificationMethod()
	{
		return FeatureIdentificationMethod.fromValue(getData().getFeatureIdentificationMethod());
	}
	
	/**
		Get a query that returns the plates that are used by this arraydesign.
		@return An {@link ItemQuery} object
		@throws BaseException If something fails when getting the query.
	*/
	public ItemQuery<Plate> getPlates() 
		throws BaseException
	{
		ItemQuery<Plate> query = Plate.getQuery();
		query.joinPermanent(Hql.innerJoin("arrayDesigns", Item.ARRAYDESIGNPLATE.getAlias()));
		query.restrictPermanent(
			Restrictions.eq(
				Hql.property(Item.ARRAYDESIGNPLATE.getAlias(), "arrayDesign"), 
				Hql.entity(this)
			)
		);
		return query;
	}
	
	/**
		Get plate connections for this array design. This method has better performance
		than using the {@link #getPlates()} and {@link #getPlatePosition(Plate)} and/or
		{@link #getPlate(int)} methods since it only requires a single query to the database.
		@return An <code>ItemQuery</code> object
	*/
	public ItemQuery<ArrayDesignPlate> getArrayDesignPlates()
	{
		ItemQuery<ArrayDesignPlate> query = ArrayDesignPlate.getQuery(this);;
		return query;
	}
	
	/**
		Get the position of a {@link Plate} in this array design.
		@param plate The plate.
		@return The position of the plate. -1 if the array design doesn't contain this plate. 
	*/
	public int getPlatePosition(Plate plate)
	{
		for (ArrayDesignPlateData adp : getData().getPlates())
		{
			if (adp.getPlate().getId() == plate.getId())
			{
				return adp.getPosition();
			}
		}
		return -1;
	}
	
	/**
		Get a {@link Plate} from a given position.
		@param position The position of the plate, starting at 0
		@return The plate or null if the position couldn't be found
	*/
	public Plate getPlate(int position)
	{
		if (position < 0 || position > getData().getPlates().size())
		{
			return null;
		}
		for (ArrayDesignPlateData adp : getData().getPlates())
		{
			if (adp.getPosition() == position)
			{
				return getDbControl().getItem(Plate.class, adp.getPlate());
			}
		}
		return null;
	}
	
	/**
		The number of plates connected to this array design.
		@return The number of plates.
	*/
	public int getNumPlates()
	{
		return getData().getPlates().size();
	}
	
	/**
		Set the list of plates to be used by this array design. If the logged in user lacks
		{@link Permission#USE} to any plate in the list no plates will be added.
		
		@param plates A list of plates to replace the old list with. It may not contain nulls, but
			if the list is null or empty all plates will be removed.
		@throws PermissionDeniedException If the logged in user doesn't 
			have {@link Permission#WRITE} permission to this ArrayDesign item or 
			{@link Permission#USE} permission to any plate item in the list or tries
			to add a plate to an arraydesign that has features. 
	*/
	public void setPlateList(List<Plate> plates)
		throws PermissionDeniedException
	{
		if (getNumDbFeatures() > 0)
		{
			throw new PermissionDeniedException("The array design already has features: "+this);
		}
		checkPermission(Permission.WRITE);
		Set<ArrayDesignPlateData> newPlates = getData().getPlates();
		newPlates.clear();
		int index = 0;
		if (plates != null)
		{
			for (Plate plate : plates)
			{
				if (plate == null) throw new InvalidUseOfNullException("plates["+index+"]");
				plate.checkPermission(Permission.USE);
				ArrayDesignPlateData adpd = new ArrayDesignPlateData();
				adpd.setArrayDesign(getData());
				adpd.setPlate(plate.getData());
				adpd.setPosition(index);
				newPlates.add(adpd);
				index++;
			}
		}
	}

	/**
		Adds a single plate to the plate list.
		
		@param plate the {@link Plate} item to be added to the list. It may be null.
		@throws PermissionDeniedException If the logged in user doesn't 
			have {@link Permission#WRITE} permission to this ArrayDesign item
			, {@link Permission#USE} permission to the plate item or tries
			to add a plate to an arraydesign that has features. 
	*/
	public void addPlate(Plate plate)
		throws PermissionDeniedException
	{
		if (getNumDbFeatures() > 0)
		{
			throw new PermissionDeniedException("The array design already has features: "+this);
		}
		checkPermission(Permission.WRITE);		
		ArrayDesignPlateData adpd = new ArrayDesignPlateData();
		adpd.setArrayDesign(getData());
		if (plate != null) 
		{
			plate.checkPermission(Permission.USE);
			adpd.setPlate(plate.getData());
		}		
		adpd.setPosition(getData().getPlates().size());
		getData().getPlates().add(adpd);
	}

	/**
		Create a new {@link ArrayBatch} with this design.
		
		@return The new <code>ArrayBatch</code> item
		@throws PermissionDeniedException If
			the logged in user doesn't have use
			permission for this design
		@throws BaseException If there is another error
	*/
	public ArrayBatch newArrayBatch() 
		throws PermissionDeniedException, BaseException
	{
		return ArrayBatch.getNew(getDbControl(), this);
	}
	
	/**
		Get a query that returns the batches for this design.
		@return An <code>ItemQuery</code> object
	*/
	public ItemQuery<ArrayBatch> getArrayBatches()
	{
		ItemQuery<ArrayBatch> query = ArrayBatch.getQuery();
		query.restrictPermanent(
			Restrictions.eq(
				Hql.property("arrayDesign"), 
				Hql.entity(this)
			)
		);
		return query;
	}
	
	/**
		Adds an {@link ArrayDesignBlock} to the array design. If the block
		already exists it is returned instead. Unlike other items, 
		the block is automatically saved to the database. The reason is that
		otherwise we wouldn't be able to add features to the block since
		the {@link FeatureBatcher} object requires that the block has an id.
		<p>
		Use the {@link #getFeatureBatcher(FeatureIdentificationMethod, Job)} 
		method to get an batcher object that can add features to the new block.
		<p>
		It is not possible to add blocks to an array design which uses a
		file-only platform. The must store feature information in files 
		instead. See {@link Platform} and {@link DataFileType}.

		@param bi The information for the new block
		@return The new <code>ArrayDesignBlock</code>
		@throws PermissionDeniedException if the logged in user doesn't have
			write permission on the array design
		@throws BaseException If there is another error
	*/
	public ArrayDesignBlock addArrayDesignBlock(BlockInfo bi)
		throws PermissionDeniedException, BaseException
	{
		if (getData().getPlatform().isFileOnly()) 
		{
			throw new PermissionDeniedException("Can't add blocks to a file-only array design: " + this.getName());
		}
		if (getNumDbFeatures() > 0) 
		{
			throw new PermissionDeniedException("Can't add blocks to a design that already has features: " + this.getName());
		}
		checkPermission(Permission.WRITE);
		if (blocks == null) blocks = new HashMap<BlockInfo, Integer>();
		ArrayDesignBlock adb;
		Integer adbId = blocks.get(bi);
		if(adbId != null)
		{
			adb = ArrayDesignBlock.getById(getDbControl(), adbId);
		}
		else
		{
			adb = ArrayDesignBlock.getNew(getDbControl(), this, bi);
			blocks.put(bi, adb.getId());
		}
		return adb;
	}
	
	/**
		Get a feature batcher which can be used to add regular features to the
		array design. 	
		<p>
		It is not possible to add blocks to an array design which uses a
		file-only platform. The must store feature information in files 
		instead. See {@link Platform} and {@link DataFileType}.
		@param fiMethod The method to use for identifying features
		@param job  The job that is currently running, or null if the feature import
			is not running as a job
		@return The feature batcher of the array design. If this method is called multiple
			times the <code>fiMethod</code> and <code>job</code> parameter is ignored 
			and the same batcher as before is returned.
		@throws PermissionDeniedException If raw data has already been added
			or the logged in user doesn't have write permission
		@throws BaseException If there is another error
		@since 2.7
	*/
	public FeatureBatcher getFeatureBatcher(FeatureIdentificationMethod fiMethod, Job job)
		throws PermissionDeniedException, BaseException
	{
		checkPermission(Permission.WRITE);
		if (isFileOnlyPlatform()) 
		{
			throw new PermissionDeniedException("Can't add features to a file-only array design: " + this.getName());
		}
		if (getNumDbFeatures() > 0) 
		{
			throw new PermissionDeniedException("Features has already been added to "+this.getName());
		}
		if (fiMethod == FeatureIdentificationMethod.NONE)
		{
			throw new InvalidDataException("Feature identification method NONE is not supported: " + this.getName());
		}
		getData().setFeatureIdentificationMethod(fiMethod.getValue());
		if (featureBatcher == null)
		{
			featureBatcher = FeatureBatcher.getNew(getDbControl(), this);
			getData().setJob(job == null ? null : job.getData());
		}
		return featureBatcher;
	}

	/**
		Get a query that returns the regular features of this array design.
		@return A {@link DataQuery} object
	*/
	public DataQuery<FeatureData> getFeatures()
	{
		DataQuery<FeatureData> query = new DataQuery<FeatureData>(FeatureData.class, null);
		query.restrictPermanent(
			Restrictions.eq(
				Hql.property("arrayDesignBlock.arrayDesign"),
				Hql.entity(this)
			)
		);
		return query;
	}

	/**
		Get a query that returns the reporters on this array design. The query
		is rooted at the features, which means that:
		<ul>
		<li>All filters, sorting, etc. that works for {@link #getFeatures()} can be 
			used without modifications on this query.
		<li>The same reporter may be returned multiple times if it is present on several
			features.
		<li>'Null' values may be returned if there are features without a reporter
		</ul>
		
		@return A {@link DataQuery} object
		@since 2.8
	*/
	public DataQuery<ReporterData> getReporters()
	{
		DataQuery<ReporterData> query = 
			new DataQuery<ReporterData>(ReporterData.class, Item.FEATURE, null, "reporter");
		// We allow reporters without READ permission unless it has been explicitely DENIED
		query.restrictPermanent(new PermissionRestriction(Permission.DENIED, Item.REPORTER));
		query.restrictPermanent(
				Restrictions.eq(
					Hql.property("arrayDesignBlock.arrayDesign"),
					Hql.entity(this)
				)
			);
		return query;
	}
	
	/**
		Get a feature when you know the id. NOTE! This method may return an uninitialized
		proxy. If an initialized object is required use {@link #getFeatureById(int, boolean)}
		instead.
		
		@param id The id of the feature
	 	@throws ItemNotFoundException If the feature could not be found.
	 	@throws BaseException If something else fails when getting the feature.
	*/
	public FeatureData getFeatureById(int id)
		throws ItemNotFoundException, BaseException
	{
		return getFeatureById(id, false);
	}
	
	/**
		Get a feature when you know the id.
		@param id The id of the feature
		@param forceInitialization If TRUE the feature and associated reporter
			are sure to be initialized
	 	@throws ItemNotFoundException If the feature could not be found.
	 	@throws BaseException If something else fails when getting the feature
	 	@since 2.15
	*/
	public FeatureData getFeatureById(int id, boolean forceInitialization)
	{
		org.hibernate.Session session = getDbControl().getHibernateSession();
		FeatureData feature = HibernateUtil.loadData(session, FeatureData.class, id);
		if (feature == null) throw new ItemNotFoundException("FeatureData[id = " + id + "]");
		if (forceInitialization)
		{
			org.hibernate.Hibernate.initialize(feature);
			org.hibernate.Hibernate.initialize(feature.getReporter());
		}
		HibernateUtil.evictData(session, feature);
		return feature;
	}
	
	/**
		Get a query that returns the block of this array design.
		@return An {@link ItemQuery} object
		@see ArrayDesignBlock#getQuery(ArrayDesign)
	*/
	public ItemQuery<ArrayDesignBlock> getArrayDesignBlocks()
	{
		return ArrayDesignBlock.getQuery(this);
	}
	
	/**
		Get the number of features on this array design that are
		stored in the database.
		@return The number of features or probesets
		@since 2.5
	*/
	public int getNumDbFeatures()
	{
		return getData().getNumDbFeatures();
	}

	/**
		Get the number of features on this array design that are
		stored inside files.
		@return The number of features or probesets
		@since 2.5
	*/
	public int getNumFileFeatures()
	{
		return getData().getNumFileFeatures();
	}
	
	/**
		Set the number of features that are stored in files.
		@param numFeatures The number of features
		@since 2.5
	*/
	public void setNumFileFeatures(int numFeatures)
	{
		checkPermission(Permission.WRITE);
		getData().setNumFileFeatures(numFeatures);
		getData().setHasFeatures(numFeatures > 0 || getNumDbFeatures() > 0);
	}

}
