/*
  $Id: Hardware.java 7381 2017-05-22 12:35:27Z nicklas $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Date;
import java.util.Set;

import net.sf.basedb.core.data.BasicData;
import net.sf.basedb.core.data.HardwareData;
import net.sf.basedb.core.hibernate.TypeWrapper;

/**
	This class is used to represent individual hardware items
	and information about them. In the current version of BASE 
	the only type of hardware we keep information about is scanners.
	
	@author enell
	@version 2.0
	@base.modified $Date: 2017-05-22 14:35:27 +0200 (mån, 22 maj 2017) $
*/
public class Hardware
	extends AnnotatedItem
	implements Registered, Subtypable
{
	/**
		The type of item represented by this class.
		@see Item#HARDWARE
		@see #getType()
	*/
	public static final Item TYPE = Item.HARDWARE;

	/**
		The id for the <code>HardwareType</code> object representing a
		scanner. 
	 	@since 3.0
	*/
	public static final String SCANNER = "net.sf.basedb.core.HardwareType.SCANNER";
	
	/**
		The id for the <code>HardwareType</code> object representing a
		print robot.
	 	@since 3.0
	*/
	public static final String PRINT_ROBOT = "net.sf.basedb.core.HardwareType.PRINT_ROBOT";
	
	/**
	 	The id for the <code>HardwareType</code> object representing a 
	 	hybridization station.
	 	@since 3.0
	*/
	public static final String HYBRIDIZATION_STATION = "net.sf.basedb.core.HardwareType.HYBRIDIZATION_STATION";

	/**
	 	The id for the <code>HardwareType</code> object representing a 
	 	cluster generator station. Used when creating a {@link PhysicalBioAssay}.
	 	@since 3.0
	*/
	public static final String CLUSTER_GENERATOR = "net.sf.basedb.core.HardwareType.CLUSTER_GENERATOR";

	/**
	 	The id for the <code>HardwareType</code> object representing a 
	 	sequencer.
	 	@since 3.0
	*/
	public static final String SEQUENCER = "net.sf.basedb.core.HardwareType.SEQUENCER";

	/**
	 	The id for the <code>HardwareType</code> object representing a 
	 	freezer.
	 	@since 3.0
	*/
	public static final String FREEZER = "net.sf.basedb.core.HardwareType.FREEZER";

	/**
	 	The id for the <code>HardwareType</code> object representing a 
	 	cabinet.
	 	@since 3.2
	*/
	public static final String CABINET = "net.sf.basedb.core.HardwareType.CABINET";

	
	/**
	 	The id for the <code>HardwareType</code> object representing something
	 	else.
	 	@since 3.0
	*/
	public static final String OTHER = "net.sf.basedb.core.HardwareType.OTHER";

	/**
		The maximum length of the version variable that can be stored in the
		database. Check the length against this value before calling the
		{@link #setVersionString(String)} method to avoid
		exceptions.
	*/
	public static final int MAX_VERSIONSTRING_LENGTH = HardwareData.MAX_VERSIONSTRING_LENGTH;

	/**
		Get the system id for the hardware subtype that should be used on an item
		that doesn't have a subtype. This includes {@link Subtypable} items with a
		null {@link ItemSubtype} and all other non-subtypable items which can be
		linked to a hardware.
		@param itemType The item type
		@return The system id of the default hardware type or null
		@since 3.0
	*/
	public static String getDefaultSystemId(Item itemType)
	{
		String defaultId = null;
		if (itemType == Item.ARRAYBATCH)
		{
			defaultId = PRINT_ROBOT;
		}
		return defaultId;
	}

	
	/**
		Create a new <code>Hardware</code> item.

		@param dc The {@link DbControl} which will be used for
			permission checking and database access.
		@return The new <code>Hardware</code> item
		@throws BaseException If there is an error
	*/
	public static Hardware getNew(DbControl dc)
		throws BaseException
	{
		Hardware hardware = dc.newItem(Hardware.class);
		hardware.setName("New hardware");
		hardware.getData().setEntryDate(new Date());
		return hardware;
	}

	/**
		Get a <code>Hardware</code> item when you know the ID.

		@param dc The {@link DbControl} which will be used for
			permission checking and database access.
		@param id The ID of the item to load
		@return The <code>Hardware</code> item
		@throws ItemNotFoundException If an item with the specified ID is not found
		@throws PermissionDeniedException If the logged in user doesn't have 
			{@link Permission#READ } permission to the item
		@throws BaseException If there is another error
	*/
	public static Hardware getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		Hardware hw = dc.loadItem(Hardware.class, id);
		if (hw == null) throw new ItemNotFoundException("Hardware[ID="+id+"]");
		return hw;
	}
	
	/**
		Get a query configured to retrieve hardware.
		
		@return An {@link ItemQuery} object
	 	@throws BaseException When the query could not be configured. 
	*/
	public static ItemQuery<Hardware> getQuery()
		throws BaseException
	{
		return new ItemQuery<Hardware>(Hardware.class);
	}

	
	/**
		Creates a new hardware item.
		
		@param data the data
	*/
	Hardware(HardwareData data)
	{
		super(data);
	}
	
	@Override
	HardwareData getData()
	{
		return (HardwareData)getBasicData();
	}
	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	@Override
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------
	/*
		From the Registered interface
		-------------------------------------------
	*/
	@Override
	public Date getEntryDate()
	{
		return DateUtil.copy(getData().getEntryDate());
	}
	@Override
	public void setEntryDate(Date entryDate)
	{
		if (isInDatabase()) 
		{
			throw new PermissionDeniedException("Not allowed to change entryDate of " + this);
		}
		getData().setEntryDate(DateUtil.setDateOrToday(entryDate));
	}
	// -------------------------------------------
	/*
		From the Annotatable interface
		-------------------------------------------
	*/
	/**
		@return Always null
		@since 3.3
	*/
	@Override
	public Set<Annotatable> getAnnotatableParents()
	{
		return null;
	}
	// -------------------------------------------
	/*
		From the Subtypable interface
		-----------------------------
	*/
	@Override
	public ItemSubtype getItemSubtype()
	{
		return getDbControl().getItem(ItemSubtype.class, getData().getItemSubtype());
	}
	@Override
	public void setItemSubtype(ItemSubtype subtype)
	{
		checkPermission(Permission.WRITE);
		if (subtype != null)
		{
			subtype.setOnItem(this);
		}
		else
		{
			getData().setItemSubtype(null);
		}
	}
	// -------------------------------------------

	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		Check if:
		<ul>
		<li>a {@link DerivedBioAssay} is linked to this hardware
		<li>an {@link ArrayBatch} is linked to this hardware
		<li>a {@link BioMaterialEvent} is linked to this hardware
		<li>a {@link PlateEvent} is linked to this hardware
		</ul>
		
		@return TRUE if this hardware is used.
		@throws BaseException if there is any error.
	*/
	@Override
	public boolean isUsed()
		throws BaseException
	{
		org.hibernate.Session session = getDbControl().getHibernateSession();
		boolean used = false;
		if (!used)
		{
			org.hibernate.query.Query<Long> q = HibernateUtil.getPredefinedQuery(session, 
				"GET_DERIVEDBIOASSAYS_FOR_HARDWARE", Long.class, "count(*)");
			/*
				SELECT {1}
				FROM DerivedBioAssayData dba
				WHERE dba.hardware = :hardware
			*/
			q.setParameter("hardware", getId(), TypeWrapper.H_INTEGER);
			used = HibernateUtil.loadData(q) > 0;
		}
		if (!used)
		{
			org.hibernate.query.Query<Long> q = HibernateUtil.getPredefinedQuery(session, 
				"GET_ARRAYBATCHES_FOR_PRINTROBOT", Long.class, "count(*)");
			/*
				SELECT {1}
				FROM ArrayBatchData ab
				WHERE ab.printRobot = :printrobot
			*/
			q.setParameter("printrobot", getId(), TypeWrapper.H_INTEGER);
			used = HibernateUtil.loadData(q) > 0;
		}
		if (!used)
		{
			org.hibernate.query.Query<Long> q = HibernateUtil.getPredefinedQuery(session, 
				"GET_BIOMATERIALEVENTS_FOR_HARDWARE", Long.class, "count(*)");
				/*
					SELECT {1}
					FROM BioMaterialEventData bme
					WHERE bme.hardware = :hardware
				*/
			q.setParameter("hardware", getId(), TypeWrapper.H_INTEGER);
			used = HibernateUtil.loadData(q) > 0;
		}
		if (!used)
		{
			org.hibernate.query.Query<Long> q = HibernateUtil.getPredefinedQuery(session, 
				"GET_PLATEEVENTS_FOR_HARDWARE", Long.class, "count(*)");
				/*
					SELECT {1}
					FROM PlateEventData ple
					WHERE ple.hardware = :hardware
				*/
			q.setParameter("hardware", getId(), TypeWrapper.H_INTEGER);
			used = HibernateUtil.loadData(q) > 0;
		}
		return used || super.isUsed();
	}
	/**
		Get all:
		<ul>
		<li>{@link DerivedBioAssay}:s with this hardware
		<li>{@link ArrayBatch}:s with this hardware
		<li>{@link BioMaterial}:s and {@link PhysicalBioAssay}:s with this hardware
		<li>{@link Plate}:s with this hardware
		</ul>
		@since 2.2
	*/
	@Override
	public Set<ItemProxy> getUsingItems()
	{
		Set<ItemProxy> using = super.getUsingItems();
		org.hibernate.Session session = getDbControl().getHibernateSession();
		
		// Derived bioassay sets
		org.hibernate.query.Query<Integer> query = HibernateUtil.getPredefinedQuery(session, 
			"GET_DERIVEDBIOASSAYS_FOR_HARDWARE", Integer.class, "dba.id");
			/*
				SELECT {1}
				FROM DerivedBioAssayData dba
				WHERE dba.hardware = :hardware
			*/
		query.setParameter("hardware", getId(), TypeWrapper.H_INTEGER);
		addUsingItems(using, Item.DERIVEDBIOASSAY, query);

		// Array batches
		query = HibernateUtil.getPredefinedQuery(session, 
			"GET_ARRAYBATCHES_FOR_PRINTROBOT", Integer.class, "ab.id");
			/*
				SELECT {1}
				FROM ArrayBatchData ab
				WHERE ab.printRobot = :printrobot
			*/
		query.setParameter("printrobot", getId(), TypeWrapper.H_INTEGER);
		addUsingItems(using, Item.ARRAYBATCH, query);
		
		// BioMaterial events - biomaterial
		org.hibernate.query.Query<BasicData> query2 = HibernateUtil.getPredefinedQuery(session, 
			"GET_BIOMATERIALEVENTS_FOR_HARDWARE", BasicData.class, "bme.bioMaterial");
			/*
				SELECT {1}
				FROM BioMaterialEventData bme
				WHERE bme.hardware = :hardware
			*/
		query2.setParameter("hardware", getId(), TypeWrapper.H_INTEGER);
		addUsingItems(using, query2);
		
		// BioMaterial events - hybridizations
		query2 = HibernateUtil.getPredefinedQuery(session, 
			"GET_BIOMATERIALEVENTS_FOR_HARDWARE", BasicData.class, "bme.physicalBioAssay");
			/*
				SELECT {1}
				FROM BioMaterialEventData bme
				WHERE bme.hardware = :hardware
			*/
		query2.setParameter("hardware", getId(), TypeWrapper.H_INTEGER);
		addUsingItems(using, query2);

		// Plates
		query = HibernateUtil.getPredefinedQuery(session, 
			"GET_PLATEEVENTS_FOR_HARDWARE", Integer.class, "ple.plate.id");
			/*
				SELECT {1}
				FROM PlateEventData ple
				WHERE ple.hardware = :hardware
			*/
		query.setParameter("hardware", getId(), TypeWrapper.H_INTEGER);
		addUsingItems(using, Item.PLATE, query);
		return using;
	}
	// -------------------------------------------
	
	/**
		Get the versionstring of this <code>Hardware</code> item.
		@return A string with the version of this item, or null if not known
	*/
	public String getVersionString()
	{
		return getData().getVersionString();
	}

	/**
		Set the versionstring for this <code>Hardware</code> item. The value
		must not be longer than the value specified by the {@link #MAX_VERSIONSTRING_LENGTH} constant.
		
		@param versionString The new version for this item, or null if not known
		@throws StringTooLongException If versionString is longer then {@link #MAX_VERSIONSTRING_LENGTH}
		@throws PermissionDeniedException If the logged in user don't have write permission on this item
	*/
	public void setVersionString(String versionString) 
		throws StringTooLongException, PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		getData().setVersionString(StringUtil.setNullableString(versionString, "versionString", MAX_VERSIONSTRING_LENGTH));
	}
}
