/*
	$Id: InternalJobQueue.java 7513 2018-11-02 07:45:02Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.plugin.Response;
import net.sf.basedb.core.signal.LocalSignalReceiver;
import net.sf.basedb.core.signal.SignalReceiver;
import net.sf.basedb.util.ClassUtil;
import net.sf.basedb.core.data.JobData;
import net.sf.basedb.core.data.PluginDefinitionData;
import net.sf.basedb.core.data.ShareableData;
import net.sf.basedb.core.data.UserData;
import net.sf.basedb.core.hibernate.TypeWrapper;
import net.sf.basedb.core.data.ItemKeyData;
import net.sf.basedb.core.data.ProjectKeyData;

import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.HashSet;
import java.util.Map;
import java.util.HashMap;
import java.util.Collections;
import java.util.TimerTask;

/**
	A simple implementation of a job queue that executes jobs 
	in order of priority and longest waiting time. To allow quick jobs
	like importing raw data or reporters to run it is possible to
	reserve slots per {@link Job.ExecutionTime} setting. The configuration
	is done in the <code>base.config</code> file with the following settings:
	
	<table>
	<caption>Internal job queue settings</caption>
	<tr>
		<th>Setting</th>
		<th>Default/possible value(s)</th>
		<th>Description</th>
	</tr>
	<tr>
		<td>jobqueue.internal.enabled</td>
		<td><b>true</b> | false</td>
		<td>
			If the internal job queue is enabled or not. Do not
			disable the job queue if you don't have another external
			job queue manager. It is possible to use both an external
			and internal job queue.
		</td>
	</tr>
	<tr>
		<td>jobqueue.internal.checkinterval</td>
		<td>30</td>
		<td>
			The interval, in seconds, that the job queue checks
			the database for new jobs.
		</td>
	</tr>
	<tr>
		<td>jobqueue.internal.maxthreadpriority</td>
		<td><code>Thread.NORM_PRIORITY - 1</code></td>
		<td>
			The maximum thread priority for the threads that execute jobs.
			See {@link Thread#setPriority(int)} for possible values.
		</td>
	</tr>
	<tr>
		<td>jobqueue.internal.shortest.threads</td>
		<td>1</td>
		<td>
			Number of threads to reserve for jobs with an estimated
			execution time of less than one minute. Jobs with this
			setting may also use threads from the other "slower" slots.
		</td>
	</tr>
	<tr>
		<td>jobqueue.internal.shortest.threadpriority</td>
		<td><code>Thread.NORM_PRIORITY - 1</code></td>
		<td>
			The thread priority to use when executing jobs in this slot.
			See {@link Thread#setPriority(int)} for possible values.
		</td>
	</tr>
	<tr>
		<td>jobqueue.internal.short.threads</td>
		<td>1</td>
		<td>
			Number of threads to reserve for jobs with an estimated
			execution time between one and ten minutes. Jobs with this
			setting may also use threads from the other "slower" slots.
		</td>
	</tr>
	<tr>
		<td>jobqueue.internal.short.threadpriority</td>
		<td><code>Thread.NORM_PRIORITY - 1</code></td>
		<td>
			The thread priority to use when executing jobs in this slot.
			See {@link Thread#setPriority(int)} for possible values.
		</td>
	</tr>
	<tr>
		<td>jobqueue.internal.medium.threads</td>
		<td>2</td>
		<td>
			Number of threads to reserve for jobs with an estimated
			execution time less than one hour. Jobs with this
			setting may also use threads from the other "slower" slots.
		</td>
	</tr>
	<tr>
		<td>jobqueue.internal.medium.threadpriority</td>
		<td><code>Thread.NORM_PRIORITY - 2</code></td>
		<td>
			The thread priority to use when executing jobs in this slot.
			See {@link Thread#setPriority(int)} for possible values.
		</td>
	</tr>
	<tr>
		<td>jobqueue.internal.long.threads</td>
		<td>4</td>
		<td>
			Number of threads to reserve for jobs with an estimated
			execution time more than one hour. Jobs with this
			setting may not use threads from the other slots.
		</td>
	</tr>
	<tr>
		<td>jobqueue.internal.long.threadpriority</td>
		<td><code>Thread.NORM_PRIORITY - 2</code></td>
		<td>
			The thread priority to use when executing jobs in this slot.
			See {@link Thread#setPriority(int)} for possible values.
		</td>
	</tr>
	</table>
	
	@author Nicklas
	@version 2.0
	@base.modified $Date: 2018-11-02 08:45:02 +0100 (fre, 02 nov. 2018) $
*/
public final class InternalJobQueue
	extends TimerTask
{

	/**
		Log job queue events.
	*/
	private static final org.slf4j.Logger log = 
		org.slf4j.LoggerFactory.getLogger(InternalJobQueue.class);

	/**
		The job queues session controller. We are logging in using our
		own keyring: {@link JobQueueKeyring}.
	*/
	private final SessionControl sc;
	
	/**
		The current number of threads executing in each slot.
	*/
	private final Map<Job.ExecutionTime, Integer> usedSlots;

	/**
		The maximum number of threads that are allowed in each slot.
	*/
	private final Map<Job.ExecutionTime, Integer> maxSlots;
	
	/**
		The thread priority to use when executing jobs in each slot.
	*/
	private final Map<Job.ExecutionTime, Integer> priorities;

	/**
		The group were all job runners are placed.
	*/
	private final ThreadGroup runnersGroup;
	
	/**
		Holds all currently executing jobs.
	*/
	private final Set<JobRunner> runners;
	
	/**
		If the {@link PluginDefinitionData#getUseInternalJobQueue()} setting
		should be ignored or not.
	*/
	private boolean ignoreUseInternalJobQueueFlag = false;

	/**
		Timeout to wait for jobs to act on the ABORT signal when shutting down BASE.
		Default value is 20 seconds.
	*/
	private int closeTimeout = 20000;
	
	/**
		Time when the next status update request for external jobs
		is scheduled.
	*/
	private long nextStatusUpdateRequest;
	
	/**
		Use the local signal receiver only.
	*/
	private static SignalReceiver signalReceiver;

	/**
		Get the signal receiver used by the internal job queue. If the 
		internal job queue is disabled a {@link LocalSignalReceiver}
		is created on the first call to this method.
		@return A signal receiver object
		@since 2.7
	*/
	public synchronized static SignalReceiver getSignalReceiver()
	{
		if (signalReceiver == null)
		{
			signalReceiver = new LocalSignalReceiver();
			signalReceiver.init("localhost:0");
		}
		return signalReceiver;
	}
	
	/**
		Create the job queue. The one and only instance is created
		by Application.start
	*/
	InternalJobQueue()
	{
		usedSlots = Collections.synchronizedMap(new HashMap<Job.ExecutionTime, Integer>());
		maxSlots = Collections.synchronizedMap(new HashMap<Job.ExecutionTime, Integer>());
		priorities = Collections.synchronizedMap(new HashMap<Job.ExecutionTime, Integer>());

		// If we should execute all jobs or only jobs for plug-ins which may use the internal queue
		ignoreUseInternalJobQueueFlag = Config.getBoolean("jobqueue.internal.runallplugins");

		// Number of execution slots and thread priorities
		for (Job.ExecutionTime et : Job.ExecutionTime.values())
		{
			usedSlots.put(et, 0);
			String configName = "jobqueue.internal."+et.name().toLowerCase();
			int configuredSlots = Config.getInt(configName+".threads", et.getDefaultSlots());
			maxSlots.put(et, configuredSlots);
			int priority = Config.getInt(configName+".threadpriority", et.getDefaultPriority());
			priorities.put(et, priority);
		}
		runners = Collections.synchronizedSet(new HashSet<JobRunner>());
		
		// Signal receiver
		String signalReceiverClass = Config.getString("jobqueue.internal.signalreceiver.class");
		String signalReceiverInit = Config.getString("jobqueue.internal.signalreceiver.init");
		if (signalReceiverClass == null)
		{
			signalReceiverClass = "net.sf.basedb.core.signal.LocalSignalReceiver";
			if (signalReceiverInit == null) signalReceiverInit = "localhost:0";
		}
		try
		{
			synchronized (InternalJobQueue.class)
			{
				if (signalReceiver == null)
				{
					signalReceiver = (SignalReceiver)ClassUtil.newInstance(Class.forName(signalReceiverClass));
					signalReceiver.init(signalReceiverInit);
				}
			}
		}
		catch (ClassNotFoundException ex)
		{
			throw new BaseException(ex);
		}
		
		// Configure thread group
		runnersGroup = new ThreadGroup("InternalJobQueue");
		runnersGroup.setDaemon(false);
		runnersGroup.setMaxPriority(Config.getInt("jobqueue.internal.maxthreadpriority", Thread.NORM_PRIORITY - 1));
		sc = new SessionControl(null, "net.sf.basedb.clients.internal", false, Application.getHostName());
		sc.login(new JobQueueKeyring());
	}

	/*
		From the TimerTask class
		-------------------------------------------
	*/
	/**
		Check the database for jobs that are waiting to be executed.
		We sort the jobs by priority and creation time and try to execute 
		them in that order. Each job is encapsulated in a {@link JobRunner}
		object which is started in it's own thread.
	*/
	@Override
	public void run()
	{
		checkInternalJobQueue();
		requestExternalStatusUpdate();
	}
	// -------------------------------------------

	private void checkInternalJobQueue()
	{
		log.info("Checking for jobs to execute");
		DbControl dc = null;
		try
		{
			dc = sc.newDbControl();
			String hqlName = ignoreUseInternalJobQueueFlag ? 
				"FIND_JOBS_IN_QUEUE" : "FIND_INTERNAL_JOBS_IN_QUEUE";
			
			org.hibernate.query.Query<JobData> query = 
				HibernateUtil.getPredefinedQuery(dc.getHibernateSession(), hqlName, JobData.class);
			/*
				SELECT job
				FROM JobData job
				WHERE job.status = :status
				AND job.type = :type
				AND job.removedBy IS NULL
				[AND job.pluginDefinition.useInternalJobQueue = true]
				[AND job.jobAgentId IS NULL]
				ORDER BY job.priority ASC, job.scheduled ASC
			*/
			query.setParameter("status", Job.Status.WAITING.getValue(), TypeWrapper.H_INTEGER);
			query.setParameter("type", Job.Type.RUN_PLUGIN.getValue(), TypeWrapper.H_INTEGER);
			List<JobData> jobs = HibernateUtil.loadList(query, null);
			
			log.info("Found " + jobs.size() + " jobs waiting for execution");
			int started = 0;
			for (JobData job : jobs)
			{
				// Find a free slot to execute the job
				Job.ExecutionTime estimated = Job.ExecutionTime.fromValue(job.getEstimatedExecutionTime());
				Job.ExecutionTime slotToUse = null;
				Job.ExecutionTime[] times =  Job.ExecutionTime.values();
				for (int i = estimated.ordinal(); i < times.length; ++i)
				{
					if (usedSlots.get(times[i]) < maxSlots.get(times[i]))
					{
						slotToUse = times[i];
						i = times.length;
					}
				}
				if (slotToUse == null)
				{
					log.info("Couldn't find a free slot for executing job: " + job);
				}
				else
				{
					// Create JobRunner in that slot
					log.info("Starting job " + job + " in slot " + slotToUse);
					log.debug("Slot: " + slotToUse + "; used: " + usedSlots.get(slotToUse) + "; max: " + maxSlots.get(slotToUse));
					JobRunner runner = new JobRunner(job, slotToUse);
					runners.add(runner);
					usedSlots.put(slotToUse, usedSlots.get(slotToUse)+1);
					
					// Create and start thread
					Thread t = new Thread(runnersGroup, runner);
					t.setDaemon(false);
					t.setPriority(priorities.get(estimated));
					t.start();
					started++;
				}
			}
			log.info("Started " + started + " job(s) of " + jobs.size() + " total");
		}
		catch (Throwable ex)
		{
			log.error("Exception while starting jobs", ex);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	private void requestExternalStatusUpdate()
	{
		// Is it time yet?
		if (System.currentTimeMillis() < nextStatusUpdateRequest) return;
		log.info("Checking for jobs to request status update");
		
		DbControl dc = null;
		try
		{
			dc = sc.newDbControl();
			
			org.hibernate.query.Query<JobData> query = 
				HibernateUtil.getPredefinedQuery(dc.getHibernateSession(), "FIND_EXTERNAL_JOBS_FOR_STATUS_UPDATE", JobData.class);
			/*
				SELECT job
				FROM JobData job
				AND job.type = :type
				AND job.removedBy IS NULL
				AND NOT job.signalTransporter IS NULL
			*/
			query.setParameter("type", Job.Type.OTHER.getValue(), TypeWrapper.H_INTEGER);
			List<JobData> jobs = HibernateUtil.loadList(query, null);
			
			log.info("Found " + jobs.size() + " jobs for status update");
			int requested = 0;
			for (JobData job : jobs)
			{
				Job j = dc.getItem(Job.class, job);
				if (j.requestStatusUpdate())
				{
					requested++;
				}
			}
			log.info("Reqested status update for " + requested + " job(s) of " + jobs.size() + " total");
		}
		catch (Throwable ex)
		{
			log.error("Exception while requesting status update", ex);
		}
		finally
		{
			if (dc != null) dc.close();
			nextStatusUpdateRequest = System.currentTimeMillis() + 15 * 60 * 1000; // 15 minutes
		}
		
	}
	
	/**
		Called from a {@link JobRunner} when it has finished 
		executing the job. IT MUST NEVER FORGET TO CALL THIS METHOD.
	*/
	private void jobDone(JobRunner runner)
	{
		log.info("Ending job " + runner.getJob() + " in slot " + runner.getUsedSlot());
		Job.ExecutionTime slot = runner.getUsedSlot();
		usedSlots.put(slot, usedSlots.get(slot) - 1);
		runners.remove(runner);
	}
	
	private void sendErrorMessage(SessionControl impersonated, int jobId, Throwable ex)
	{
		DbControl dc = null;
		try
		{
			dc = impersonated.newDbControl();
			Job job = Job.getById(dc, jobId);
			job.doneError(ex.getMessage(), Arrays.asList(ex));
			dc.commit();
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	/**
		Cleanup
	*/
	void close()
	{
		log.info("Closing job queue. " + runners.size() + " job(s) still active.");
		sc.logout();

		// Send SHUTDOWN/ABORT to all jobs that supports signalling
		signalReceiver.close(closeTimeout);
	}
	
	/**
		Encapsulates the running of a job in a separate thread.
		Calls the {@link #jobDone(JobRunner)} method when finished.
	*/
	private class JobRunner
		implements Runnable
	{
	
		private final JobData jobData;
		private final int jobId;
		private final int ownerId;
		private final Job.ExecutionTime slot;
		
		private JobRunner(JobData jobData, Job.ExecutionTime slot)
		{
			this.jobData = jobData;
			this.jobId = jobData.getId();
			this.ownerId = jobData.getOwner().getId();
			this.slot = slot;
		}
	
		/*
			From the Runnable interface
			-------------------------------------------
		*/
		/**
			Execute the job. 
		*/
		@Override
		public void run()
		{
			Job job = null;
			DbControl dc = null;
			try
			{
				// Log in as the owner of the job
				SessionControl impersonated = sc.impersonateLogin(ownerId, "Running job " + getJob());
				dc = impersonated.newDbControl();
				job = Job.getById(dc, jobId);
				
				// Status may have changed since the main query
				if (job.getStatus() != Job.Status.WAITING) return;
				
				// Change the status to Job.Status.PREPARED
				job.setPrepared(Application.getHostName(), null);
				try
				{
					dc.commit();
				}
				catch (ItemModifiedException ex)
				{
					// Another thread or process has started the job already, return 
					// without starting the job
					return;
				}
				finally
				{
					dc.close();
				}
				// Now we know that noby else is trying to execute the job
				// Reload the job again in a new DbControl
				dc = impersonated.newDbControl();
				job = Job.getById(dc, jobId);
				
				// Set the active project
				int projectId = job.getActiveProjectId();
				if (projectId != 0)
				{
					try
					{
						Project activeProject = Project.getById(dc, projectId);
						impersonated.setActiveProject(activeProject);
					}
					catch (Throwable t)
					{
						log.warn("Exception while setting active project to Project[id=" + projectId + 
								"]. Continuing with no active project.", t);
					}
				}
				
				// Get an execution request and close the db connection
				PluginExecutionRequest exec = null;
				try
				{
					exec = job.execute(null, Application.getHostName());
				}
				catch (Throwable t)
				{
					// The job could not be started, set error status and send message to owner
					sendErrorMessage(impersonated, jobId, t);
					throw t;
				}
				exec.registerSignalReceiver(signalReceiver);
				dc.commit();
				
				// Now, we can execute the job
				PluginResponse pres = exec.invoke();
				
				// Log out and check the return status
				impersonated.logout();
				if (pres.getStatus() == Response.Status.ERROR)
				{
					List<? extends Throwable> errors = pres.getErrorList();
					Throwable firstError = (errors == null || errors.size() == 0) ? null : errors.get(0);
					throw new BaseException(pres.getMessage(), firstError);
				}
			}
			catch (Throwable ex)
			{
				// All errors are logged, no use in re-throwing them
				log.error("Exception while executing job: " + job, ex);
			}
			finally
			{
				jobDone(this); // IMPORTANT, FREE THE SLOT WE ARE USING !!!!
				if (dc != null) dc.close();
			}
		}
		// -------------------------------------------
		
		/**
			The slot the job is executing in.
		*/
		private Job.ExecutionTime getUsedSlot()
		{
			return slot;
		}
		
		/**
			Get the job.
		*/
		private JobData getJob()
		{
			return jobData;
		}
	}
	
	/**
		Our own keyring containing the permissions we need to execute the jobs.
		The only required permission is: {@link Permission#ACT_AS_ANOTHER_USER}.
		All other permissions are denied.
	*/
	private static class JobQueueKeyring
		extends Keyring
	{
	
		private final int systemPermissions;
		private final int jobPermissions;
		private final Set<Integer> emptySet;
		private final Map<Integer, Integer> emptyMap;
	
		private JobQueueKeyring()
		{
			super(0);
			this.systemPermissions = Permission.grant(Permission.ACT_AS_ANOTHER_USER);
			this.jobPermissions = Permission.grant(Permission.READ);
			this.emptySet = Collections.emptySet();
			this.emptyMap = Collections.emptyMap();
		}
		/*
			From the Keyring class
			-------------------------------------------
		*/
		/**
			Never reload since everything is static.
		*/
		@Override
		boolean getReload()
		{
			return false;
		}
		/**
			Return {@link Permission#ACT_AS_ANOTHER_USER} if <code>itemType</code> = 
			{@link Item#SYSTEM}, 0 otherwise. 
		*/
		@Override
		int getRolePermissions(Item itemType)
		{
			if (itemType == Item.SYSTEM)
			{
				return systemPermissions;
			}
			else if (itemType == Item.JOB)
			{
				return jobPermissions;
			}
			else
			{
				return 0;
			}
		}
		/**
			Return 0.
		*/
		@Override
		int getSharedPermissions(ShareableData sharableData)
		{
			return 0;
		}
		/**
			Return 0.
		*/
		@Override
		int getAllPermissions(Item itemType, UserData owner, ItemKeyData itemKey, ProjectKeyData projectKey)
		{
			return 0;
		}
		/**
			Return empty set.
		*/
		@Override
		Set<Integer> getRoles()
		{
			return emptySet;
		}
		/**
			Return empty set.
		*/
		Set<Integer> getGroups()
		{
			return emptySet;
		}
		/**
			Return empty set.
		*/
		@Override
		Set<Integer> getUsers()
		{
			return emptySet;
		}
		/**
			Return empty map.
		*/
		@Override
		Map<Integer, Integer> getProjects()
		{
			return emptyMap;
		}
		/**
			Return empty set.
		*/
		Set<Integer> getItemKeys()
		{
			return emptySet;
		}
		/**
			Return empty set.
		*/
		Set<Integer> getProjectKeys()
		{
			return emptySet;
		}
		// -------------------------------------------
	
	}
}
