/*
	$Id: ParameterType.java 6987 2015-10-30 08:42:18Z nicklas $

	Copyright (C) 2005 Samuel Andersson, Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.ParameterValueData;
import net.sf.basedb.util.Enumeration;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
	This is the base class for all parameter types. A <code>ParameterType</code>
	is used by objects that take some kind of parameter input from an user.
	
	@author Samuel, Nicklas, Enell
	@version 2.0
	@base.modified $Date: 2015-10-30 09:42:18 +0100 (fre, 30 okt. 2015) $
*/
public abstract class ParameterType<T>
{
	/**
		List of allowed values.
	*/
	private List<T> items;
	
	/**
		List of allowed values.
	*/
	private Enumeration<T, String> enumeration;
	
	private boolean checkValues;
	
	/**
		The class of allowed parameter values.
	*/
	private final Class<T> clazz;

	/**
		The matching type of parameter values (can be null)
	*/
	private final Type valueType;
	
	/**
		Default value of the parameter.
	*/
	private final T defaultValue;
	
	/**
		If the parameter can be null or not.
	*/
	private final boolean notNull;

	/**
		Height of the input field for the parameter in the GUI.
		Only used as a hint for the client.
	*/
	private final int height;

	/**
		Width of the input field for the parameter in the GUI.
		Only used as a hint for the client.
	*/
	private final int width;

	/**
		How many values that can be tied to this
		parameter. 0 is unlimited values and 1 is default.
	*/
	private final int multiplicity;

	/**
		Creates a parameter type with no default value and allowing null values.
		@param clazz The class of the parameters that this type holds
	*/
	ParameterType(Type valueType, Class<T> clazz)
	{
		this(valueType, clazz, null, false);
	}
	
	/**
		Creates a new parameter type.
		@param clazz The class of the parameters that this type holds.
		@param defaultValue The parameters default value.
		@param notNull If the parameter is allowed to be null.
	*/
	ParameterType(Type valueType, Class<T> clazz, T defaultValue, boolean notNull)
	{
		this(valueType, clazz, defaultValue, notNull, 1, 0, 0);
	}
	
	ParameterType(Type valueType, Class<T> clazz, T defaultValue, boolean notNull, int multiplicity, int width, int height)
	{
		this.valueType = valueType;
		this.clazz = clazz;
		this.defaultValue = defaultValue;
		this.notNull = notNull;
		this.multiplicity = multiplicity;
		this.width = width;
		this.height = height;
	}

	ParameterType(Type valueType, Class<T> clazz, T defaultValue, boolean notNull, int multiplicity, int width, int height, List<T> items)
	{
		this(valueType, clazz, defaultValue, notNull, multiplicity, width, height);
		this.items = items == null ? null : Collections.unmodifiableList(new ArrayList<T>(items));
	}
	
	ParameterType(Type valueType, Class<T> clazz, T defaultValue, boolean notNull, int multiplicity, int width, int height, Enumeration<T, String> enumeration, boolean checkValues)
	{
		this(valueType, clazz, defaultValue, notNull, multiplicity, width, height);
		this.enumeration = enumeration;
		this.checkValues = checkValues;
		if (enumeration != null)
		{
			enumeration.lock();
			List<T> temp = new ArrayList<T>(enumeration.size());
			for (int i = 0; i < enumeration.size(); ++i)
			{
				temp.add(enumeration.getKey(i));
			}
			this.items = Collections.unmodifiableList(temp);
		}
	}
	
	/**
		Get the date type of values that can be stored in this parameter. May
		return null if there is no matching Type definition.
		@return A Type object or null
		@since 2.16
	*/
	public Type getValueType()
	{
		return valueType;
	}
	
	/**
		Get the class a value must have to match this parameter type.
	*/
	public Class<T> getParameterClass()
	{
		return clazz;
	}
	
	/**
		Get the height of the input field for the parameter in the GUI.
		Only used as a hint for the client.
		@return The height of the input field.
	*/
	public int getHeight()
	{
		return height;
	}
	

	/**
		Get the width of the input field in the GUI.
		Only used as a hint for the client.
		@return The width of the input field.
	*/
	public int getWidth()
	{
		return width;
	}
	
	/**
		Check if the parameter can be null. Will return true if it can't
		be null.
		@return True if not null, otherwise false.
	*/
	public boolean getNotNull()
	{
		return notNull;
	}

	/**
		Get how many values that can be tied to this
		parameter. 0 is unlimited values and 1 is default.
		@return The multiplicity.
	*/
	public int getMultiplicity()
	{
		return multiplicity;
	}
	
	/**
		Should this parameter be masked in the GUI?
		@return FALSE, unless overridden by subclass
		@since 3.7
	*/
	public boolean isMasked()
	{
		return false;
	}
	
	/**
		Get the default value.
		@return The default value or null if none i set.
	*/
	public T getDefaultValue()
	{
		return defaultValue;
	}

	/**
		Check if a value is valid according to the settings of this parameter type.
		@param name The name of the parameter, used for better exception messages
		@param value The value to test
		@throws InvalidDataException If the value is invalid
	*/
	public void validate(String name, Object value)
		throws InvalidDataException
	{
		if (value == null && getNotNull()) value = getDefaultValue();
		if (value == null)
		{
			if (getNotNull()) throw new InvalidUseOfNullException(name);
		}
		else if (!clazz.isInstance(value))
		{
			throw new InvalidDataException("Value for parameter '"+name+"' is a '"+value.getClass().getName()+"', not a '"+clazz.getName()+"'");
		}
		else if(items != null && items.size() > 0)
		{
			if (!items.contains(value))
			{
				boolean found = false;
				if (enumeration != null && checkValues)
				{
					// Value not found among keys in the enumeration, also check values
					found = enumeration.getEntriesByValue((String)value).size() > 0;
				}
				if (!found)
				{
					throw new InvalidDataException("The value '" + value + "' isn't in the list of allowed values for parameter '" + name + "'.");
				}
			}
		}
		else
		{
			validateValue(name, clazz.cast(value));
		}
	}

	/**
		Check if a list of values contain invalid values. 
	 	@param name The name of the parameter, used for better exception messages.  
		@param values A list of objects to check
		@throws InvalidDataException If the list contains too many values as 
			specified by the {@link #getMultiplicity()} setting or has invalid values
			as checked by {@link #validate(String, Object)}
	*/
	public void validate(String name, List<?> values)
		throws InvalidDataException
	{
		if ((values == null || values.size() == 0) && getNotNull() && getDefaultValue() != null)
		{
			values = Arrays.asList(getDefaultValue());
		}
		if (values != null && values.size() > 0)
		{
			if (multiplicity > 0 && values.size() > multiplicity) 
			{
				throw new InvalidDataException("Found "+values.size()+" values in parameter '"+name+"'. Only "+multiplicity+" is allowed.");
			}
			for (Object value : values)
			{
				validate(name, value);
			}
		}
		else if (getNotNull())
		{
			 throw new InvalidUseOfNullException(name);
		}
	}

	/**
		Create a new {@link ParameterValueData} subclass which is used to
		store values of this value type in the database.
	*/
	abstract ParameterValueData<?> newParameterValueData();
	
	/**
		Validate a value. The validate methods in this class checks for
		not-null and multiplicity (if needed). Each subclass must implement
		checks for other properties, such as maximum string length, lower and upper
		limits, etc. By the time this method is called, it is certain that
		the value is of the correct class and isn't null.
		@param name The name of the parameter, use the name if there is need to
			throw an exception
		@param value The value
		@throws InvalidDataException If the value is not valid
	*/
	abstract void validateValue(String name, T value)
		throws InvalidDataException;
	
	/**
		Check if the parameter type enumerates the allowed values
		or not. If the parameter is an enumeration the values can be 
		retreived by {@link #getItems()} or {@link #getEnumeration()}.
		NOTE! If an <code>Enumeration</code> has been supplied the <code>getItems</code>
		method will return a list of the keys of the enumeration.
		@return TRUE if the parameter type is an enumeration, FALSE otherwise
	*/
	public boolean isEnumeration()
	{
		return (enumeration != null && enumeration.size() > 0) || (items != null && items.size() > 0);
	}
		
	/**
		Get the list of values to choose from.
		@return List of values
		@see #getEnumeration()
	*/
	public List<T> getItems()
	{
		return items;
	}

	/**
		Get the list of values to choose from as an enumeration. An enumeration
		is different from the regular list returned by {@link #getItems()}
		in that it has second value attached to each value. The second value
		should be used for display purposes only while the first value is
		the one that is used.
		
		@return The enumeration of values or null
		@see #getItems()
	*/
	public Enumeration<T, String> getEnumeration()
	{
		return enumeration;
	}

}
