/*
	$Id: Presets.java 6898 2015-05-12 09:27:08Z nicklas $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.util.XmlUtil2;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;

import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;

/**
	This is a generic class for handling named presets consisting of multiple
	key/value pairs. One instance can handle one default preset and 
	multiple named ones. Saving and loading is done to/from XML strings. The XML
	strings could for example be stored as user settings 
	(for example with {@link SessionControl#setUserDefaultSetting(String, String)}
	or as properties of items in the database.

	@author Nicklas
	@version 2.2
	@base.modified $Date: 2015-05-12 11:27:08 +0200 (tis, 12 maj 2015) $
*/
public class Presets
	implements Iterable<Presets.Preset>
{

	/**
		The DTD which is used to validate the XML data.
	*/
	private static final URL dtdFile = Presets.class.getResource("/net/sf/basedb/core/dtd/presets.dtd");

	private Preset defaultPreset;
	private Map<String, Preset> presets;
	
	/**
		Create a new empty presets object.
	*/
	public Presets()
	{
		presets = new TreeMap<String, Preset>();
	}
	
	/*
		From the Iterable interface
		-------------------------------------------
	*/
	/**
		Iterate over all named presets. The default preset, if it exists,
		is not included in the iteration.
	*/
	@Override
	public Iterator<Presets.Preset> iterator()
	{
		return presets.values().iterator();
	}
	// -------------------------------------------

	
	/**
		Load presets from an XML string.
		@param xml The XML string
	*/
	public void loadFrom(String xml)
	{
		try
		{
			Document dom = XmlUtil2.getValidatedXml(xml, dtdFile);
			loadPresets(dom);
		}
		catch (Exception ex)
		{
			throw new BaseException(ex);
		}
	}
	
	/**
		Load presets from a stream.
		@param in The stream to read from
		@param filename The original filename the stream is loaded
			from, or null if not known
		@since 2.7
	*/
	public void loadFrom(InputStream in, String filename)
	{
		try
		{
			Document dom = XmlUtil2.getValidatedXML(in, dtdFile, filename);
			loadPresets(dom);
		}
		catch (Exception ex)
		{
			throw new BaseException(ex);
		}
	}
	
	/**
		Write the prestes to a stream as an XML file.
		@param out The stream to write to.
		@since 2.7
	*/
	public void writeTo(OutputStream out)
	{
		try
		{
			new XMLOutputter(Format.getPrettyFormat()).output(buildDocument(), out);
		}
		catch (Exception ex)
		{
			throw new BaseException(ex);
		}
	}
	
	/**
		Convert the presets to an XML string.
		@return The XML string
	*/
	public String asXml()
	{
		return new XMLOutputter().outputString(buildDocument());
	}

	/**
		Convert the presets to a XML document.
		@since 2.7
	*/
	private Document buildDocument()
	{
		Document dom = XmlUtil2.createDom("presets", "presets.dtd");
		Element root = dom.getRootElement();
		if (defaultPreset != null)
		{
			Element el = new Element("default");
			defaultPreset.buildDom(el);
			root.addContent(el);
		}
		for (Preset p : presets.values())
		{
			Element el = new Element("preset");
			p.buildDom(el);
			root.addContent(el);
		}
		return dom;
	}
	
	/**
		Get the default preset. If no default exists a new default
		will be created.
	*/
	public Preset getDefault()
	{
		if (defaultPreset == null) defaultPreset = new Preset();
		return defaultPreset;
	}
	
	/**
		Get a named preset. If no preset with the given name exists
		a new preset is created.
		@param name The name of the preset
		@return The preset with the given name
	*/
	public Preset getPreset(String name)
	{
		Preset p = presets.get(name);
		if (p == null)
		{
			p = new Preset(name);
			presets.put(name, p);
		}
		return p;
	}
	
	/**
		Check if a preset with the given name exists.
		@since 3.4
	*/
	public boolean exists(String name)
	{
		return presets.containsKey(name);
	}
	
	/**
		Delete a named preset.
		@param name The name of the preset to delete
	*/
	public void deletePreset(String name)
	{
		presets.remove(name);
	}
	
	/**
		Parse an XML Document and load all presets.
	*/
	private void loadPresets(Document dom)
	{
		presets.clear();
		Element defaultPresetElement = dom.getRootElement().getChild("default");
		if (defaultPreset != null)
		{
			defaultPreset = new Preset(defaultPresetElement);
		}
		List<Element> presetElements = dom.getRootElement().getChildren("preset");
		for (Element el : presetElements)
		{
			Preset p = new Preset(el);
			presets.put(p.getName(), p);
		}
	}
	
	/**
		Represents a single preset. A preset contains any number of key-value
		pairs (string). A named preset has a non-null name.
	*/
	public static class Preset
		implements Iterable<Map.Entry<String, String>>
	{
		
		/**
			The key-value pairs in this preset.
		*/
		private Map<String, String> settings;
		
		/**
			The name of this preset. Null for the default.
		*/
		private String name;

		
		private Preset()
		{
			settings = new HashMap<String, String>();
		}
		
		private Preset(String name)
		{
			this();
			this.name = name;
		}
		
		/**
			Load preset name and key-value pairs from XML Document.
		*/
		private Preset(Element preset)
		{
			this(preset.getAttributeValue("name"));
			List<Element> settingtElements = preset.getChildren("setting");
			for (Element el : settingtElements)
			{
				String name = el.getAttributeValue("name");
				String value = el.getText();
				settings.put(name, value);
			}
		}
		
		/*
			From the Iterable interface
			-------------------------------------------
		*/
		/**
			Iterate over all settings.
		*/
		@Override
		public Iterator<Map.Entry<String, String>> iterator()
		{
			return settings.entrySet().iterator();
		}
		// -------------------------------------------
	
		/**
			Add XML elements to the parent element for this preset's key-value
			pairs. The parent element is either a &lt;preset&gt; or a &lt;default&gt;
			element.
			@param element The parent element to add this preset to
		*/
		private void buildDom(Element element)
		{
			if (name != null)
			{
				element.setAttribute("name", name);
			}
			for (Map.Entry<String, String> entry : settings.entrySet())
			{
				String name = entry.getKey();
				String value = entry.getValue();
				Element el = new Element("setting");
				el.setAttribute("name", name);
				el.setText(value);
				element.addContent(el);
			}
		}
		
		/**
			Get the name of this preset.
			@return The name, or null if this is the default preset
		*/
		public String getName()
		{
			return name;
		}
		
		/**
			Get the value for the setting with the specified name.
			@param name The name of the setting
			@return The value or null if no value exists
		*/
		public String getSetting(String name)
		{
			return settings.get(name);
		}
		
		/**
			Set a value for the specified setting.
			@param name The name of the setting
			@param value The value, use null to remove the setting
		*/
		public void setSetting(String name, String value)
		{
			if (value == null)
			{
				settings.remove(name);
			}
			else
			{
				settings.put(name, value);
			}
		}
		
		/**
			Get the number of settings in the preset.
			@since 3.4
		*/
		public int size()
		{
			return settings.size();
		}
		
		/**
			Get all keys in this preset. The returned list is a 
			copy of the settings and are not affected by later
			additions or removals.
		*/
		public List<String> getKeys()
		{
			return new ArrayList<String>(settings.keySet());
		}
	
	}

}
