/*
	$Id: ProjectPermissions.java 5368 2010-06-22 12:51:41Z nicklas $

	Copyright (C) 2005 Samuel Andersson, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.core;

import net.sf.basedb.core.data.ProjectData;

import java.util.Collections;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;

/**
	This class is used to specify permissions to share an item to
	projects. It is a three-step procedure:
	<ul>
	<li>Fill this object with permissions for specific projects
	<li>Create a new {@link ProjectKey} using this object as the
		input parameter to {@link ProjectKey#getNewOrExisting(DbControl, ProjectPermissions)}
	<li>Assign the project key to the item: {@link Shareable#setProjectKey(ProjectKey)}
	</ul>

	Don't forget to commit the transaction.
	<p>
	NOTE! As of BASE 2.6 this class is final because of a security issue. Existing
	code that has subclassed this class will no longer work. There is no workaround.
		
	@see ProjectKey
	@author Nicklas
	@version 2.0
	@base.modified $Date: 2010-06-22 14:51:41 +0200 (tis, 22 juni 2010) $
*/
public final class ProjectPermissions
{

	private final Map<ProjectData,Integer> projectPermissions;
	private final Map<ProjectData,Integer> originalPermissions;

	/**
		Create a new empty object.
	*/
	public ProjectPermissions()
	{
		projectPermissions = new HashMap<ProjectData,Integer>();
		originalPermissions = null;
	}
	
	/**
		Create a new object, and initialise it with the permissions from a
		<code>ProjectKey</code>.
		@param projectKey The project key from which we take the current permissions
		@throws InvalidDataException If the project key is null
	*/
	public ProjectPermissions(ProjectKey projectKey)
		throws InvalidDataException
	{
		if (projectKey == null) throw new InvalidUseOfNullException("projectKey");
		projectPermissions = new HashMap<ProjectData,Integer>(projectKey.getData().getProjects());
		originalPermissions = new HashMap<ProjectData,Integer>(projectPermissions);
	}
	
	/**
		Merge the permissions for all projects from the given project key with the permissions 
		in this object. If the project key contains permissions for a project that already 
		has permissions in this object the permissions are combined (added) to create 
		a new permission.
		
		@param projectKey A project key (if null, this method simply returns)
		@since 2.16
	*/
	public void merge(ProjectKey projectKey)
	{
		if (projectKey == null) return;
		for (Map.Entry<ProjectData, Integer> entry : projectKey.getData().getProjects().entrySet())
		{
			ProjectData project = entry.getKey();
			int permissions = IntegerUtil.getInt(entry.getValue());
			int current = IntegerUtil.getInt(projectPermissions.get(project));
			projectPermissions.put(project, current | permissions);
		}
	}

	
	/**
		Grant permissions to a project. Use an empty <code>Set</code> or null
		to remove all permissions for the project.

		@param project The <code>Project</code>
		@param permissions The permissions to grant
		@throws PermissionDeniedException If the logged in user doesn't have
			{@link Permission#USE} permission for the project
		@throws InvalidDataException If the project is null
		@see Permission
	*/
	public void setPermissions(Project project, Set<Permission> permissions)
		throws PermissionDeniedException, InvalidDataException
	{
		if (project == null) throw new InvalidUseOfNullException("project");
		project.checkPermission(Permission.USE);
		if (permissions == null || permissions.isEmpty())
		{
			projectPermissions.remove(project.getData());
		}
		else
		{
			projectPermissions.put(project.getData(), Permission.grant(permissions));
		}
	}

	/**
		Grant permissions to a project, keeping those that have already been set.
		
		@param project The <code>Project</code>
		@param permissions The permissions to grant
		@throws PermissionDeniedException If the logged in user doesn't have
			{@link Permission#USE} permission for the project
		@throws InvalidDataException If the project is null
		@see Permission
	*/
	public void addPermissions(Project project, Set<Permission> permissions)
		throws PermissionDeniedException, InvalidDataException
	{
		if (project == null) throw new InvalidUseOfNullException("project");
		project.checkPermission(Permission.USE);
		
		if (permissions != null && !permissions.isEmpty())
		{
			int current = IntegerUtil.getInt(projectPermissions.get(project.getData()));
			projectPermissions.put(project.getData(), current | Permission.grant(permissions));
		}
	}

	/**
		Get the permissions for a project.
		@param project The <code>Project</code> for which we want to get the permission
		@return An <code>Set</code> containing the granted permissions, or an
			empty set if no permissions have been granted
		@throws InvalidDataException If the project is null
		@see #getOriginalPermissions(Project)
		@see Permission
	*/
	public Set<Permission> getPermissions(Project project)
		throws InvalidDataException
	{
		if (project == null) throw new InvalidUseOfNullException("project");
		return Permission.fromInt(IntegerUtil.getInt(projectPermissions.get(project.getData())));
	}
	
	/**
		Get the original permissions for a project if this object was created with the
		{@link #ProjectPermissions(ProjectKey)} constructor.
		@param project The <code>Project</code> for which we want to get the permission
		@return An <code>Set</code> containing the original permissions, or an
			empty set
		@throws InvalidDataException If the project is null
		@see #getPermissions(Project)
		@since 2.6
	*/
	public Set<Permission> getOriginalPermissions(Project project)
	{
		if (project == null) throw new InvalidUseOfNullException("project");
		if (originalPermissions == null) return Collections.emptySet();
		return Permission.fromInt(IntegerUtil.getInt(originalPermissions.get(project.getData())));
	}
	
	/**
		Check if the current permissions for the project has been changed from the original
		permissions. This method will return false if the {@link #getPermissions(Project)}
		returns a set with the same permissions {@link #getOriginalPermissions(Project)}.
		@param project The project to check
		@return TRUE if the permissions have changed, FALSE otherwise
		@throws InvalidDataException If the project is null
		@since 2.6
	*/
	public boolean hasChanged(Project project)
	{
		if (project == null) throw new InvalidUseOfNullException("project");
		int current = IntegerUtil.getInt(projectPermissions.get(project.getData()));
		int original = originalPermissions == null ?
			0 : IntegerUtil.getInt(originalPermissions.get(project.getData()));
		return current != original;
	}	
	
	/**
		Get the number of entries.
	*/
	public int size()
	{
		return projectPermissions.size();
	}

	/**
		Get the internal representation of the permissions.
	*/
	final Map<ProjectData,Integer> getProjectPermissions()
	{
		return projectPermissions;
	}
	/**
		Get the internal representation of the original permissions.
		@since 2.6
	*/
	final Map<ProjectData,Integer> getOriginalPermissions()
	{
		return originalPermissions;
	}
}
