/**
	$Id: ReporterCloneBatcher.java 6908 2015-05-13 09:36:12Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import net.sf.basedb.core.data.RawBioAssayData;
import net.sf.basedb.core.data.ReporterCloneTemplateData;
import net.sf.basedb.core.data.UserData;
import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.JoinType;
import net.sf.basedb.core.query.Orders;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.query.Selects;
import net.sf.basedb.core.query.SqlResult;
import net.sf.basedb.core.signal.ThreadSignalHandler;

/**
	Batcher implementation that is used to handle cloned reporter
	information in the dynamic database.


	@author Nicklas
	@since 3.1
	@base.modified $Date: 2015-05-13 11:36:12 +0200 (ons, 13 maj 2015) $
*/
public class ReporterCloneBatcher
	extends AbstractBatcher
{

	/**
		The experiment this batcher inserts data for.
	*/
	private final Experiment experiment;
	private final VirtualDb virtualDb;
	/**
		The number of bytes used by a single row.
	*/
	private final long bytesPerRow;
	
	/**
		The statement that inserts cloned reporter data.
<pre class="code">
INSERT INTO D#Reporter# (id, version, external_id, ...)
VALUES (?, ?, ?, ...)
</pre>
	*/
	private PreparedStatement insertSql = null;

	/**
	 	The SQL string for the {@link #insertSql} statement.
	*/
	private String insertSqlStatement;
	
	private int[] parameterTypes;
		
	ReporterCloneBatcher(DbControl dc, Experiment experiment)
	{
		super();
		this.experiment = Experiment.getById(dc, experiment.getId());
		this.virtualDb = experiment.getVirtualDb();
		this.bytesPerRow = VirtualTable.CLONED_REPORTERS.getBytesPerRow(virtualDb);
		setDbControl(dc);
	}
	
	/*
		From the Batcher interface
		-------------------------------------------
	*/
	@Override
	public void flush() 
		throws BaseException
	{
		try
		{
			if (insertSql != null) flushInternal();
		}
		catch (SQLException ex)
		{
			logSql.error(insertSqlStatement, ex);
			LogUtil.logSQLExceptionChain(logSql, ex);
			throw new BaseException(ex);
		}
	}
	// -------------------------------------------
	/*
		From the AbstractBatcher class
		-------------------------------------------
	*/
	/**
		Update disk usage and close open SQL statements.
	*/
	@Override 
	void onBeforeClose()
		throws BaseException
	{
		try
		{
			if (insertSql != null) 
			{
				insertSql.close();
				insertSql = null;
			}
		}
		catch (SQLException ex)
		{
			logSql.error(ex.getMessage(), ex);
			LogUtil.logSQLExceptionChain(logSql, ex);
		}
		if (!getDbControl().isRolledBack() && virtualDb.hasClonedReporters())
		{
			String catalog = Application.getDynamicCatalog();
			String schema = Application.getDynamicSchema();
			String table = VirtualTable.CLONED_REPORTERS.getTableName(virtualDb);
			analyzeTable(catalog, schema, table);
		}
	}
	// -------------------------------------------

	/**
		Get the current experiment this batcher is used with.
	*/
	public Experiment getExperiment()
	{
		return experiment;
	}
	
	/**
		Drop the cloned reporter information from the dynamic database. If the
		current experiment doesn't have any cloned reporters, nothing is done.
		@throws PermissionDeniedException If the logged in user does't have
			write permission in this experiment
	*/
	public void dropClonedReporters()
	{
		experiment.checkPermission(Permission.WRITE);
		DbControl dc = getDbControl();
		
		if (virtualDb.hasClonedReporters())
		{
			// Schedule the old table for removal on successful commit
			String oldTableName = VirtualTable.CLONED_REPORTERS.getTableName(virtualDb);
			dc.addTransactionalAction(new DropDynamicTableAction(oldTableName, true, false));
			
			// Delete the old template
			ReporterCloneTemplateData oldClone = virtualDb.getData().getReporterCloneTemplate();
			HibernateUtil.deleteData(dc.getHibernateSession(), oldClone);
			virtualDb.getData().setReporterCloneTemplate(null);
			
			// Return used bytes to the experiment
			experiment.addBytes(-oldClone.getBytes());
		}
	}
	
	/**
		Create a reporter clone table in the dynamic database for this
		experiment and populate it with the current reporter annotations
		for all reporters that are part of the experiment. 
		<p>
		
		To find out which reporters that are part of the experiment, the batcher 
		can use either what has already been inserted into the {@link VirtualTable#POSITION}
		table (source={@link CloneSource#VIRTUAL}) or the reporters that are referenced 
		by the {@link RawBioAssay}:s attached to the experiment (source={@link CloneSource#RAW}). 
		The AUTO method uses the position table if it exists. The RAW method can't be used 
		with raw data types that doesn't store raw data in the database.
		
		<p>
		This method will create an immutable copy of the template so that we can always know
		which fields that have been cloned, even if the template is modified.
		If there already is a cloned reporter information it will be removed.
		<p>
		
		
		@param template The reporter clone template to use or null to clone
			all properties
		@param source The source of the cloned reporters, null is the same as AUTO
		@param progress An optional progress reporter
		@return The number of cloned reporters
		@throws InvalidUseOfNullException If the template is null
		@throws PermissionDeniedException If the logged in user does't have
			write permission in this experiment or use permission for the template
			or read permission for reporters
	*/
	public int cloneReporters(CloneSource source, ReporterCloneTemplate template, 
		ProgressReporter progress)
	{
		// Permission checks
		experiment.checkPermission(Permission.WRITE);
		if (template != null) template.checkPermission(Permission.USE);
		if (!getSessionControl().hasPermission(Permission.READ, Item.REPORTER))
		{
			throw new PermissionDeniedException(Permission.READ, "Reporter");
		}
		
		if (source == null) source = CloneSource.AUTO;
		if (source == CloneSource.VIRTUAL)
		{
			if (!virtualDb.hasTable(VirtualTable.POSITION))
			{
				throw new InvalidDataException("Can't use VIRTUAL as source since there is no position-to-reporter data");
			}
		}
		if (source == CloneSource.AUTO)
		{
			if (virtualDb.hasTable(VirtualTable.POSITION))
			{
				source = CloneSource.VIRTUAL;
			}
			else
			{
				source = CloneSource.RAW;
			}
		}
		if (source == CloneSource.RAW)
		{
			if (!experiment.getRawDataType().isStoredInDb())
			{
				throw new InvalidDataException("Can't use RAW as source since raw data is not stored in database");
			}
		}
		
		
		DbControl dc = getDbControl();
		
		// Drop existing cloned reporters (if any)
		dropClonedReporters();
		
		// Create a copy of the template
		ReporterCloneTemplateData copy = template != null ? 
			template.createLockedCopy() : ReporterCloneTemplate.createAllCopy("All");
		copy.setOwner(HibernateUtil.loadData(dc.getHibernateSession(), UserData.class, dc.getSessionControl().getLoggedInUserId()));
		virtualDb.getData().setReporterCloneTemplate(copy);
		HibernateUtil.saveData(dc.getHibernateSession(), copy);

		// Create the clone table...
		if (progress != null) progress.display(0, "Creating clone table...");
		String cloneTableName = HibernateUtil.createVirtualTable(virtualDb, VirtualTable.CLONED_REPORTERS);
		// ...and an action that drops it in the case of a rollback
		dc.addTransactionalAction(new DropDynamicTableAction(cloneTableName, false, true));
		
		long bytes = 0;
		int insertCount = 0;
		int batchSize = getBatchSize();
		try
		{
			boolean useRawData = source == CloneSource.RAW;
			
			DynamicPositionQuery query = useRawData ? null : buildSelectQuery();
			DynamicRawDataQuery rawQuery = useRawData ? buildRawSelectQuery() : null;
			
			long totalCount = query != null ? query.count(dc) : rawQuery.count(dc);
			int interval = (int)(totalCount / 50);
			if (interval < 10) interval = 10;
			if (progress != null)
			{
				progress.display(5, "Cloning reporters (0/" + totalCount + " done)...");
			}
			
			PreparedStatement insertSql = buildInsertSql();
			
			DynamicResultIterator it = query != null ? query.iterate(dc) : rawQuery.iterate(dc);
			while (it.hasNext())
			{
				ThreadSignalHandler.checkInterrupted();
				
				SqlResult row = it.next();
				long stringBytes = 0;
				for (int index = 1; index < parameterTypes.length; ++index)
				{
					Object valueToCopy = row.getObject(index);
					insertSql.setObject(index, valueToCopy, parameterTypes[index]);
					if (valueToCopy instanceof String)
					{
						stringBytes += valueToCopy.toString().length();
					}
				}
				bytes += bytesPerRow + stringBytes;
				
				insertSql.addBatch();
				if (++insertCount == batchSize)
				{
					flushInternal();
				}
				
				if (progress != null && insertCount % interval == 0)
				{
					int percent = 5 + (int)((90L * insertCount) / totalCount);
					progress.display(percent, "Cloning reporters ("+ 
							insertCount + "/" + totalCount + " done)...");
				}
			}
			
			virtualDb.getReporterCloneTemplate().addBytes(bytes);
			experiment.addBytes(bytes);
			
			flushInternal();
			
		}
		catch (SQLException ex)
		{
			logSql.error(insertSqlStatement, ex);
			LogUtil.logSQLExceptionChain(logSql, ex);
			throw new BaseException(ex);
		}
		
		if (progress != null)
		{
			progress.display(100, "Successfully cloned " + insertCount + " reporters");
		}
		
		return insertCount;
	}
	
	
	private void flushInternal()
		throws SQLException
	{
		insertSql.executeBatch();
	}
	
	/**
		Builds the insert SQL statement.
		@see BatchUtil#buildInsertSql(VirtualDb, VirtualTable, VirtualColumn[], Object[])
	*/
	private PreparedStatement buildInsertSql()
		throws SQLException, BaseException
	{

		// Create VirtualColumn representations of all cloned properties
		VirtualColumn[] columns = VirtualTable.CLONED_REPORTERS.getColumns(virtualDb);
		
		insertSqlStatement =  BatchUtil.buildInsertSql(
			virtualDb,
			VirtualTable.CLONED_REPORTERS,
			columns,
			new Object[] {}
		);
				
		insertSql = HibernateUtil.getConnection(getDbControl().getHibernateSession()).prepareStatement(insertSqlStatement);
		return insertSql;
	}
	
	/**
		Builds a query that select all reporter properties that should be cloned
		from the real reporter table.
	*/
	private DynamicPositionQuery buildSelectQuery()
	{
		List<ClonableProperty> cloned = virtualDb.getReporterCloneTemplate().getClonableProperties();
		parameterTypes = new int[cloned.size()+1];
		
		DynamicPositionQuery query = new DynamicPositionQuery(experiment);
		query.setUseClonedReporters(false);
		query.joinReporters(JoinType.INNER);
		query.setDistinct(true);

		query.order(Orders.asc(Dynamic.reporter("id")));

		int index = 1;
		for (ClonableProperty cp : cloned)
		{
			query.select(Selects.expression(Dynamic.reporter(cp.getName()), "cp" + index, cp.getName().equals(ClonableProperty.ID)));
			parameterTypes[index] = cp.getType().getTypeWrapper().getSQLType();
			index++;
		}
		return query;
	}
	
	private DynamicRawDataQuery buildRawSelectQuery()
	{
		List<ClonableProperty> cloned = virtualDb.getReporterCloneTemplate().getClonableProperties();
		parameterTypes = new int[cloned.size()+1];

		DynamicRawDataQuery query = new DynamicRawDataQuery(experiment.getRawDataType());
		query.joinReporters(JoinType.INNER);
		query.setDistinct(true);
		query.order(Orders.asc(Dynamic.reporter("id")));

		Set<Integer> ids = new HashSet<Integer>();
		for (RawBioAssayData rba : experiment.getData().getRootRawBioAssays().keySet())
		{
			ids.add(rba.getId());
		}
		
		query.restrict(Restrictions.in(Dynamic.rawData("rawBioAssay"), Expressions.parameter("rawBioAssays")));
		query.setParameter("rawBioAssays", ids, Type.INT);
		
		int index = 1;
		for (ClonableProperty cp : cloned)
		{
			query.select(Selects.expression(Dynamic.reporter(cp.getName()), "cp" + index, cp.getName().equals(ClonableProperty.ID)));
			parameterTypes[index] = cp.getType().getTypeWrapper().getSQLType();
			index++;
		}

		return query;
	}

	/**
		Specifies which source table to use for finding out which reporters
		that are part of the current experiment.
	*/
	public static enum CloneSource
	{
		/**
			Select VIRTUAL if the experiment has position data in the virtual database,
			otherwise the RAW options used.
		*/
		AUTO,
		/**
			Use the raw data for all raw bioassays in the experiment as the source for reporters.
		*/
		RAW,
		/**
			Use the position-to-reporter mapping in the experiment.
		*/
		VIRTUAL
		
	}
	
}
