/**
	$Id$

	Copyright (C) 2014 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.sql.SQLException;
import java.util.Collections;
import java.util.Map;
import java.util.Set;

import net.sf.basedb.core.data.ItemKeyData;
import net.sf.basedb.core.data.ProjectKeyData;
import net.sf.basedb.core.data.ShareableData;
import net.sf.basedb.core.data.UserData;
import net.sf.basedb.core.dbengine.DbEngine;
import net.sf.basedb.core.hibernate.GetCurrentCatalogWork;
import net.sf.basedb.core.hibernate.SchemaExistsWork;
import net.sf.basedb.core.hibernate.SchemaGenerator;

import org.hibernate.FlushMode;
import org.hibernate.query.Query;
import org.hibernate.Session;
import org.hibernate.SessionBuilder;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.boot.MetadataSources;
import org.hibernate.boot.registry.BootstrapServiceRegistryBuilder;
import org.hibernate.boot.registry.StandardServiceRegistryBuilder;
import org.hibernate.dialect.Dialect;

/**
	A special session control implementation used for services. It provides extra
	functionality that an extension can use to create database objects in a 
	separate database schema/catalog for their own use. Note that the extra
	API in this class depends on Hibernate API and may change without 
	notification in case the Hibernate API is changed (though we will try to
	give advanced notice).
	
	@author nicklas
	@since 3.4
*/
public class ServiceSessionControl
	extends SessionControl
{
	
	private static final org.slf4j.Logger log =
		org.slf4j.LoggerFactory.getLogger(ServiceSessionControl.class);
	
	ServiceSessionControl() 
	{
		super(null, "net.sf.basedb.clients.internal", false, Application.getHostName());
		login(new ServicesKeyring());
	}
	
	/**
		Create an object that can be used for collecting configuration options 
		and other settings that are needed to create session factories for services.
		<p>
		An application id, schema and schema version is required. This 
		information is used by BASE to check the currently installed schema 
		version and only update the database if there is a change. Note that the 
		schema version must always increase from one version to the next.
		<p>
		A class loader is needed so that Hibernate can load the classes from
		extension classes.
		<p>
		It is possible to use {@link SessionFactoryConfiguration#getInstallMode()}
		and {@link SessionFactoryConfiguration#getInstalledSchemaVersion()} to check
		the current status of the database.
		<p>
		Use {@link SessionFactoryConfiguration#getRegistryBuilder()} to get a Hibernate
		object for setting things that are typically configured in <code>hibernate.cfg.xml</code>.
		Typically, {@link StandardServiceRegistryBuilder#applySetting(String, Object)} is the main
		method to use for this.
		<p>
		Use {@link SessionFactoryConfiguration#getMetadataSources()} to get a
		Hibernate object for adding mapped entity classes. Typically, 
		{@link MetadataSources#addAnnotatedClass(Class)} is the main method to use for this.
		Note that once the metadata sources object has been created it is not
		possible to modify the registry settings.
		<p>
		Also note that by default the configured session factory need to be
		"piggy-backed" on a BASE {@link DbControl} transaction. Create
		Hibernate sessions with {@link #newSession(org.hibernate.SessionBuilder, DbControl)}.
		<p>
		If you want to use connections/transactions that are independent from BASE 
		call {@link SessionFactoryConfiguration#setUseSeparateConnections(boolean)} 
		but make sure you also get connection pooling configured. 
		You can still use "piggy-backed" connections.
		
		@param appId The ID of the extensions (which should be unique)
		@param schema  The schema name the extension wants to use for it's database tables
		@param schemaVersion The version of the current schema. If this is different from the
			database scheme it will automatically be updated when {@link #buildSessionFactory(SessionFactoryConfiguration)}
			is called.
		@param classLoader A class loader is needed for Hibernate to be able to locate the
			classes. Use the classloader returned from {@link Class#getClassLoader()} using the
			class from one of the classes in the extension
	*/
	public SessionFactoryConfiguration createSessionFactoryConfiguration(String appId, String schema, int schemaVersion, ClassLoader classLoader)
	{
		if (appId == null) throw new ConfigurationException("An application-id must be specified");
		if (schema == null) throw new ConfigurationException("A schema must be specified");
		
		SessionFactoryConfiguration sfCfg = new SessionFactoryConfiguration(appId, schema, schemaVersion, classLoader);
		sfCfg.init();
		return sfCfg;
	}
	
	String getCatalog()
	{
		Session session = null;
		String catalog = null;
		try
		{
			session = HibernateUtil.newSession();
			catalog = HibernateUtil.doReturningWork(session, new GetCurrentCatalogWork());
			if (log.isDebugEnabled()) log.debug("Current catalog: " + catalog);
		}
		catch (SQLException ex)
		{
			throw new DatabaseException(ex);
		}
		finally
		{
			if (session != null) HibernateUtil.close(session);
		}
		return catalog;
	}
	
	boolean schemaExists(String catalog, String schema)
	{
		Session session = null;
		boolean schemaExists = false;
		try
		{
			session = HibernateUtil.newSession();
			schemaExists = HibernateUtil.doReturningWork(session, new SchemaExistsWork(catalog, schema));
		}
		catch (SQLException ex)
		{
			throw new DatabaseException(ex);
		}
		finally
		{
			if (session != null) HibernateUtil.close(session);
		}
		return schemaExists;
	}
	
	/**
		Create a session factory for the given configuration. This method will add
		connection information from the base.config file automatically
		to the configuration object. Note that sessions can't
		
		Classes that have been defined in the configuration should be without a
		schema definition and will then be put into the specified schema automatically.
		
		If the given schema and tables doesn't exists they will be created automatically,
		otherwise an UPDATE action is performed to modify an existing installation.
		
		
		@param config A configuration instance containing information needed to build
			the session factory
		@return A session factory instance
	*/
	public SessionFactory buildSessionFactory(SessionFactoryConfiguration config)
	{
		String schema = config.getSchema();
		String actualCatalog = config.getActualCatalog();
		String actualSchema = config.getActualSchema();
		if (log.isDebugEnabled()) 
		{
			log.debug("Building session factory for schema: " + schema);
			log.debug("Actual catalog/schema: " + actualCatalog + "/" + actualSchema);
		}
		
		SchemaGenerator.Mode installMode =  config.getInstallMode();
		if (log.isDebugEnabled()) log.debug("Mode: " +installMode);

		MetadataSources ms = config.getMetadataSources();
		org.hibernate.boot.Metadata metadata = ms.buildMetadata();

		DbEngine dbEngine = HibernateUtil.getDbEngine();
		Dialect dialect = HibernateUtil.getDialect();

		Session session = null;
		Transaction tx = null;
		SessionFactory sf = null;
		
		try
		{
			session = HibernateUtil.newSession();
			tx = HibernateUtil.newTransaction(session);
			
			if (installMode == SchemaGenerator.Mode.INSTALL)
			{
				// We need to create the schema/catalog
				String sql = dbEngine.getCreateSchemaSql(actualCatalog, actualSchema);
				if (log.isDebugEnabled()) 
				{
					log.debug("Creating schema " + actualCatalog + "/" + actualSchema + ": " + sql);
				}
				Query<?> q = session.createNativeQuery(sql);
				q.executeUpdate();
			}
			
			if (installMode != null)
			{
				// Generates tables and most other things
				SchemaGenerator schemaGenerator = new SchemaGenerator(metadata, dialect, dbEngine, installMode, null);
				session.doWork(schemaGenerator);

				// Update schema version in BASE
				Update.setSchemaVersion(session, config.getAppId(), config.getSchemaVersion(), 0);
				HibernateUtil.commit(tx);
				tx = null;
			}
			
			sf = metadata.buildSessionFactory();
			if (log.isDebugEnabled())
			{
				log.debug("Created session factory for schema: " + schema);
			}
		}
		catch (RuntimeException ex)
		{
			if (tx != null) HibernateUtil.rollback(tx);
			log.error("Could not build session factory for schema: " + schema, ex);
			throw ex;
		}
		catch (Exception ex)
		{
			if (tx != null) HibernateUtil.rollback(tx);
			log.error("Could not build session factory for schema: " + schema, ex);
			throw new BaseException(ex);
		}
		finally
		{
			if (session != null) HibernateUtil.close(session);
		}
		
		return sf;
	}
	
	/**
		Create a new Hibernate session for the given session factory that
		is piggy-backed on the given DbControl transaction. 
		
		Note that the life-cycle of the returned session is managed by the
		DbControl. There is no need to flush or close the session since this
		is done automatically and may cause exceptions if done when not needed.
		
		@param builder A session builder (eg. SessionFactory.withOptions())
		@param dc An open DbControl
	*/
	public Session newSession(SessionBuilder<?> builder, DbControl dc)
	{
		Session s = builder.connection(HibernateUtil.getConnection(dc.getHibernateSession())).openSession();
		s.beginTransaction();
		s.setHibernateFlushMode(FlushMode.COMMIT);
		dc.addTransactionalAction(new FlushSessionAction(s));
		return s;
	}
	
	/**
		Used for creating a configuration that is the basis for building a 
		session factory. The typical workflow is:
		
		1. Create an instance of this class by calling {@link ServiceSessionControl#createSessionFactoryConfiguration(String, String, int, ClassLoader)}
		2. Set configuration properties on {@link #getRegistryBuilder()}
		3. Add mapped entity classes on {@link #getMetadataSources()}
		4. Build the session factory: {@link ServiceSessionControl#buildSessionFactory(SessionFactoryConfiguration)}
		
		Store the created session factory in a service instance variable. When a
		database connection is needed use:
		
		1. {@link ServiceSessionControl#newSession(SessionBuilder, DbControl)}: for a transaction that is piggy-backed on
			a BASE {@link DbControl}
		2. {@link SessionFactory#openSession()}: for a standalone transaction.
		
		@since 3.8
	*/
	public class SessionFactoryConfiguration
	{
		private final String appId;
		private final String schema;
		private final int schemaVersion;
		private final ClassLoader classLoader;
		
		private StandardServiceRegistryBuilder registryBuilder;
		private MetadataSources metadataSources;
		
		private String actualCatalog;
		private String actualSchema;
		private SchemaGenerator.Mode installMode;
		private int installedSchemaVersion;
		private boolean useSeparateConnections;
		
		SessionFactoryConfiguration(String appId, String schema, int schemaVersion, ClassLoader classLoader)
		{
			this.appId = appId;
			this.schema = schema;
			this.schemaVersion = schemaVersion;
			this.classLoader = classLoader;
		}
		
		void init()
		{
			Session session = null;
			DbEngine dbEngine = HibernateUtil.getDbEngine();
			Dialect dialect = HibernateUtil.getDialect();
			try
			{
				session = HibernateUtil.newSession();
				
				// Pre-process catalog+schema so that it works for the actual database
				// Postgres leave it as it is...
				// but MySQL don't have schemas so actual catalog is a combined of both
				String catalog = HibernateUtil.doReturningWork(session, new GetCurrentCatalogWork());
				actualCatalog = dbEngine.getCatalogName(catalog, schema);
				actualSchema = dbEngine.getSchemaName(catalog, schema);
				if (log.isDebugEnabled())
				{
					log.debug("Current catalog: " + catalog);
					log.debug("Actual catalog/schema: " + actualCatalog + "/" + actualSchema);
				}
				
				// Check if the schema exists or not
				boolean schemaExists = HibernateUtil.doReturningWork(session, new SchemaExistsWork(actualCatalog, actualSchema));
				installMode = schemaExists ? SchemaGenerator.Mode.UPDATE : SchemaGenerator.Mode.INSTALL;
				if (log.isDebugEnabled()) log.debug("Mode: " +installMode);
				
				// Check which schema version that is installed
				if (schemaExists)
				{
					installedSchemaVersion = Update.getSchemaVersion(session, appId);
					if (installedSchemaVersion == schemaVersion)
					{
						// No update needed
						installMode = null;
					}
					else if (installedSchemaVersion > schemaVersion)
					{
						throw new ConfigurationException("Installed schema version for schema '" + schema + 
							"' is higher than expected: " + installedSchemaVersion + ">"+schemaVersion);
					}
				}
				
				// Create builder for registering configuration settings
				BootstrapServiceRegistryBuilder bootStrap = new BootstrapServiceRegistryBuilder();
				bootStrap.enableAutoClose();
				bootStrap.applyClassLoader(classLoader);
				registryBuilder = new StandardServiceRegistryBuilder(bootStrap.build());
			}
			catch (SQLException ex)
			{
				throw new DatabaseException(ex);
			}
			finally
			{
				if (session != null) HibernateUtil.close(session);
			}

		}
		
		/**
			Get the schema.
		*/
		public String getSchema()
		{
			return schema;
		}
		
		/**
			Get the application id for schema versioning.
		*/
		public String getAppId()
		{
			return appId;
		}
		
		/**
			Get the schema version of the current configuration.
		*/
		public int getSchemaVersion()
		{
			return schemaVersion;
		}
		
		/**
			Get the installation mode that BASE will use when creating the session
			factory. Returns null if no install/update is needed.
		*/
		public SchemaGenerator.Mode getInstallMode()
		{
			return installMode;
		}
		
		/**
			Get the currently installed version of the schema or
			0 if not installed.
		*/
		public int getInstalledSchemaVersion()
		{
			return installedSchemaVersion;
		}

		/**
			The actual schema that will be used by BASE.
		*/
		public String getActualSchema()
		{
			return actualSchema;
		}
		
		/**
			The actual catalog that will be used by BASE.
		*/
		public String getActualCatalog()
		{
			return actualCatalog;
		}
		
		/**
			Get the class loader used for accessing mapped classes.
		*/
		public ClassLoader getClassLoader()
		{
			return classLoader;
		}
		
		/**
			Set to TRUE to use connections separate from BASE connections. 
			If you enable this, you will probably also want to configure 
			connection pooling since the default connection pooling is
			not recommended for production use. Connection pooling
			is configured on {@link #getRegistryBuilder()} object. Basically
			you need to set the 'connection.provider_class' property 
			to 'org.hibernate.connection.C3P0ConnectionProvider' and
			some other properties. You'll need to look for this
			documentation in other places.
		*/
		public void setUseSeparateConnections(boolean separate)
		{
			checkLocked("setUseSeparateConnections");
			useSeparateConnections = separate;
		}

		/**
			Should the session factory be configured to use separate
			connections or is piggy-backed sessions required?
		*/
		public boolean useSeparateConnections()
		{
			return useSeparateConnections;
		}
		
		/**
			Check that the 
		*/
		private void checkLocked(String method)
		{
			if (metadataSources != null)
			{
				throw new ConfigurationException("Method " + method + "() can't be called after getMetadataSources()");
			}
		}
		
		/**
			Get the registry for collecting configuration options. The registry
			is used for configuration options that go into the hibernate.cfg.xml
			file. Note that BASE will automatially add configuration options for
			connecting to the database and the following options will be overwritten:
			<p>
			
			* hibernate.dialect
			* hibernate.connection_driver_class
			* hibernate.default_catalog
			* hibernate.default_schema
			* hibernate.connection.url
			* hibernate.connection.username
			* hibernate.connection.password
			<p>
			
			The registry must be fully configured before calling {@link #getMetadataSources()}. 
			This method	will throw an exception if called in the incorrect order. 
		*/
		public StandardServiceRegistryBuilder getRegistryBuilder()
		{
			checkLocked("getRegistryBuilder");
			return registryBuilder;
		}

		/**
			Get an object for registering mapped entity classes. Once this
			method has been called it is not possible to modify the registry builder
			or other configuration settings.
		*/
		public MetadataSources getMetadataSources()
		{
			if (metadataSources == null)
			{
				registryBuilder.applySetting("hibernate.dialect", Config.getString("db.dialect", ""));
				if (actualCatalog != null)
				{
					registryBuilder.applySetting("hibernate.default_catalog", "`"+actualCatalog+"`");
				}
				if (actualSchema != null)
				{
					registryBuilder.applySetting("hibernate.default_schema", "`"+actualSchema+"`");
				}
				
				if (useSeparateConnections())
				{
					registryBuilder.applySetting("hibernate.connection.url", Config.getString("db.url", ""));
					registryBuilder.applySetting("hibernate.connection.username", Config.getString("db.username", ""));
					registryBuilder.applySetting("hibernate.connection.password", Config.getString("db.password", ""));
				}
				
				metadataSources = new MetadataSources(registryBuilder.build());
			}
			return metadataSources;
		}

	}
	
	/**
		Our own keyring containing the only permission: {@link Permission#ACT_AS_ANOTHER_USER}.
		All other permissions are denied.
	*/
	static class ServicesKeyring
		extends Keyring
	{
	
		private final int systemPermissions;
		private final Set<Integer> emptySet;
		private final Map<Integer, Integer> emptyMap;
	
		private ServicesKeyring()
		{
			super(0);
			this.systemPermissions = Permission.grant(Permission.ACT_AS_ANOTHER_USER);
			this.emptySet = Collections.emptySet();
			this.emptyMap = Collections.emptyMap();
		}
		/*
			From the Keyring class
			-------------------------------------------
		*/
		/**
			Never reload since everything is static.
		*/
		@Override
		boolean getReload()
		{
			return false;
		}
		/**
			Return {@link Permission#ACT_AS_ANOTHER_USER} if <code>itemType</code> = 
			{@link Item#SYSTEM}, 0 otherwise. 
		*/
		@Override
		int getRolePermissions(Item itemType)
		{
			if (itemType == Item.SYSTEM)
			{
				return systemPermissions;
			}
			else
			{
				return 0;
			}
		}
		/**
			Return 0.
		*/
		@Override
		int getSharedPermissions(ShareableData sharableData)
		{
			return 0;
		}
		/**
			Return 0.
		*/
		@Override
		int getAllPermissions(Item itemType, UserData owner, ItemKeyData itemKey, ProjectKeyData projectKey)
		{
			return 0;
		}
		/**
			Return empty set.
		*/
		@Override
		Set<Integer> getRoles()
		{
			return emptySet;
		}
		/**
			Return empty set.
		*/
		Set<Integer> getGroups()
		{
			return emptySet;
		}
		/**
			Return empty set.
		*/
		@Override
		Set<Integer> getUsers()
		{
			return emptySet;
		}
		/**
			Return empty map.
		*/
		@Override
		Map<Integer, Integer> getProjects()
		{
			return emptyMap;
		}
		/**
			Return empty set.
		*/
		Set<Integer> getItemKeys()
		{
			return emptySet;
		}
		/**
			Return empty set.
		*/
		Set<Integer> getProjectKeys()
		{
			return emptySet;
		}
		// -------------------------------------------
	}

	/**
		Flushes the given session before the transaction is committed.
	*/
	static class FlushSessionAction
		implements TransactionalAction
	{
		
		private final Session session;
		
		FlushSessionAction(Session session)
		{
			this.session = session;
		}

		@Override
		public void onBeforeCommit() 
		{
			session.flush();
		}

		@Override
		public void onRollback() 
		{
			HibernateUtil.close(session);
		}
		
		@Override
		public void onAfterCommit() 
		{
			HibernateUtil.close(session);
		}
		
		
	}
	
}