/*
	$Id: StringParameterType.java 6881 2015-04-21 07:59:42Z nicklas $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Johan Enell, Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.ParameterValueData;
import net.sf.basedb.core.data.StringParameterValueData;
import net.sf.basedb.core.data.TextParameterValueData;
import net.sf.basedb.util.Enumeration;

import java.util.List;

/**
	This class represent a parameter type that is a string.
	This type defines a {@link #getMaxLength() max length}
	for the text.

	@author Samuel, Nicklas
	@version 2.0
	@base.modified $Date: 2015-04-21 09:59:42 +0200 (tis, 21 apr. 2015) $
*/
public class StringParameterType
	extends ParameterType<String>
{
	private final Integer maxLength;

	/**
		Create a new string parameter type, setting the max length to 255 and
		nulls are allowed.
	*/
	public StringParameterType()
	{
		this(255, null, false);
	}

	/**
		Create a new string parameter type.
		@param maxLength The maximum length of the string that is allowed or null
			to allow as long as possible
		@param defaultValue The default value of this parameter
		@param notNull FALSE if nulls values are allowed, TRUE otherwise
	*/
	public StringParameterType(Integer maxLength, String defaultValue, boolean notNull)
	{
		super(typeFromLength(maxLength), String.class, defaultValue, notNull);
		this.maxLength = maxLength;
	}
	
	public StringParameterType(Integer maxLength, String defaultValue, boolean notNull, int multiplicity, int width, int height)
	{
		super(typeFromLength(maxLength), String.class, defaultValue, notNull, multiplicity, width, height);
		this.maxLength = maxLength;
	}

	public StringParameterType(Integer maxLength, String defaultValue, boolean notNull, int multiplicity, int width, int height, List<String> items)
	{
		super(typeFromLength(maxLength), String.class, defaultValue, notNull, multiplicity, width, height, items);
		this.maxLength = maxLength;
	}
	
	public StringParameterType(Integer maxLength, String defaultValue, boolean notNull, int multiplicity, int width, int height, Enumeration<String, String> items)
	{
		this(maxLength, defaultValue, notNull, multiplicity, width, height, items, false);
	}
	
	public StringParameterType(Integer maxLength, String defaultValue, boolean notNull, int multiplicity, int width, int height, Enumeration<String, String> items, boolean checkValues)
	{
		super(typeFromLength(maxLength), String.class, defaultValue, notNull, multiplicity, width, height, items, checkValues);
		this.maxLength = maxLength;
	}

	/*
		From the Object class
		-------------------------------------------
	*/
	@Override
	public String toString()
	{
		return "StringParameterType: MaxLength " + maxLength;
	}
	// -------------------------------------------

	/*
		From the ParameterType class
		-------------------------------------------
	*/
	/**
		Checks if the value is a <code>String</code> object
		and shorter than the maximum length.
		@param value The value to test
		@throws InvalidDataException If the value is not a string or too long
	*/
	@Override
	void validateValue(String name, String value)
		throws InvalidDataException
	{
		if (maxLength != null && value.length() > maxLength)
		{
			throw new StringTooLongException(name, value, maxLength);
		}
	}
	/**
		Create a new {@link TextParameterValueData} object.
	*/
	@Override
	ParameterValueData<String> newParameterValueData()
	{
		if (maxLength == null || maxLength > 255)
		{
			return new TextParameterValueData();
		}
		else
		{
			return new StringParameterValueData();
		}
	}
	// -------------------------------------------

	/**
		Get the max length of the parameter.
		@return Max length or null if none is set.
	*/
	public Integer getMaxLength()
	{
		return maxLength;
	}
	
	private static Type typeFromLength(Integer maxLength)
	{
		return maxLength == null || maxLength > 255 ? Type.TEXT : Type.STRING;
	}
}
