/*
	$Id: UserPermissions.java 5368 2010-06-22 12:51:41Z nicklas $

	Copyright (C) 2005 Samuel Andersson, Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.core;

import net.sf.basedb.core.data.UserData;

import java.util.Collections;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;

/**
	This class is used to specify permissions to share an item to
	users. It is a three-step procedure:
	<ul>
	<li>Fill this object with permissions for specific users
	<li>Create a new {@link ItemKey} using this object as one of the
		input parameters to {@link ItemKey#getNewOrExisting(DbControl, UserPermissions, GroupPermissions)}
	<li>Assign the item key to the item: {@link Shareable#setItemKey(ItemKey)}
	</ul>

	Don't forget to commit the transaction.
	
	@see ItemKey
	@see GroupPermissions
	@see ProjectPermissions
	@author Nicklas
	@version 2.0
	@base.modified $Date: 2010-06-22 14:51:41 +0200 (tis, 22 juni 2010) $
*/
public class UserPermissions
{

	private final Map<UserData,Integer> userPermissions;
	private final Map<UserData,Integer> originalPermissions;

	/**
		Create a new empty object.
	*/
	public UserPermissions()
	{
		userPermissions = new HashMap<UserData,Integer>();
		originalPermissions = null;
	}
	
	/**
		Create a new object, and initialise it with the permissions from a
		<code>ItemKey</code>.
		@param itemKey The item key from which we take the current permissions
		@throws InvalidDataException If the item key is null
	*/
	public UserPermissions(ItemKey itemKey)
		throws InvalidDataException
	{
		if (itemKey == null) throw new InvalidUseOfNullException("itemKey");
		userPermissions = new HashMap<UserData,Integer>(itemKey.getData().getUsers());
		originalPermissions = new HashMap<UserData,Integer>(userPermissions);
	}

	/**
		Merge the permissions for all users from the given item key with the permissions 
		in this object. If the item key contains permissions for a user that already 
		has permissions in this object the permissions are combined (added) to create 
		a new permission.
		
		@param itemKey An item key (if null, this method simply returns)
		@since 2.16
	*/
	public void merge(ItemKey itemKey)
	{
		if (itemKey == null) return;
		for (Map.Entry<UserData, Integer> entry : itemKey.getData().getUsers().entrySet())
		{
			UserData user = entry.getKey();
			int permissions = IntegerUtil.getInt(entry.getValue());
			int current = IntegerUtil.getInt(userPermissions.get(user));
			userPermissions.put(user, current | permissions);
		}
	}
	
	/**
		Grant permissions to a user. Use an empty <code>Set</code> or null
		to remove all permissions for the user.

		@param user The <code>User</code>
		@param permissions The permissions to grant
		@throws InvalidDataException If the user is null
		@see Permission
	*/
	public void setPermissions(User user, Set<Permission> permissions)
		throws InvalidDataException
	{
		if (user == null) throw new InvalidUseOfNullException("user");
		if (permissions == null || permissions.isEmpty())
		{
			userPermissions.remove(user.getData());
		}
		else
		{
			userPermissions.put(user.getData(), Permission.grant(permissions));
		}
	}

	/**
		Grant permissions to a user, keeping those that have already been set.
		
		@param user The <code>User</code>
		@param permissions The permissions to grant
		@throws InvalidDataException If the user is null
		@see Permission
	*/
	public void addPermissions(User user, Set<Permission> permissions)
		throws InvalidDataException
	{
		if (user == null) throw new InvalidUseOfNullException("user");
		if (permissions != null && !permissions.isEmpty())
		{
			int current = IntegerUtil.getInt(userPermissions.get(user.getData()));
			userPermissions.put(user.getData(), current | Permission.grant(permissions));
		}
	}

	/**
		Get the permissions for a user.
		@param user The <code>User</code> for which we want to get the permission
		@return An <code>Set</code> containing the granted permissions, or an
			empty set if no permissions have been granted
		@throws InvalidDataException If the user is null
		@see #getOriginalPermissions(User)
		@see Permission
	*/
	public Set<Permission> getPermissions(User user)
		throws InvalidDataException
	{
		if (user == null) throw new InvalidUseOfNullException("user");
		return Permission.fromInt(IntegerUtil.getInt(userPermissions.get(user.getData())));
	}
	
	/**
		Get the original permissions for a user if this object was created with the
		{@link #UserPermissions(ItemKey)} constructor.
		@param user The <code>User</code> for which we want to get the permission
		@return An <code>Set</code> containing the original permissions, or an
			empty set
		@throws InvalidDataException If the user is null
		@see #getPermissions(User)
		@since 2.6
	*/
	public Set<Permission> getOriginalPermissions(User user)
	{
		if (user == null) throw new InvalidUseOfNullException("user");
		if (originalPermissions == null) return Collections.emptySet();
		return Permission.fromInt(IntegerUtil.getInt(originalPermissions.get(user.getData())));
	}
	
	/**
		Check if the current permissions for the user has been changed from the original
		permissions. This method will return false if the {@link #getPermissions(User)}
		returns a set with the same permissions {@link #getOriginalPermissions(User)}.
		@param user The user to check
		@return TRUE if the permissions have changed, FALSE otherwise
		@throws InvalidDataException If the user is null
		@since 2.6
	*/
	public boolean hasChanged(User user)
	{
		if (user == null) throw new InvalidUseOfNullException("user");
		int current = IntegerUtil.getInt(userPermissions.get(user.getData()));
		int original = originalPermissions == null ?
			0 : IntegerUtil.getInt(originalPermissions.get(user.getData()));
		return current != original;
	}
	
	/**
		Get the number of entries.
	*/
	public int size()
	{
		return userPermissions.size();
	}
	
	/**
		Get the internal representation of the permissions.
	*/
	final Map<UserData,Integer> getUserPermissions()
	{
		return userPermissions;
	}
	
	/**
		Get the internal representation of the original permissions.
		@since 2.6
	*/
	final Map<UserData,Integer> getOriginalPermissions()
	{
		return originalPermissions;
	}
}
