/*
  $Id: ContextData.java 6468 2014-06-10 11:27:02Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Map;
import java.util.HashMap;

/**
	This class holds context information to help client applications
	with storing common settings for tables.
	
	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.ItemContext
	@see <a href="../../../../../../html/developer/api/data_api.clients.html">Developer documentation: Client, session and settings</a>
	@base.modified $Date: 2014-06-10 13:27:02 +0200 (tis, 10 juni 2014) $
	@hibernate.class table="`Contexts`" lazy="true"
 */
public class ContextData
	extends BasicData
{
	public ContextData()
	{}

	private UserData user;
	/**
		Get the user that owns this context.
		@hibernate.many-to-one column="`user_id`" not-null="true" update="false" outer-join="false" unique-key="Context_uniquecontext"
	*/
	public UserData getUser()
	{
		return user;
	}
	public void setUser(UserData user)
	{
		this.user = user;
	}

	private ClientData client;
	/**
		Get the client which uses this context.
		@hibernate.many-to-one column="`client_id`" not-null="true" update="false" outer-join="false" unique-key="Context_uniquecontext"
	*/
	public ClientData getClient()
	{
		return client;
	}
	public void setClient(ClientData client)
	{
		this.client = client;
	}

	/**
		The maximum length of the name that can be stored in the database.
		@see #setName(String)
	*/
	public static final int MAX_NAME_LENGTH = 255;
	private String name;
	/**
		Get the name of this <code>Context</code> item.
		@hibernate.property column="`name`" type="string" length="255" not-null="true" update="true" unique-key="Context_uniquecontext"
	*/
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}

	private int itemType;
	/**
		Get the item type this context is valid for.
		@hibernate.property column="`item_type`" type="int" not-null="true" update="false" unique-key="Context_uniquecontext"
	*/
	public int getItemType()
	{
		return itemType;
	}
	public void setItemType(int itemType)
	{
		this.itemType = itemType;
	}
	
	/**
		The maximum length of the name that can be stored in the database.
		@see #setName(String)
	*/
	public static final int MAX_SUBCONTEXT_LENGTH = 255;
	private String subContext;
	/**
		Get the subcontext this context belongs to.
		@hibernate.property column="`subcontext`" type="string" length="255" not-null="true" update="true" unique-key="Context_uniquecontext"
	*/
	public String getSubContext()
	{
		return subContext;
	}
	public void setSubContext(String subContext)
	{
		this.subContext = subContext;
	}

	private boolean isPublic;
	/**
		If the settings are public (ie. can be used by anyone) or not.
		@hibernate.property column="`is_public`" type="boolean"
	*/
	public boolean isPublic()
	{
		return isPublic;
	}
	public void setPublic(boolean isPublic)
	{
		this.isPublic = isPublic;
	}
	
	private int rowsPerPage;
	/**
		Get the number of rows that are displayed on each table page.
		@hibernate.property column="`rows_per_page`" type="int" 
	*/
	public int getRowsPerPage()
	{
		return rowsPerPage;
	}
	public void setRowsPerPage(int rowsPerPage)
	{
		this.rowsPerPage = rowsPerPage;
	}

	private int itemId;
	/**
		Get the current item ID or 0 if no item is current.
		@hibernate.property column="`item_id`" type="int" 
	*/
	public int getItemId()
	{
		return itemId;
	}
	public void setItemId(int itemId)
	{
		this.itemId = itemId;
	}
	
	private int page;
	/**
		Get the current page number, starting at 0.
		@hibernate.property column="`page`" type="int" 
	*/
	public int getPage()
	{
		return page;
	}
	public void setPage(int page)
	{
		this.page = page;
	}

	/**
		The maximum length of the sort order that can be stored in the database.
		@see #setSortProperty(String)
	*/
	public static final int MAX_SORTPROPERTY_LENGTH = 255;
	private String sortProperty;
	/**
		Get property name to sort on.
		@hibernate.property column="`sort_property`" type="string" length="255" 
	*/
	public String getSortProperty()
	{
		return sortProperty;
	}
	public void setSortProperty(String sortProperty)
	{
		this.sortProperty = sortProperty;
	}

	private int sortDirection;
	/**
		Get the sort direction, 0 = ascending, 1 = descending
		@hibernate.property column="`sort_direction`" type="int" 
	*/
	public int getSortDirection()
	{
		return sortDirection;
	}
	public void setSortDirection(int sortDirection)
	{
		this.sortDirection = sortDirection;
	}

	private int include;
	/**
		Flags for specifying include options to an entity query.
		@hibernate.property column="`include`" type="int" 
	*/
	public int getInclude()
	{
		return include;
	}
	public void setInclude(int include)
	{
		this.include = include;
	}

	private Map<String, PropertyFilterData> propertyFilters;
	/**
		Get the map that manages property filters in this context.

		@hibernate.map table="`PropertyFilters`" lazy="true"
		@hibernate.collection-key column="`context_id`"
		@hibernate.collection-index column="`property`" type="string" length="255" 
		@hibernate.collection-composite-element class="net.sf.basedb.core.data.PropertyFilterData" not-null="true"
	*/
	public Map<String, PropertyFilterData> getPropertyFilters()
	{
		if (propertyFilters == null) propertyFilters = new HashMap<String, PropertyFilterData>();
		return propertyFilters;
	}
	void setPropertyFilters(Map<String, PropertyFilterData> propertyFilters)
	{
		this.propertyFilters = propertyFilters;
	}

	private Map<String, String> settings;
	/**
		Get a map for other settings.
		
		@hibernate.map table="`ContextSettings`" lazy="true"
		@hibernate.collection-key column="`context_id`"
		@hibernate.collection-index column="`name`" type="string" length="255"
		@hibernate.collection-element column="`value`" type="text"
	*/
	public Map<String, String> getSettings()
	{
		if (settings == null) settings = new HashMap<String, String>();
		return settings;
	}
	void setSettings(Map<String, String> settings)
	{
		this.settings = settings;
	}

	
}
