/*
 $Id: RawData.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.basedb.core.data;

import java.util.Map;
import java.util.HashMap;

/**
	Represents the measurements for single raw data spot. A raw data entry consists 
	of two parts, coordinate/reporter information and measured values (intensities, etc.)
	What kind of measurements are available are defined by
	the {@link net.sf.basedb.core.RawDataType} of the raw bioassay.
	
	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.RawDataBatcher
	@see net.sf.basedb.core.RawDataUtil
	@see <a href="../../../../../../html/developer/api/data_api.bioassays.html">Developer documentation: Bioassays and raw data</a>
	@base.modified $Date: 2012-09-14 09:28:35 +0200 (fre, 14 sep. 2012) $
*/
public class RawData 
	extends BasicData
	implements BatchableData, ExtendableData
{

	public RawData()
	{}

	public RawData(RawBioAssayData rawBioAssay)
	{
		this.rawBioAssay = rawBioAssay;
	}

	/*
		From the ExtendableData interface
		------------------------------------------
	*/
	private Map<String, Object> extendedProperties;
	@Override
	public Object getExtended(String name)
	{
		return extendedProperties == null ? null : extendedProperties.get(name);
	}
	@Override
	public void setExtended(String name, Object value)
	{
		if (extendedProperties == null) extendedProperties = new HashMap<String, Object>();
		extendedProperties.put(name, value);
	}
	// -------------------------------------------

	private RawBioAssayData rawBioAssay;
	/**
		The raw bioassay this raw data spot belongs to. Cannot be public since we don't
		want to expose data objects to clients.
	*/
	RawBioAssayData getRawBioAssay()
	{
		return rawBioAssay;
	}
	void setRawBioAssay(RawBioAssayData rawBioAssay)
	{
		this.rawBioAssay = rawBioAssay;
	}

	private FeatureData feature;
	/**
		The feature this raw data is connected with.
	*/
	public FeatureData getFeature()
	{
		return feature;
	}
	void setFeature(FeatureData feature)
	{
		this.feature = feature;
	}

	private ReporterData reporter;
	/**
		The reporter this raw data is connected with. If a feature is also connected
		the reporter must be the same as the feature's reporter.
	*/
	public ReporterData getReporter()
	{
		return reporter;
	}
	public void setReporter(ReporterData reporter)
	{
		this.reporter = reporter;
	}
	
	private int position;
	public int getPosition()
	{
		return position;
	}
	/**
		This method was private before 2.6. If the raw bioassay is connected to
		an array design which uses the POSITION feature identifier method
		a value must be set. If the raw bioassay is connected to an array design
		which uses another feature identifier method this value will be overwritten
		with the position value from the feature. If the raw bioassay is not connected
		to an array design a position will be automatically generated if no value
		has been set, ie. if the value is 0.
		The value must be a positive integer.
		@param position The position number of the spot
		@since 2.6
	*/
	public void setPosition(int position)
	{
		this.position = position;
	}

	private Float x;
	public Float getX()
	{
		return x;
	}
	public void setX(Float x)
	{
		this.x = x;
	}

	private Float y;
	public Float getY()
	{
		return y;
	}
	public void setY(Float y)
	{
		this.y = y;
	}

	private Integer block;
	public Integer getBlock()
	{
		return block;
	}
	public void setBlock(Integer block)
	{
		this.block = block;
	}

	private Integer metaGridX;
	public Integer getMetaGridX()
	{
		return metaGridX;
	}
	public void setMetaGridX(Integer metaGridX)
	{
		this.metaGridX = metaGridX;
	}

	private Integer metaGridY;
	public Integer getMetaGridY()
	{
		return metaGridY;
	}
	public void setMetaGridY(Integer metaGridY)
	{
		this.metaGridY = metaGridY;
	}

	private Integer row;
	public Integer getRow()
	{
		return row;
	}
	public void setRow(Integer row)
	{
		this.row = row;
	}

	private Integer column;
	public Integer getColumn()
	{
		return column;
	}
	public void setColumn(Integer column)
	{
		this.column = column;
	}

	public Map<String, Object> getAllExtended()
	{
		if (extendedProperties == null) extendedProperties = new HashMap<String, Object>();
		return extendedProperties;
	}
	
}
