/**
	$Id: ExecuteUpdateWork.java 6468 2014-06-10 11:27:02Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.hibernate;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import org.hibernate.jdbc.ReturningWork;

import net.sf.basedb.core.QueryExecutor;
import net.sf.basedb.core.SessionControl;
import net.sf.basedb.core.signal.SignalException;

/**
	A JDBC work implementation that executes an update/insert/delete 
	query and returns the number of rows that was affected by the query.
	By subclassing this class and overridding the {@link #setParameters(PreparedStatement)}
	method it is possible to set parameters on the prepared statement before 
	the SQL is executed.

	@author nicklas
	@version 2.9
	@base.modified $Date: 2014-06-10 13:27:02 +0200 (tis, 10 juni 2014) $
	@see PreparedStatement#executeUpdate()
*/
public class ExecuteUpdateWork
	implements ReturningWork<Integer>
{

	private SessionControl sc;
	private String sql;
		
	/**
		Create a new work item. 
		@param sc The session control to keep alive during the execution
			of the sql, or null
		@param sql The sql to execute
	 */
	public ExecuteUpdateWork(SessionControl sc, String sql)
	{
		this.sc = sc;
		this.sql = sql;
	}
	
	/**
		From the Work interface
		-----------------------
	*/
	@Override
	public Integer execute(Connection connection) 
		throws SQLException
	{
		PreparedStatement ps = null;
		int affectedRows = 0;
		try
		{
			ps = connection.prepareStatement(sql);
			setParameters(ps);
			affectedRows = QueryExecutor.executeUpdate(ps, sc); 
		}
		catch (InterruptedException ex)
		{
			throw new SignalException("Aborted by user", ex);
		}
		finally
		{
			try
			{
				if (ps != null) ps.close();
			}
			catch (Throwable t)
			{}
		}
		return affectedRows;
	}
	// --------------------------
	
	/**
		This method should be overridden by subclasses that needs to
		set parameters on the statement that is going to be executed.
		The default implementation does nothing. This method is called
		after the statement has been prepared with 
		{@link Connection#prepareStatement(String)} and before it is executed
		with {@link PreparedStatement#executeUpdate()}.
		@param ps The prepared statement
	*/
	protected void setParameters(PreparedStatement ps)
		throws SQLException
	{}

}
