/**
	$Id$

	Copyright (C) 2014 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.hibernate;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.hibernate.jdbc.ReturningWork;

/**
	A work implementation that checks if a given catalog+schema exists 
	in the database or not. There are two variants: if a schema is not
	given, the check is only if the catalog exists or not.

	@author nicklas
	@since 3.4
*/
public class SchemaExistsWork
	implements ReturningWork<Boolean>
{
	private final String catalog;
	private final String schema;
	
	/**
		Create a new work item.
		@param catalog The catalog the schema is located in (required)
		@param schema The schema to check or null to only check if the catalog exists
	*/
	public SchemaExistsWork(String catalog, String schema)
	{
		this.catalog = catalog;
		this.schema = schema;
	}
	
	/**
		From the Work interface
		-----------------------
	*/
	@Override
	public Boolean execute(Connection connection) 
		throws SQLException
	{
		ResultSet tables = null;
		boolean hasTable = false;
		try
		{
			DatabaseMetaData metaData = connection.getMetaData();
			if (schema == null)
			{
				// Check only catalog
				tables = metaData.getCatalogs();
				while (tables.next())
				{
					if (catalog.equals(tables.getString(1)))
					{
						hasTable = true;
						break;
					}
				}
			}
			else
			{
				tables = metaData.getSchemas(catalog, schema);
				hasTable = tables.next();
			}
		}
		finally
		{
			try
			{
				if (tables != null) tables.close();
			}
			catch (Throwable t)
			{}
		}
		return hasTable;
	}
	// --------------------------
	

}
