/**
	$Id: EntityDetails.java 7642 2019-03-12 06:53:27Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.log;


import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map;

import net.sf.basedb.core.Item;
import net.sf.basedb.core.data.BasicData;
import net.sf.basedb.core.data.BioWellData;
import net.sf.basedb.core.data.ChangeHistoryDetailData;
import net.sf.basedb.core.data.NameableData;
import net.sf.basedb.util.Coordinate;
import net.sf.basedb.util.formatter.DateFormatter;

import org.hibernate.collection.spi.PersistentCollection;
import org.hibernate.type.ComponentType;
import org.hibernate.type.TimestampType;
import org.hibernate.type.Type;

/**
	Contains detailed information about the changes made to an
	entity during a transaction. This information is sent to
	an {@link EntityLogger} that is responsible for the actual
	logging.
	<p>
	NOTE! The information in this object is more or less drawn from
	what Hibernate reveals about the entity. It is recommended that
	no information is changed.

	@author Nicklas
	@version 2.13
	@base.modified $Date: 2019-03-12 07:53:27 +0100 (tis, 12 mars 2019) $
*/
public class EntityDetails
{

	public static final DateFormatter DATE_FORMAT = new DateFormatter("yyyy-MM-dd");
	public static final DateFormatter DATETIME_FORMAT = new DateFormatter("yyyy-MM-dd hh:mm:ss");
	
	private final Object entity;
	private final ChangeType changeType;
	private final Object[] state;
	private final Object[] previousState;
	private final String[] properties;
	private final Type[] types;
	private final int[] dirty;
	
	public EntityDetails(Object entity, ChangeType changeType,
			Object[] state, Object[] previousState, String[] properties, Type[] types)
	{
		this.entity = entity;
		this.changeType = changeType;
		this.state = state;
		this.previousState = previousState;
		this.properties = properties;
		this.types = types;
		this.dirty = new int[types.length]; // 0 = not known, -1 = not dirty, 1 = dirty
		initializeDirtynessForSomeProperties();
	}
		
	/**
		Get the entity that was changed.
	*/
	public Object getEntity()
	{
		return entity;
	}
	
	/**
		Was it a creation/update/deletion?
	*/
	public ChangeType getChangeType()
	{
		return changeType;
	}
	
	/**
		Get an array with the current state of the entity.
	*/
	public Object[] getState()
	{
		return state;
	}
	
	/**
	 	Get an array with the previous state of the entity. The
	 	array is null for new items and is identical to the array
	 	returned by {@link #getState()} for delete events.
	*/
	public Object[] getPreviousState()
	{
		return previousState;
	}

	/**
		Get an array with the names of all properties.
	*/
	public String[] getProperties()
	{
		return properties;
	}
	
	/**
		Get an array representing the data type for each property.
	*/
	public Type[] getTypes()
	{
		return types;
	}
	
	public int getDirty(int index)
	{
		if (changeType != ChangeType.UPDATE) return -1;
		if (dirty[index] == 0)
		{
			Type t = types[index];
			Object c = state[index];
			Object o = previousState[index];
			dirty[index] = t.isEqual(c, o) ? -1 : 1;
		}
		return dirty[index];
	}
	
	/**
		Get the array index of the given property. Return -1 if not found.
		@since 3.3
	*/
	public int getIndex(String property)
	{
		for (int i = 0; i < properties.length; ++i)
		{
			if (properties[i].equals(property)) return i;
		}
		return -1;
	}
	
	/**
		Set the dirtyness for the property with the given index.
		@param index The index number of the property
		@param value The dirtyness (-1=not dirty, 0=not known, 1=dirty)
		@since 3.3
	*/
	public void setDirty(int index, int value)
	{
		dirty[index] = value;
	}
	
	/**
		Get all modified properties as a string, separating each
		property name with the given separator.
		@param prefix A string that is added first in the result, or null
		@param separator A separator string (if null, comma is used)
		@param suffix A string that is added last in the result, or null
		@return A string with the modified properties, or null
			if this is a CREATE or DELETE change or if no modified 
			properties was found
	*/
	public String getModifiedProperties(String prefix, String separator, String suffix)
	{
		if (changeType != ChangeType.UPDATE) return null;
		
		StringBuilder modified = new StringBuilder();
		if (prefix != null) modified.append(prefix);
		if (separator == null) separator = ",";
		String s = "";
		int numModified = 0;
		for (int i = 0; i < types.length; ++i)
		{
			if (getDirty(i) == 1)
			{
				modified.append(s).append(properties[i]);
				s = separator;
				numModified++;
			}
		}
		if (suffix != null) modified.append(suffix);
		return numModified == 0 ? null : modified.toString();
	}
	
	/**
		Checks if the given property is the only modified property. 
		@return TRUE if the given property (but no other) is dirty, FALSE in
			all other cases
		@since 3.2.3
	*/
	public boolean isOnlyDirty(String property)
	{
		boolean propertyIsDirty = false;
		for (int i = 0; i < types.length; ++i)
		{
			if (getDirty(i) == 1)
			{
				if (property.equals(properties[i])) 
				{
					propertyIsDirty = true;
				}
				else
				{
					// Found another dirty property
					return false;
				}
			}
		}
		return propertyIsDirty;
	}
	
	/**
		Create a list of ChangeHistoryDetailData objects from the information
		in this object assuming that the entity is a subclass of {@link BasicData}.
		
		@param detailedProperties If TRUE, one log entry is generated for each modified 
			property (UPDATE only), otherwise only a single log entry is created
		@param oldPropertyValues If TRUE, the log entry will save the old and new
			value (ignored unless detailedProperties is also set)
			
		@return A list of ChangeHistoryDetailData object (may be empty)
		@since 3.3
	*/
	public List<ChangeHistoryDetailData> toChangeDetails(boolean detailedProperties, boolean oldPropertyValues)
	{
		BasicData data = (BasicData)entity;
		return toChangeDetails(Item.fromDataObject(data), data.getId(), null, changeType, detailedProperties, oldPropertyValues);
	}
	
	/**
		Create a list of ChangeHistoryDetailData objects using a mix of the information
		in this object, and the given parameters. This method can be used to attach
		log entries to a different item, than what the changes in this EntityDetails
		object are associated with. Typically used to record changes in a child object
		on the parent object.
	
		@param itemType The type of the item the log entries should be attached to
		@param itemId The id of the item the log entries should be attached
		@param propertyPrefix A prefix that is added to the property description to indicate what child object
			that was modified
		@param changeType The change type
		@param detailedProperties If TRUE, the {@link ChangeHistoryDetailData#getChangeInfo()}
			is populated with a list of the names of the modified properties (only
			relevant for UPDATE changes)	
		@param oldPropertyValues If TRUE, the log entry will save the old and new
			value (ignored unless detailedProperties is also set)
		
		@return A list of ChangeHistoryDetailData object (may be empty)
		@since 3.3
	*/
	public List<ChangeHistoryDetailData> toChangeDetails(Item itemType, int itemId, String propertyPrefix, 
		ChangeType changeType, boolean detailedProperties, boolean oldPropertyValues)
	{
		List<ChangeHistoryDetailData> changes = new ArrayList<ChangeHistoryDetailData>();

		int changeValue = changeType.getValue();
		int itemValue = itemType.getValue();
		boolean createSingleLog = true;
		
		if (changeType == ChangeType.UPDATE)
		{
			// We need to check if there are any dirty properties before we know if something should be logged
			createSingleLog = false;
			for (int i = 0; i < types.length; ++i)
			{
				if (getDirty(i) == 1)
				{
					if (detailedProperties)
					{
						if (types[i].isComponentType())
						{
							Object newComponent = state[i];
							Object oldComponent = previousState[i];
							
							// Create one entry for each modified part of the component
							ComponentType ct = (ComponentType)types[i];
							Type[] ctypes = ct.getSubtypes();
							String[] cnames = ct.getPropertyNames();
							
							for (int ci = 0; ci < ctypes.length; ci++)
							{
								Type ctype = ctypes[ci];
								Object newValue = ct.getPropertyValue(newComponent, ci);
								Object oldValue = ct.getPropertyValue(oldComponent, ci);
								
								if (!ctype.isEqual(newValue, oldValue))
								{
									ChangeHistoryDetailData details = new ChangeHistoryDetailData();
									details.setChangeType(changeValue);
									details.setItemId(itemId);
									details.setItemType(itemValue);
									details.setChangeInfo((propertyPrefix == null ? properties[i] : propertyPrefix + "." + properties[i])+"."+cnames[ci]);
									if (oldPropertyValues)
									{
										details.setNewValue(toLoggableString(ctype, newValue));
										details.setOldValue(toLoggableString(ctype, oldValue));
									}
									changes.add(details);
								}
							}
						}
						else
						{
							ChangeHistoryDetailData details = new ChangeHistoryDetailData();
							details.setChangeType(changeValue);
							details.setItemId(itemId);
							details.setItemType(itemValue);
							details.setChangeInfo(propertyPrefix == null ? properties[i] : propertyPrefix + "." + properties[i]);
							if (oldPropertyValues)
							{
								details.setNewValue(toLoggableString(types[i], state[i]));
								details.setOldValue(toLoggableString(types[i], previousState[i]));
							}
							changes.add(details);
						}
					}
					else
					{
						// No need to check any further, break out of the loop
						createSingleLog = true;
						break;
					}
				}
			}
		}
		
		if (createSingleLog)
		{
			ChangeHistoryDetailData details = new ChangeHistoryDetailData();
			details.setChangeType(changeValue);
			details.setItemId(itemId);
			details.setItemType(itemValue);
			details.setChangeInfo(propertyPrefix);
			changes.add(details);
		}
		
		return changes;
	}
	
	/**
		We must initialize "dirtyness" for collections and some other
		properties from the constructor. The "dirtyness" of a
		collection is reset after flush the SQL to the database, which is 
		completed before the rest of the logging is done.
	*/
	private void initializeDirtynessForSomeProperties()
	{
		EntityLogOptions logOptions = EntityLogOptions.getEntityLogOptions(entity);
		
		for (int i = 0; i < types.length; ++i)
		{
			if (logOptions != null && logOptions.isNotLoggable(properties[i])) 
			{
				dirty[i] = -1;
			}
			else
			{
				Type t = types[i];
				if (t.isCollectionType())
				{
					Object c = state[i];
					if (c instanceof PersistentCollection)
					{
						dirty[i] = ((PersistentCollection)c).isDirty() ? 1 : -1;
					}
					else
					{
						dirty[i] = -1;
					}
				}
			}
		}
	}
	
	
	public static String toLoggableString(Type type, Object value)
	{
		if (value == null) return null;
		
		String log = null;
		
		if (value instanceof String || value instanceof Number)
		{
			// Strings and numbers are common and easy
			log = value.toString();
		}
		else if (value instanceof Date)
		{
			// Dates may or may not have a time-part
			if (type instanceof TimestampType)
			{
				log = DATETIME_FORMAT.format((Date)value);
			}
			else
			{
				log = DATE_FORMAT.format((Date)value);
			}
		}
		else if (value instanceof Boolean)
		{
			log = value.toString();
		}
		else if (value instanceof NameableData)
		{
			// Entity references are logged with class[name]
			log = getDataClassName(value) + "[" +  ((NameableData)value).getName() + "]";
		}
		else if (value instanceof BioWellData)
		{
			// Get coordinate on the bioplate
			BioWellData well = (BioWellData)value;
			log = Coordinate.numericToAlpha(well.getRow()+1) + (well.getColumn()+1);
		}
		else if (value instanceof Collection || value instanceof Map)
		{
			// Collections have already been updated and the previousState
			// is actually the same as the currentState
			log = null;
		}
		else
		{
			// Default
			log = value.toString();
		}
		return log;
	}
	
	
	private static String getDataClassName(Object value)
	{
		Class<?> valueClass;
		if (value instanceof org.hibernate.proxy.HibernateProxy)
		{
			valueClass = org.hibernate.proxy.HibernateProxyHelper.getClassWithoutInitializingProxy(value);
		}
		else
		{
			valueClass = value.getClass();			
		}
		return valueClass.getSimpleName().replace("Data", "");
	}
}
