/**
	$Id: LogManagerFactory.java 6079 2012-08-06 13:20:43Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.log;

import net.sf.basedb.core.LogControl;
import net.sf.basedb.util.extensions.Action;

/**
	A log manager factory handles the creation of log managers and
	entity loggers. There is usually only one factory instance created
	for the entire BASE application. Implementations must be thread safe since
	the factory is used by multiple treads at the same time.
	<p>
	Each transaction that results in updates to the database are
	intercepted by the {@link LoggingInterceptor} which calls
	{@link #getLogManager(LogControl)} to create a new log
	manager for the transaction. 
	<p>
	For each created, modified or deleted item in the transaction,
	{@link #getEntityLogger(LogManager, Object)} is called. If this method 
	returns an object, {@link EntityLogger#logChanges(LogManager, EntityDetails)}
	is called and the rest is up to the actual implementation. 

	@author Nicklas
	@version 2.13
	@base.modified $Date: 2012-08-06 15:20:43 +0200 (mån, 06 aug. 2012) $
*/
public interface LogManagerFactory
	extends Action
{
	
	/**
		Get/create a log manager to log events in a given transaction.
		The log manager is used for this transaction only and is only used
		by a single thread in the core. The factory may re-use the same
		log manager in other (concurrent) transactions if the log manager 
		implementation is thread safe.
		@param logControl A log controller with information about the
			current transaction 
		@return A log manager object
	*/
	public LogManager getLogManager(LogControl logControl);

	/**
		Checks if changes to the given entity should be logged or not.
		If this method returns true, then {@link #getEntityLogger(LogManager, Object)}
		mustn't return null if called with the same entity parameter.
		@param entity The entity to check
		@return TRUE if the changes to the entity should be logged, FALSE
			if not
	*/
	public boolean isLoggable(Object entity);
	
	/**
		Get/create an entity logger for logging changes to the given entity.
		This method is called for each modified entity and is used within
		a single transaction only. The factory may re-use the same entity
		loggers in other transactions if the implementation is thread safe.
		
		@param logManager The log manager used in the current transaction
		@param entity The entity to get a logger for
		@return An entity logger, or null if changes to the entity shouldn't be
			logged
	*/
	public EntityLogger getEntityLogger(LogManager logManager, Object entity);
	
	
}
