/*
	$Id: Annotations.java 7643 2019-03-12 07:29:06Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import net.sf.basedb.core.InvalidUseOfNullException;
import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.InvalidDataException;

/**
	A factory class to create restrictions based on annotation values.

	@author Nicklas
	@version 2.0
*/
public class Annotations
{

	/**
		Create an expression representing the value of an annotation to the root item
		of the query. This method works best with single-valued annotation types. It
		can generate duplicate result rows for the same entity for multi-valued annotations.
		@param annotationType The annotation type to generate an expression for
		@return An expression
		@throws InvalidDataException If the annotationType is null
		@since 3.2
	*/
	public static Expression expression(AnnotationType annotationType)
	{
		return expression(null, annotationType);
	}
	
	/**
		Create an expression representing the value of an annotation to a joined item
		of the query. This method works best with single-valued annotation types. It
		can generate duplicate result rows for the same entity for multi-valued annotations.
		@param alias The alias of a joined item or null to use the root item
		@param annotationType The annotation type to generate an expression for
		@return An expression
		@throws InvalidDataException If the annotationType is null
		@since 3.2
	*/
	public static Expression expression(String alias, AnnotationType annotationType)
	{
		if (annotationType == null) throw new InvalidUseOfNullException("annotationType");
		return new AnnotationExpression(alias, annotationType);
	}
	
	/**
		Create an inner join between the root element of a query and
		the annotation values for the given annotation type.
		
		@param annotationType The annotation type to to join
		@param joinedAlias The alias to use for the joined annotation values
		@return A join query element
		@throws InvalidDataException If the annotationType or joinedAlias is null
		@since 3.2
	*/
	public static Join innerJoin(AnnotationType annotationType, String joinedAlias)
	{
		return innerJoin(null, annotationType, joinedAlias);
	}
	
	/**
		Create an inner join between an already joined element of a query and
		the annotation values for the given annotation type.
		
		@param alias The alias of a joined item or null to use the root item
		@param annotationType The annotation type to to join
		@param joinedAlias The alias to use for the joined annotation values
		@return A join query element
		@throws InvalidDataException If the annotationType or joinedAlias is null
		@since 3.2
	*/
	public static Join innerJoin(String alias, AnnotationType annotationType, String joinedAlias)
	{
		if (annotationType == null) throw new InvalidUseOfNullException("annotationType");
		if (joinedAlias == null) throw new InvalidUseOfNullException("joinedAlias");
		return new AnnotationJoin(JoinType.INNER, alias, annotationType, joinedAlias);
	}
	
	/**
		Create a left join between the root element of a query and
		the annotation values for the given annotation type.
		
		@param annotationType The annotation type to to join
		@param joinedAlias The alias to use for the joined annotation values
		@return A join query element
		@throws InvalidDataException If the annotationType or joinedAlias is null
		@since 3.3
	*/
	public static Join leftJoin(AnnotationType annotationType, String joinedAlias)
	{
		return leftJoin(null, annotationType, joinedAlias);
	}
	
	/**
		Create a left join between an already joined element of a query and
		the annotation values for the given annotation type.
		
		@param alias The alias of a joined item or null to use the root item
		@param annotationType The annotation type to to join
		@param joinedAlias The alias to use for the joined annotation values
		@return A join query element
		@throws InvalidDataException If the annotationType or joinedAlias is null
		@since 3.2
	*/
	public static Join leftJoin(String alias, AnnotationType annotationType, String joinedAlias)
	{
		if (annotationType == null) throw new InvalidUseOfNullException("annotationType");
		if (joinedAlias == null) throw new InvalidUseOfNullException("joinedAlias");
		return new AnnotationJoin(JoinType.LEFT, alias, annotationType, joinedAlias);
	}
	
}
