/*
	$Id: ReporterListExpression.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.Collection;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.HibernateUtil;
import net.sf.basedb.core.RealTable;
import net.sf.basedb.core.ReporterList;
import net.sf.basedb.core.query.Expression;
import net.sf.basedb.core.query.Query;
import net.sf.basedb.core.query.QueryElement;
import net.sf.basedb.core.query.QueryType;
import net.sf.basedb.util.EqualsHelper;

/**
	Represents a property of a reporter list in a dynamic query:
	<code>alias.property</code>.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2012-09-14 09:28:35 +0200 (fre, 14 sep. 2012) $
*/
public class ReporterListExpression
	implements Expression
{
	
	private final boolean hqlMode;
	private final ReporterList reporterList;
	private final String tableAlias;
	private String tableColumn;
	private String joinAlias;
	private String joinProperty;
	private String property;
	private final JoinType autoJoinType;
	
	/**
		Create a new expression for the specified reporter list.
		
		@param reporterList The reporter list
		@param tableColumn The name of the column we should select
		@param autoJoinType If an automatic join is done use this type, the join
			type depends on if we are checking if a reporter is part of or not part
			of a reporter list
		@see net.sf.basedb.core.query.Dynamic#isPartOf(ReporterList)
		@see net.sf.basedb.core.query.Dynamic#isNotPartOf(ReporterList)
		@see net.sf.basedb.core.query.Dynamic#score(ReporterList)
	*/
	ReporterListExpression(ReporterList reporterList, String tableColumn, JoinType autoJoinType)
	{
		this.hqlMode = false;
		this.reporterList = reporterList;
		this.tableAlias = RealTable.REPORTER_LIST_SCORES.getAlias(reporterList);
		this.tableColumn = tableColumn;
		this.autoJoinType = autoJoinType;
	}
	
	ReporterListExpression(ReporterList reporterList, String joinAlias, String joinProperty, String property, JoinType autoJoinType)
	{
		this.hqlMode = true;
		this.reporterList = reporterList;
		this.tableAlias = RealTable.REPORTER_LIST_SCORES.getAlias(reporterList);
		this.joinAlias = joinAlias;
		this.joinProperty = joinProperty;
		this.property = property;
		this.autoJoinType = autoJoinType;
	}
	
	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	@Override
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		if (query.getQueryType() == QueryType.SQL && !hqlMode)
		{
			String alias = HibernateUtil.quote(tableAlias);
			return alias + "." + HibernateUtil.quote(tableColumn);
		}
		else if (query.getQueryType() == QueryType.HQL && hqlMode)
		{
			return tableAlias + "." + property.replace("#", ".");
		}
		else
		{
			throw new UnsupportedOperationException("ReporterListExpression operation not supported by "+query.getQueryType()+" query.");
		}
	}
	/**
		@return Always null
	*/
	@Override
	public Collection<? extends QueryElement> getChildren()
	{
		return null;
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	@Override
	public String toString()
	{
		if (!hqlMode)
		{
			return tableAlias + "." + tableColumn;
		}
		else
		{
			return tableAlias + "." + property;
		}
	}
	/**
		This expression is equal to another ReporterListExpression if they 
		use the same property from the same reporter list and uses the same auto
		join type.
	*/
	@Override
	public boolean equals(Object other)
	{
		if (this == other) return true;
		if (other == null || this.getClass() != other.getClass()) return false;
		ReporterListExpression o = (ReporterListExpression)other;
		if (!EqualsHelper.equals(reporterList, o.reporterList) || autoJoinType != o.autoJoinType) return false;
		if (hqlMode)
		{
			return EqualsHelper.equals(joinAlias, o.joinAlias) && 
				EqualsHelper.equals(joinProperty, o.joinProperty) &&
				EqualsHelper.equals(property, o.property);
		}
		else
		{
			return EqualsHelper.equals(tableColumn, o.tableColumn);
		}
	}
	@Override
	public int hashCode()
	{
		int hashCode = EqualsHelper.hashCode(reporterList, autoJoinType);
		if (hqlMode)
		{
				hashCode += EqualsHelper.hashCode(joinAlias, joinProperty, property);
		}
		else
		{
			hashCode += EqualsHelper.hashCode(tableColumn);
		}
		return hashCode;
	}
	// -------------------------------------------

	/**
		Get the reporter list.
	*/
	public ReporterList getReporterList()
	{
		return reporterList;
	}
	
	public String getJoinAlias()
	{
		return joinAlias;
	}
	
	public String getJoinProperty()
	{
		return joinProperty;
	}
	
	/**
		Get the join type for automatic joins. 
		@return A join type or null if the default should be used
	*/
	public JoinType getAutoJoinType()
	{
		return autoJoinType;
	}
	
}
