/**
	$Id: SharedToRestriction.java 7564 2019-01-08 07:18:21Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Shareable;
import net.sf.basedb.util.EqualsHelper;

/**
	A special restriction that works on {@link Shareable} items. 
	The restriction is used to find items that have or have not
	been shared to users/groups and/or projects that meet some
	specific criteria.
	
	@see Hql#sharedTo(boolean, Restriction, Restriction, Restriction)
	@author Nicklas
	@version 2.15
	@base.modified $Date: 2019-01-08 08:18:21 +0100 (tis, 08 jan. 2019) $
*/
public class SharedToRestriction
	implements Restriction
{

	private boolean sharedTo;
	private Restriction users;
	private Restriction groups;
	private Restriction projects;
	
	SharedToRestriction(boolean sharedTo, Restriction users, Restriction groups, Restriction projects)
	{
		this.sharedTo = sharedTo;
		this.users = users;
		this.groups = groups;
		this.projects = projects;
	}
	
	@Override
	public Collection<? extends QueryElement> getChildren()
	{
		List<Restriction> children = new ArrayList<Restriction>();
		if (users != null) children.add(users);
		if (groups != null) children.add(groups);
		if (projects != null) children.add(projects);
		return children.size() == 0 ? null : children;
	}

	@Override
	public String toQl(Query query, DbControl dc) 
		throws BaseException
	{
		if (query.getQueryType() != QueryType.HQL)
		{
			throw new UnsupportedOperationException("'sharedTo' operation not supported by "+
					query.getQueryType()+" query.");
		}
		
		StringBuilder sb = new StringBuilder();
		String root = query.getRootAlias();
		sb.append("(");
		if (users == null && groups == null && projects == null)
		{
			if (sharedTo)
			{
				// Find all items that are not shared at all
				sb.append(root).append(".itemKey IS NULL AND ");
				sb.append(root).append(".projectKey IS NULL");
			}
			else
			{
				// Find all items that are shared to anyone
				sb.append(root).append(".itemKey IS NOT NULL OR ");
				sb.append(root).append(".projectKey IS NOT NULL");
			}
		}
		else
		{
			if (users != null || groups != null)
			{
				sb.append("(");
				if (!sharedTo)
				{
					sb.append(root).append(".itemKey IS NULL OR ");
				}
				if (users != null)
				{
					sb.append(root).append(".itemKey ").append(sharedTo ? "IN (" : "NOT IN (");
					sb.append("select ik.id from UserData usr JOIN usr.itemKeys ik ");
					sb.append("where " + users.toQl(query, dc));
					sb.append(")");
				}
				if (groups != null)
				{
					if (users != null) sb.append(sharedTo ? " OR " : " AND ");
					sb.append(root).append(".itemKey ").append(sharedTo ? "IN (" : "NOT IN (");
					sb.append("select ik.id from GroupData grp JOIN grp.itemKeys ik ");
					sb.append("where " + groups.toQl(query, dc));
					sb.append(")");
				}
				sb.append(")");
			}
			if (projects != null)
			{
				if (users != null || groups != null) sb.append(sharedTo ? " OR " : " AND ");
				sb.append("(");
				if (!sharedTo)
				{
					sb.append(root).append(".projectKey IS NULL OR ");
				}
				sb.append(root).append(".projectKey ").append(sharedTo ? "IN (" : "NOT IN (");
				sb.append("select pk.id from ProjectData prj JOIN prj.projectKeys pk ");
				sb.append("where " + projects.toQl(query, dc));
				sb.append("))");
			}
		}
		sb.append(")");
		return sb.toString();
	}

	@Override
	public String toString() 
	{
		StringBuilder sb = new StringBuilder();
		sb.append("(");
		if (users == null && groups == null && projects == null)
		{
			if (sharedTo)
			{
				// Find all items that are not shared at all
				sb.append("itemKey IS NULL AND ");
				sb.append("projectKey IS NULL");
			}
			else
			{
				// Find all items that are shared to anyone
				sb.append("itemKey IS NOT NULL OR ");
				sb.append("projectKey IS NOT NULL");
			}
		}
		else
		{
			if (users != null || groups != null)
			{
				sb.append("(");
				if (!sharedTo)
				{
					sb.append("itemKey IS NULL OR ");
				}
				if (users != null)
				{
					sb.append("itemKey ").append(sharedTo ? "IN (" : "NOT IN (");
					sb.append("select ik.id from UserData usr JOIN usr.itemKeys ik ");
					sb.append("where " + users.toString());
					sb.append(")");
				}
				if (groups != null)
				{
					if (users != null) sb.append(sharedTo ? " OR " : " AND ");
					sb.append("itemKey ").append(sharedTo ? "IN (" : "NOT IN (");
					sb.append("select ik.id from GroupData grp JOIN grp.itemKeys ik ");
					sb.append("where " + groups.toString());
					sb.append(")");
				}
				sb.append(")");
			}
			if (projects != null)
			{
				if (users != null || groups != null) sb.append(sharedTo ? " OR " : " AND ");
				sb.append("(");
				if (!sharedTo)
				{
					sb.append("projectKey IS NULL OR ");
				}
				sb.append("projectKey ").append(sharedTo ? "IN (" : "NOT IN (");
				sb.append("select pk.id from ProjectData prj JOIN prj.projectKeys pk ");
				sb.append("where " + projects.toString());
				sb.append("))");
			}
		}
		sb.append(")");
		return sb.toString();
	}
	/**
		This restriction is equal to another SharedToRestriction if their 
		child restrictions and other parameters are equal.
	*/
	@Override
	public boolean equals(Object other)
	{
		if (this == other) return true;
		if (other == null || this.getClass() != other.getClass()) return false;
		SharedToRestriction o = (SharedToRestriction)other;
		return sharedTo == o.sharedTo && EqualsHelper.equals(users, o.users) &&
			EqualsHelper.equals(groups, o.groups) && EqualsHelper.equals(projects, o.projects);
	}
	@Override
	public int hashCode()
	{
		return EqualsHelper.hashCode(users, groups, projects);
	}
	// -------------------------------------------

	
}
