/**
	$Id: EnhancedThreadSignalHandler.java 6898 2015-05-12 09:27:08Z nicklas $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.signal;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

/**
	An extension to the thread signal handler that supports any number of 
	signals. When a signal is recieved it will call {@link Thread#interrupt()}
	on the worker thread. This signal handler is also registered as an interrupt
	handler with {@link ThreadSignalHandler#setInterruptHandler(InterruptHandler)},
	and will throw a {@link SignalReceivedException} when interrupted.
	<p>
	When the worker thread becomes aware of the notification	it should call 
	{@link #getReceivedSignals()} or {@link #hasReceived(Signal)}
	to find out which signal that was sent and then take the proper action.
	<p>
	
	There are usually three different ways to get a notification about 
	the signal:
	<ul>
	<li>An {@link InterruptedException} is thrown. This usually happens when the
		code involves blocking IO operations or other blocking synchronization
		functions.
	<li>A {@link SignalReceivedException} is thrown. This is usually
		thrown from the BASE core when it detects that the thread has been
		interrupted.
	<li>Check the Thread.interrupted() flag. But we recommend that
		{@link ThreadSignalHandler#checkInterrupted()} is used instead.
	</ul>

	
	<pre class="code">
// ... code in worker thread
threadSignalHandler.setWorkerThread();
beginTransaction();
boolean done = false;
boolean interrupted = false;
while (!done &amp;&amp; !interrupted)
{
   try
   {
      done = doSomeWork(); // NOTE! This must not take forever!
      ThreadSignalHandler.checkInterrupted();
   }
   catch (SignalReceivedException ex)
   {
   	// This exception can be thrown from BASE core when a thread is interrupted
   	interrupted = true;
   }
   catch (InterruptedException ex)
   {
      // NOTE! Try-catch is only needed if thread calls a blocking method 
      interrupted = true;
   }
}
if (interrupted)
{
	if (threadSignalHandler.hasReceived(Signal.SHUTDOWN))
	{
		// save current state so that we can continue when the system
		// is up and running again
	}
   rollbackTransaction();
}
else
{
   commitTransaction();
}
</pre>
	
	@author nicklas
	@since 2.16
	@base.modified $Date: 2015-05-12 11:27:08 +0200 (tis, 12 maj 2015) $
*/
public class EnhancedThreadSignalHandler
	extends AbstractSignalHandler
	implements InterruptHandler
{

	/**
		Log signals processing.
	*/
	private static final org.slf4j.Logger logger = 
		org.slf4j.LoggerFactory.getLogger(EnhancedThreadSignalHandler.class);

	private final List<Signal> received;
	
	/**
		The worker thread that should be interrupted when a signal is
		received.
	*/
	private Thread workerThread;
	
	/**
		Create a new signal handler that supports the specified 
		signals.
		@param supported A collection with the signals that are initially supported.
			More signals can be added with {@link #addSignal(Signal)}
	*/
	public EnhancedThreadSignalHandler(Collection<Signal> supported)
	{
		super(supported);
		this.received = Collections.synchronizedList(new ArrayList<Signal>());
		setWorkerThread();
	}

	/**
		Create a signal handler that supports the specified 
		signals.
		@param supported An array with the signals that are initially supported.
			More signals can be added with {@link #addSignal(Signal)}
	*/
	public EnhancedThreadSignalHandler(Signal... supported)
	{
		this(Arrays.asList(supported));
	}
	
	/*
		From the SignalHandler interface
		-------------------------------------------
	*/
	/**
		When a supported signal is received	{@link Thread#interrupt()} is called 
		on the worker thread that was registered when this object was created. 
		Note that this method is called from a different thread. The worker thread 
		is busy doing something  else. If the worker thread has already ended, 
		this method does nothing. It is the responsibility of the worker
		thread to regularly check with this signal handler if it has recieved
		any signals.
		
		@param signal The signal
		@throws UnsupportedSignalException If signal is not supported
	*/
	@Override
	public void handleSignal(Signal signal)
	{
		logger.debug("Got signal: " + signal);
		if (!workerThread.isAlive()) return;
		if (supports(signal))
		{
			received.add(signal);
			logger.debug("Sending signal " + signal + " to thread: " + workerThread);
			workerThread.interrupt();
		}
		else
		{
			logger.debug("Signal not supported: " + signal);
			throw new UnsupportedSignalException(signal);
		}
	}
	// -------------------------------------------

	/*
		From the InterruptHandler interface
		-----------------------------------
	*/
	/**
		Throws a {@link SignalReceivedException} if one ore more
		signals has been received.
	*/
	@Override
	public void handleInterrupt()
	{
		if (received.size() > 0)
		{
			throw new SignalReceivedException(received);
		}

	}
	// -----------------------------------

	/**
		Set the worker thread to the current thread. This will also register
		this object as an interrupt handler with {@link 
		ThreadSignalHandler#setInterruptHandler(InterruptHandler)}.
	*/
	public void setWorkerThread()
	{
		this.workerThread = Thread.currentThread();
		ThreadSignalHandler.setInterruptHandler(this);
	}
	
	/**
		Check if any signals has been received by this handler.
		@return TRUE if at least one signal has been received, FALSE otherwise
	*/
	public boolean hasReceivedSignals()
	{
		return received.size() > 0;
	}
	
	/**
		Get the list of received signals. Calling this method clears the internal
		list and if no signals are received in the meantime, the next call to
		this method will return null.
		@return A list with signals in the order they were
			received, or null if no signals has been received
	*/
	public List<Signal> getReceivedSignals()
	{
		List<Signal> toReturn = null;
		if (received.size() > 0)
		{
			toReturn = new ArrayList<Signal>(received);
			received.clear();
		}
		return toReturn;
	}
	
	/**
		If at least one signal has been received a {@link SignalReceivedException}
		is thrown. This method clears the received signals.
	*/
	public void checkForSignals()
	{
		if (received.size() > 0)
		{
			throw new SignalReceivedException(getReceivedSignals());
		}
	}
	
	/**
		Check if the given signal has been received by this signal handler.
		@param signal The signal to check for
		@return TRUE if the signal has been received, FALSE otherwise
	*/
	public boolean hasReceived(Signal signal)
	{
		return received.contains(signal);
	}

}
