/**
	$Id: CelValidationAction.java 5689 2011-08-11 09:09:06Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.affymetrix;

import affymetrix.fusion.cdf.FusionCDFData;
import affymetrix.fusion.cel.FusionCELData;
import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.File;
import net.sf.basedb.core.FileSetMember;
import net.sf.basedb.core.FileStoreUtil;
import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.core.InvalidRelationException;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.util.fileset.SingleFileValidationAction;

/**
	Action for validation of CEL files in a file set. The
	CEL file must be connected with a {@link RawBioAssay}.
	The raw bioassay should have an {@link ArrayDesign} that
	has a CDF file matching the chip type.
	<p>
	
	This class is just a wrapper for {@link CelValidator} so
	that we can hook the validation into the extensions system.
	

	@author Nicklas
	@since 3.0
	@base.modified $Date: 2011-08-11 11:09:06 +0200 (tors, 11 aug. 2011) $
*/
public class CelValidationAction
	extends SingleFileValidationAction
{

	private final DbControl dc;
	private final RawBioAssay rba;
	private final CelValidator validator;
	
	/**
		Create a new action. 
		@param dc An open DbControl
		@param rba The raw bioassay the CEL file is connected with
	*/
	public CelValidationAction(DbControl dc, RawBioAssay rba)
	{
		super(DataFileType.AFFYMETRIX_CEL);
		this.dc = dc;
		this.rba = rba;
		this.validator = new CelValidator();
	}

	/*
		From the ValidationAction interface
		-----------------------------------
	*/
	@Override
	public void validateAndExtractMetadata()
		throws InvalidDataException, InvalidRelationException
	{
		FileSetMember celMember = getAcceptedFile();
		
		FusionCELData celData = validator.loadCelFile(celMember.getFile());
		validator.copyMetadata(celData, rba);
		
		// Check if the CEL matches the CDF on the ArrayDesign. 
		// A failure will result in InvalidRelationException
		try
		{
			ArrayDesign design = rba.getArrayDesign();
			if (design == null) 
			{
				throw new InvalidDataException("Raw bioassay '" + rba.getName() + 
					"' has no array design");
			}
			if (!AffymetrixUtil.isAffymetrix(design))
			{
				throw new InvalidDataException("The array design '" + design.getName() + 
					"' is not an Affymetrix design.");
			}

			// Verify that CEL chip type match CDF type
			File cdfFile = FileStoreUtil.getDataFile(dc, design, DataFileType.AFFYMETRIX_CDF, true);
			if (cdfFile == null)
			{
				throw new InvalidDataException("The array design '" + design.getName() + 
					"' doesn't have a valid CDF file.");
			}
			FusionCDFData cdfData = new CdfValidator().loadCdfFile(cdfFile);
			validator.validateCelAndCdf(celData, cdfData, cdfFile.getName());
		}
		catch (Throwable t)
		{
			throw new InvalidRelationException(t);
		}
	}

	@Override
	public void resetMetadata()
	{
		validator.resetMetadata(rba);
	}
	// -----------------------------------

}
