/**
	$Id: AbstractDataWriterFactory.java 5212 2010-01-12 13:21:49Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.bfs;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.nio.charset.Charset;
import java.util.HashSet;
import java.util.Set;

import net.sf.basedb.core.ItemAlreadyExistsException;

/**
	Abstract implementation of a data writer factory. This class contains
	a filename generation algorithm that simply generates a sequence of
	filenames using a numeric counter and prefix/suffix strings.
	<p>
	
	Subclasses may override the filename generation. Subclasses
	must also provide an implementation that, given a filename, creates
	an output stream for it.

	@author Nicklas
	@version 2.15
	@base.modified $Date: 2010-01-12 14:21:49 +0100 (tis, 12 jan. 2010) $
*/
public abstract class AbstractDataWriterFactory<T>
	implements DataWriterFactory<T>
{

	private Set<String> usedFiles;
	private FilenameGenerator<T> filenameGenerator;
	
	/**
		Create a new factory that uses the given file name generator.
	*/
	protected AbstractDataWriterFactory(FilenameGenerator<T> filenameGenerator)
	{
		this.filenameGenerator = filenameGenerator;
		this.usedFiles = new HashSet<String>();
	}
	
	/*
		From the DataWriterFactory interface
		------------------------------------
	*/
	@Override
	public DataWriter createDataWriter(T owner, String suggestedFilename)
		throws IOException
	{
		String filename = filenameGenerator.getNextFilename(owner, suggestedFilename);
		if (!isUnique(filename))
		{
			throw new ItemAlreadyExistsException("File '" + filename + "'");
		}
		usedFiles.add(filename);
		OutputStream out = getOutputStream(filename);
		DataWriter writer = new DataWriter(new OutputStreamWriter(out, Charset.forName("UTF-8")));
		writer.setFilename(filename);
		return writer;
	}
	// ---------------------------------------

	/**
		Create an output stream for writing to a file with the given
		filename. The returned output stream will be wrapped in a 
		writer that uses the UTF-8 character set.
		@throws IOException If there is a problem creating the file
	*/
	protected abstract OutputStream getOutputStream(String filename)
		throws IOException;
	
	/**
		Checks if the suggested file name has already been used
		by this data writer factory or not.
		@param filename The filename to check
		@return TRUE if the name is unique or FALSE if it has already been used
	*/
	protected boolean isUnique(String filename)
	{
		return !usedFiles.contains(filename);
	}
		
	
}
