/**
	$Id: SerialBaseFileExporter.java 5689 2011-08-11 09:09:06Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.export.spotdata;

import java.util.Arrays;
import java.util.List;

import net.sf.basedb.core.BioAssay;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DynamicSpotQuery;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.signal.ThreadSignalHandler;
import net.sf.basedb.util.basefile.BaseFileWriter;

/**
	Exporter implementation that exports bioassay set data as a serial
	BASEfile.
	
	@author Nicklas
	@version 2.12
	@base.modified $Date: 2011-08-11 11:09:06 +0200 (tors, 11 aug. 2011) $
*/
public class SerialBaseFileExporter
	extends BaseFileExporter
{

	public static final String SUBTYPE = "serial";

	private List<BioAssay> bioAssays;
	private int nextBioAssay;
	private BaseFileWriter out;
	private DynamicSpotQuery spotQuery;
	private DynamicSpotQuery countQuery;
	private long spotCount;
	
	/**
		Create a new serial BASEfile exporter.
	*/
	public SerialBaseFileExporter()
	{}

	/**
		From AbstractBioAssaySetExporter/BaseFileExporter class
		-------------------------------------------------------
	*/
	/**
		Prepare the export by pre-loading some information and
		configure the queries that we are going to use.
	*/
	@Override
	protected void beginExport()
	{
		super.beginExport();
		bioAssays = getBioAssays();
		nextBioAssay = 0;
		out = getBaseFileWriter();
		spotQuery = getSpotQuery(true);
		countQuery = getCountQuery(true);
		setProgress(0, "Caching reporter data...");
		cacheReporterData();
	}

	/**
		Exports headers for a 'spots' section, and make preparations
		for the current data section (one section per bioassay).
	*/
	@Override
	protected void exportSectionHeader()
	{
		// Get configuration options
		DbControl dc = getDbControl();
		BioAssay current = bioAssays.get(nextBioAssay);
		int bioAssayColumn = (int)current.getDataCubeColumnNo();
		
		// Set parameters on the queries
		spotQuery.setParameter("bioAssayColumn", bioAssayColumn, Type.INT);
		countQuery.setParameter("bioAssayColumn", bioAssayColumn, Type.INT);
		
		int percent = 10+(int)((90L * nextBioAssay) / bioAssays.size());
		setProgress(percent, "Exporting bioassay '" + current.getName() + 
			"' (" + (nextBioAssay+1)+" of " + bioAssays.size() + ")");

		// Count the spots in this section. Delegate output to superclass
		long count = countQuery.count(dc);
		spotCount += count;
		ThreadSignalHandler.checkInterrupted();
		exportSpotSectionHeaders(Arrays.asList(current), count);
	}

	/**
		Exports data for the current bioassay in a 'spots' section.
	*/
	@Override
	protected void exportSectionData()
	{
		BioAssay current = bioAssays.get(nextBioAssay);
		exportSpotSectionData(Arrays.asList(current), spotQuery, null, 0);
	}

	/**
		Ends the current 'spots' section.
		@return Always FALSE if this was the last bioassay, or TRUE if 
			there are more bioassays to export
	*/
	@Override
	protected boolean exportSectionFooter()
	{
		out.baseEndSection();
		out.flush();
		nextBioAssay++;
		return nextBioAssay < bioAssays.size();
	}

	@Override
	protected void endExport(RuntimeException e)
	{
		if (e == null) setProgress(100, "Export complete. " + spotCount + " spots done");
		out = null;
		bioAssays = null;
		spotQuery = null;
		countQuery = null;
		super.endExport(e);
	}
	// -------------------------------------------------------
	
	
	
	
}
