/**
	$Id: ErrorHandler.java 5486 2010-11-12 13:59:38Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions;

/**
	Error handlers are created by the factory for each request/use of 
	an extension point. The error handler doesn't have to be thread-safe, but
	must be if the factory re-uses a single instance. Whenever an exception
	happens during the processing of an extension, the error is re-directed
	to the error handler. The error handler is more or less free to do what
	it wants. The default implementation {@link LoggingErrorHandlerFactory}
	simply puts a message in the log. Note that it is not usually practical 
	to let the error propagate up to the gui level.

	@author Nicklas
	@since 2.17
	@base.modified $Date: 2010-11-12 14:59:38 +0100 (fre, 12 nov. 2010) $
*/
public interface ErrorHandler<A extends Action>
{
	
	/**
		Handle an exception that has happened. It is up to
		the error handler what should happen next. Typically,
		error should always be logged but the error handler
		may choose to re-throw the exception. If the exception
		isn't already a runtime exception it must be wrapped.
		
		<p>
		Note that there are three stages in the extension 
		mechanism, which may cause some of the parameters to be null:
		
		<ul>
		<li>Extension point intialization: Eg. when the render factory
			is being prepared (context and action are both null)
		<li>Extension intialization: Eg. when the action factory
			is being prepared (action is null)
		<li>Extension rendering: Eg. when rendering the actions of
			a single extension (context and action are non-null)
		</ul>
		
		@param context The extension context, which can be null
		@param action The action that was being rendered when the
			error happened, which can be null
		@param message An optional error message
		@param t The error
	*/
	public void handleError(InvokationContext<? extends A> context, A action, 
			String message, Throwable t);
}
