/**
	$Id: Extension.java 7689 2019-04-05 11:42:26Z nicklas $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions;

import net.sf.basedb.core.plugin.About;
import net.sf.basedb.util.extensions.xml.XmlLoader;

/**
	Represents a concrete implementation of an extension to an
	extension point. Each extension must have a unique ID. We recommend 
	that ID:s are generated in the same way as Java package names.
	An extension defines an {@link ActionFactory} which is a factory class
	that knows how to create actions. The actions should be of the same
	class (or a subclass) as specified by the 
	{@link ExtensionPoint#getActionClass()} method of the extension point that
	the extension is extending.
	<p>
	If the extension point allows it, the extension may also provide a
	{@link RendererFactory}. See {@link ExtensionPoint#allowRendererOverride()}.

	<p>
	Extensions can, for example, be created:
	<ul>
	<li>Programmatically, using an {@link ExtensionBean} object.
	<li>Loaded from extension definition XML files using a {@link XmlLoader}.
	<li>Or, in any other way by providing a custom implementation of this interface.
	</ul>
	
	<p>
	Before an extension can be used it must be registered with 
	{@link Registry#registerExtension(Extension, ClassLoader)}. The extension point that
	the extension extends must of course already be registered in the same
	registry.
	
	@author nicklas
	@version 2.7
	@base.modified $Date: 2019-04-05 13:42:26 +0200 (fre, 05 apr. 2019) $
*/
public interface Extension<A extends Action>
{

	/**
		Get the ID of the extension.
		@return A non-null unique identifier value
	*/
	public String getId();

	/**
		Get the ID of the extension point this extension is extending.
		@return A non-null identifier value
	*/
	public String getExtends();
	
	/**
		The extensions in an extension point are by default ordered by their
		index value. Extensions with a low value are ordered before
		extensions with a high value. The ordering may be overridden by providing
		an external {@link ExtensionsFilter} implementation.
		@return The index value
	*/
	public float getIndex();
	
	/**
		Get information about the extensions. This is optional, but
		we recommend that at least some documentation and contact
		information is given.
		@return Optional information about the extension
	*/
	public About getAbout();
	
	/**
		Get the factory class for creating actions. The factory must create
		objects of the same class or a subclass as the {@link ExtensionPoint#getActionClass()}
		method of the extended extension point returns.
		@return An action factory (required)
	*/
	public ActionFactory<? extends A> getActionFactory();
	
	/**
		Get an optional factory class for creating renderers for the actions.
		This factory will only be used if the extension point allows it.
		
		@return A renderer factory or null
		@see ExtensionPoint#allowRendererOverride()
	*/
	public RendererFactory<? super A> getRendererFactory();
	
}
