/**
	$Id: InvokationContext.java 6030 2012-03-28 10:38:04Z nicklas $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions;

/**
	Keeps contextual information about a single invokation of
	extensions. The {@link Registry#useExtensions(ClientContext, ExtensionsFilter, String...)}
	will create invokation contexts for each enabled extension that
	extends any of the given extension points.
	<p>
	
	Instances of this class are passed to factory methods,
	for example {@link ActionFactory#getActions(InvokationContext)}
	to make it possible for the factories to know anything about
	the context the extension was invoked in.
	<p>
	
	Do not confuse the invokation context with the a 
	{@link ClientContext} object. The invokation context contains
	the client context (see {@link #getClientContext()} but has also
	a lot more information collected from other parts of the 
	extensions system.

	@author nicklas
	@version 2.7
	@base.modified $Date: 2012-03-28 12:38:04 +0200 (ons, 28 mars 2012) $
*/
public abstract class InvokationContext<A extends Action>
{

	protected InvokationContext()
	{}
	
	/**
		Get the client context that was passed to the 
		{@link Registry#useExtensions(ClientContext, ExtensionsFilter, String...)}
		method.
		@return The client context, or null if the client didn't pass
			any context object
	*/
	public abstract ClientContext getClientContext();

	/**
		Get information about the extension point.
		@return An extension point object
	*/
	public abstract ExtensionPoint<? super A> getExtensionPoint();
	
	/**
		Get information about the extension. This method may
		return null if there is no current extension, which 
		happens when any of the {@link RendererFactory} methods are called 
		for a renderer factory attached to an extension point
		and for the {@link ErrorHandlerFactory} methods.
		@return An extension object, or null if there is no current extension
	*/
	public abstract Extension<A> getExtension();
	
	/**
		Get the class loader that is used for the given extension
		or extension point.
		@since 3.2
	*/
	public abstract ClassLoader getClassLoader();
	
	/**
		Get the value of an attribute that has been registered
		for the extension.
		
		@param name The name of the attribute
		@return The attribute value, or null if no attribute has
			been registered
		@see Registry#getAttribute(String, String)
	*/
	public abstract Object getAttribute(String name);

	/**
		Get the registry where this invokation originated from.
	*/
	protected abstract Registry getRegistry();
	
	/**
		Prepare the renderer factory if the extension or extension
		point has one.
	*/
	protected abstract void prepareRendererFactory();
	
	/**
		Get the renderer, if any. This method should always return the
		same rendere instance if called multiple times on the same object. 
	*/
	protected abstract Renderer<? super A> getRenderer();

	/**
		Get the actions, or null if there is no current extension.
	*/
	protected abstract A[] getActions();
	
	/**
		Register an error for the current extension or extension point. 
		Registered errors can be retrieved by {@link Registry#getLastExtensionError(String)}
		and {@link Registry#getLastExtensionPointError(String)}
		@param action The action that caused the error
		@param t The error
		@since 2.9
	*/
	protected abstract void setError(A action, Throwable t);
	
	/**
		Clear the registered error for the current extension or extension point.
		@since 2.9
	*/
	protected abstract void clearError();
	
	/**
		Get the error handler used in this context. Should never
		be null, since if not the extension point provides one, the
		system should.
		@since 2.17
	*/
	protected abstract ErrorHandler<? super A> getErrorHandler();
	
}
