/**
	$Id: LoggingErrorHandlerFactory.java 6875 2015-04-20 09:08:18Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions;

import net.sf.basedb.util.Values;


/**
	Exception handler factory implementation that logs error messages
	using the default logging mechanism. This factory is it's own
	error handler and the implementation is thread-safe (once the level
	has been set).
	
	@author Nicklas
	@since 2.17
	@base.modified $Date: 2015-04-20 11:08:18 +0200 (mån, 20 apr. 2015) $
*/
public class LoggingErrorHandlerFactory
	implements ErrorHandlerFactory<Action>, ErrorHandler<Action>
{
	
	private static final org.slf4j.Logger log = 
		org.slf4j.LoggerFactory.getLogger(LoggingErrorHandlerFactory.class);

	private Level level;
	private boolean rememberLast;
	
	/**
		Create a new error-level loggin error handler factory.
	*/
	public LoggingErrorHandlerFactory()
	{
		this.level = Level.ERROR;
		this.rememberLast = false;
	}

	/**
		Create a new custom-level logging error handler factory.
	*/
	public LoggingErrorHandlerFactory(String level)
	{
		setLevel(level);
	}

	/*
		From the ErrorHandlerFactory interface
		--------------------------------------
	*/
	@Override
	public ErrorHandler<Action> getErrorHandler(InvokationContext<? extends Action> context)
	{
		return this;
	}
	// --------------------------------------

	/*
		From the ErrorHandler interface
		--------------------------------------
	*/
	@Override
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public void handleError(InvokationContext<? extends Action> context, Action action, 
		String message, Throwable t)
	{
		if (rememberLast) ((InvokationContext)context).setError(action, t);
		level.log(log, message, t);
	}
	// --------------------------------------
	
	/**
		Set the level of the error handler.
	*/
	public void setLevel(String level)
	{
		try
		{
			this.level = Level.valueOf(level);
		}
		catch (Exception ex)
		{}
	}
	
	/**
		Set a flag indicating if errors should be remembered
		by the registry. The last error for an extension or 
		extension point can be retreived by calling 
		{@link Registry#getLastExtensionError(String)} and/or
		{@link Registry#getLastExtensionPointError(String)}.
		@param remember A string that evaluates to a boolean
	*/
	public void setRememberLast(String remember)
	{
		this.rememberLast = Values.getBoolean(remember);
	}
	
	/**
		Log level implementations for slf4j. Use {@link Level#log(org.slf4j.Logger, String, Throwable)}
		to log a message at the given level to the slf4j logger.
		@since 3.3
	*/
	public static enum Level
	{
		OFF 
			{
				@Override
				public void log(org.slf4j.Logger logger, String message, Throwable t)
				{}
			},
		FATAL
		{
			@Override
			public void log(org.slf4j.Logger logger, String message, Throwable t)
			{
				logger.error(message, t);
			}
		},
		ERROR
		{
			@Override
			public void log(org.slf4j.Logger logger, String message, Throwable t)
			{
				logger.error(message, t);
			}
		},
		WARN
		{
			@Override
			public void log(org.slf4j.Logger logger, String message, Throwable t)
			{
				logger.warn(message, t);
			}
		},
		INFO
		{
			@Override
			public void log(org.slf4j.Logger logger, String message, Throwable t)
			{
				logger.info(message, t);
			}
		},
		DEBUG
		{
			@Override
			public void log(org.slf4j.Logger logger, String message, Throwable t)
			{
				logger.debug(message, t);
			}
		},
		TRACE
		{
			@Override
			public void log(org.slf4j.Logger logger, String message, Throwable t)
			{
				logger.trace(message, t);
			}
		};
		
		public abstract void log(org.slf4j.Logger logger, String message, Throwable t);
		
	}
}
