/**
	$Id: ExtensionsFileProcessor.java 5823 2011-10-24 13:02:46Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions.manager;

import net.sf.basedb.util.extensions.manager.ExtensionsFile.WriteableExtensionsFile;

/**
	Interface that is used to request a callback for each extensions file
	managed by an extensions manager. Implementations are more or less free
	to do whatever they want with the file. Typically, they will scan the
	XML metadata or the contents of a JAR file and do something with it.
	<p>
	
	A processer implementation is used by calling {@link 
	ExtensionsManager#processFiles(ExtensionsFileProcessor)}.
	<p>
	
	The manager will first call {@link #begin(ExtensionsManager, int)} and
	then {@link #processFile(ExtensionsManager, ExtensionsFile.WriteableExtensionsFile)} for each
	file that has been confirmed to contain a valid extension definitions 
	file. If all files could be processed without problems the manager
	finishes by calling {@link #done(ExtensionsManager)}. If the processor
	throws an exception the processing is immediately aborted and 
	{@link #done(ExtensionsManager, Throwable)} is called instead.
	
	@author Nicklas
	@since 3.0
	@base.modified $Date: 2011-10-24 15:02:46 +0200 (mån, 24 okt. 2011) $
*/
public interface ExtensionsFileProcessor
{

	/**
		Called by the extensions manager before the processing starts.
		
		@param manager The manager that is executing the action
		@param numFiles The number of extension files that is going to be processed this time
	*/
	public void begin(ExtensionsManager manager, int numFiles);
	
	/**
		Called by the extensions manager for each extensions file that it
		manages. The order of the files is generally not predicatable and
		may change from one invokation to the next.
		<p>
		
		If the processor throws an exception the manager will abort
		processing and call {@link #done(ExtensionsManager, Throwable)}.
		Error that are related to a specific file should be catched
		by the processor and registered by calling .......
		<p>
		
		If all files was processed without exceptions the manager
		will call {@link #done(ExtensionsManager)}.

		<p>
		Note that the file is read-only to begin with. If the processor
		wants to call any write-operation the file must be opened by
		calling {@link WriteableExtensionsFile#open()}. It is recommended
		that the processor calls {@link WriteableExtensionsFile#close()}
		once it is done writing.
		
		@param manager The manager that is executing the action
		@param file The file to be processed
	*/
	public void processFile(ExtensionsManager manager, WriteableExtensionsFile file);

	/**
		Called by the extensions manager after a successful completion of the
		processing. 
		@param manager The manager that is executing the action
	*/
	public void done(ExtensionsManager manager);
	
	/**
		Called by the extensions manager if an exception was thrown
		by the processor. 
		@param manager The manager that is executing the action
		@param t The exception
	*/
	public void done(ExtensionsManager manager, Throwable t);
	
}
