/**
	$Id: ExtractResourcesProcessor.java 7642 2019-03-12 06:53:27Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions.manager.processor;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import net.sf.basedb.util.FileUtil;
import net.sf.basedb.util.extensions.manager.ExtensionsFile;
import net.sf.basedb.util.extensions.manager.ExtensionsFileProcessor;
import net.sf.basedb.util.extensions.manager.ExtensionsManager;
import net.sf.basedb.util.extensions.manager.ProcessResults;
import net.sf.basedb.util.extensions.manager.ExtensionsFile.WriteableExtensionsFile;
import net.sf.basedb.util.extensions.manager.filter.ValidAndNewOrModifiedFilter;

/**
	A file processor implementation that extracts resources from JAR files.
	The main target directory must be specified when an instance is constructed.
	Each processed JAR file is extracted into a subdirectory with the same name
	as the JAR file. Existing files are always overwritten.
	<p>
	
	The filter (in the form of a regular expressions pattern) can be
	specified to limit which files are extracted from the JAR file.
	The filter is also used to re-write target file names if they
	need to be different from the file names in the JAR file.
	<p>
	
	Note! It is recommended that this processor is paired with a 
	{@link ValidAndNewOrModifiedFilter} since it is usually no
	point in extracting resources from invalid extensions.

	@author Nicklas
	@since 3.0
	@base.modified $Date: 2019-03-12 07:53:27 +0100 (tis, 12 mars 2019) $
*/
public class ExtractResourcesProcessor
	implements ExtensionsFileProcessor
{

	private static final org.slf4j.Logger log = 
		org.slf4j.LoggerFactory.getLogger(ExtractResourcesProcessor.class);
	
	private final File mainDir;
	private final Pattern filter;
	private final String replacement;
	private final ProcessResults results;
	
	private int numFiles;
	private int numError;
	private int numTotalExtracted;

	/**
		Create a new processor.
		
		@param mainDir The main directory to where the resources should be extracted
	*/
	public ExtractResourcesProcessor(File mainDir, ProcessResults results)
	{
		this(mainDir, null, null, results);
	}

	/**
		Create a new processor.
		
		@param mainDir The main directory to where the resources should be extracted
		@param filter A regular expression filter that must match the file name
			in the JAR file
		@param replacement A replacement string to create a new file name
			using {@link Matcher#replaceAll(String)}.
	*/
	public ExtractResourcesProcessor(File mainDir, Pattern filter, String replacement, ProcessResults results)
	{
		this.mainDir = mainDir;
		this.filter = filter;
		this.replacement = replacement;
		this.results = results;
	}

	
	/*
		From the ExtensionsFileProcessor interface
		------------------------------------------
	*/
	@Override
	public void begin(ExtensionsManager manager, int numFiles)
	{
		this.numFiles = 0;
		this.numError = 0;
		this.numTotalExtracted = 0;
	}

	@Override
	public void processFile(ExtensionsManager manager, WriteableExtensionsFile wFile)
	{
		ExtensionsFile xtFile = wFile.getExtensionsFile();
		if (!xtFile.isJar()) 
		{
			log.info("File is not a JAR file (skipping): " + xtFile);
			return;
		}

		File jar = xtFile.getFile();
		File targetDir = new File(mainDir, jar.getName());

		log.info("Extracting resources from '" + jar.getName() + 
				"' to " + targetDir);
		log.debug("filter=" + filter + "; replacement=" + replacement);

		ZipInputStream zipStream = null;
		int numExtracted = 0;
		try
		{
			zipStream = new ZipInputStream(new FileInputStream(jar));
			ZipEntry zipEntry = null;
			while ((zipEntry = zipStream.getNextEntry()) != null)
			{
				// Get the name of the zipped file, but strip first slash
				String name = zipEntry.getName();
				if (name.startsWith("/")) name = name.substring(1);

				// Should the current entry be extracted or not? We start by assuming so ...
				boolean extract = true;
				// A filter may change the actual name of the extracted file
				String extractionName = name;
				if (zipEntry.isDirectory()) 
				{
					// Ignore directories
					extract = false;
					log.debug("Jar entry '" + name + "' is a directory (ignored).");
				}
				else if (filter != null)
				{
					// Ignore files not matched by the filter
					Matcher matcher = filter.matcher(name);
					if (!matcher.matches())
					{
						extract = false;
						log.debug("Jar entry '" + name + "' doesn't match filter (ignored).");
					}
					else
					{
						extractionName = matcher.replaceAll(replacement);
					}
				}
				
				File targetFile = new File(targetDir, extractionName);
				
				// Now... it is finally time to extract the file
				if (extract)
				{
					log.debug("Extracting '" + name + "' to " + targetFile);
					File dir = targetFile.getParentFile();
					dir.mkdirs();
					if (!dir.exists())
					{
						throw new FileNotFoundException("Failed to create directory: " + dir);
					}
					OutputStream out = null;
					try
					{
						out = new FileOutputStream(targetFile);
						FileUtil.copy(zipStream, out);
						out.close();
					}
					finally
					{
						FileUtil.close(out);
					}
					numExtracted++;
					// Always set current time to force Tomcat to re-compile JSP files
					if (!targetFile.setLastModified(System.currentTimeMillis()))
					{
						log.warn("Could not set last modification time on file: " + targetFile);
					}
					else
					{
						log.info("Extracted file '" + name + "' to " + targetFile);
					}
				}
			}
			numFiles++;
			numTotalExtracted += numExtracted;
			if (numExtracted > 0 && results != null)
			{
				results.addMessage(xtFile,	numExtracted + " resource(s) extracted successfully.");
			}
			log.info("Extracted " + numExtracted + " resources from '" + jar.getName() + "' to " + targetDir);
		}
		catch (Throwable ex)
		{
			wFile.setError(true);
			numError++;
			if (results != null)
			{
				results.addErrorMessage(xtFile, "Failed to extract resources: " + ex.getClass().getName() + "; " + ex.getMessage());
			}
			log.error("Failed to extract resources to directory: " + targetDir, ex);
		}
		finally
		{
			FileUtil.close(zipStream);
		}
	}
	
	@Override
	public void done(ExtensionsManager manager)
	{}
	
	@Override
	public void done(ExtensionsManager manager, Throwable t)
	{}
	// ---------------------------------------------------
	
	/**
		Get the number of files that was successfully processed.
	*/
	public int getNumFiles()
	{
		return numFiles;
	}
	
	/**
		Get the number of files that was had an error.
	*/
	public int getNumError()
	{
		return numError;
	}
	
	/**
		Get the total number of resources that was extracts.
	*/
	public int getNumExtracted()
	{
		return numTotalExtracted;
	}

	
}
