/**
	$Id: UnregisterExtensionsProcessor.java 6875 2015-04-20 09:08:18Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions.manager.processor;

import net.sf.basedb.util.extensions.Extension;
import net.sf.basedb.util.extensions.ExtensionPoint;
import net.sf.basedb.util.extensions.Registry;
import net.sf.basedb.util.extensions.manager.ExtensionKey;
import net.sf.basedb.util.extensions.manager.ExtensionPointKey;
import net.sf.basedb.util.extensions.manager.ExtensionsFile;
import net.sf.basedb.util.extensions.manager.ExtensionsFileProcessor;
import net.sf.basedb.util.extensions.manager.ExtensionsManager;
import net.sf.basedb.util.extensions.manager.FactoryParametersKey;
import net.sf.basedb.util.extensions.manager.ProcessResults;
import net.sf.basedb.util.extensions.manager.ExtensionsFile.WriteableExtensionsFile;
import net.sf.basedb.util.extensions.manager.filter.DeletedFilter;

/**
	Extensions file processor implementation that unregisters all extension
	points and extensions. This processor is usually paired with a 
	{@link DeletedFilter} to unregister extensions from deleted files. 
	Using this processor without a filter will unregister all 
	extensions.

	@author Nicklas
	@since 3.0
	@base.modified $Date: 2015-04-20 11:08:18 +0200 (mån, 20 apr. 2015) $
*/
public class UnregisterExtensionsProcessor
	implements ExtensionsFileProcessor
{

	private static final org.slf4j.Logger log = 
		org.slf4j.LoggerFactory.getLogger(UnregisterExtensionsProcessor.class);

	private final ProcessResults results;
	private int numFiles;
	private int numError;
	private int numUnregistered;
	
	/**
		Create a new processor.
		
		@param results Optional object for storing results
	*/
	public UnregisterExtensionsProcessor(ProcessResults results)
	{
		this.results = results;
	}
	
	/*
		From the ExtensionsFileProcessor interface
		------------------------------------------
	*/
	@Override
	public void begin(ExtensionsManager manager, int numFiles)
	{
		this.numFiles = 0;
		this.numError = 0;
		this.numUnregistered = 0;
	}

	@Override
	public void processFile(ExtensionsManager manager,	WriteableExtensionsFile wFile)
	{
		ExtensionsFile xtFile = wFile.getExtensionsFile();
		
		log.info("Unregistering extensions in file: " + xtFile);
		try
		{
			wFile.open();
			int num = 0;
			Registry registry = manager.getRegistry();
			for (ExtensionPoint<?> ep : xtFile.getObjectsOfClass(ExtensionPoint.class))
			{
				unregisterExtensionPoint(wFile, ep, registry);
				num++;
			}
			for (Extension<?> ext : xtFile.getObjectsOfClass(Extension.class))
			{
				unregisterExtension(wFile, ext, registry);
				num++;
			}
			numUnregistered += num;
			numFiles++;
			if (results != null)
			{
				if (!xtFile.exists()) results.setStatus(xtFile, "Deleted");
				if (num > 0) results.addMessage(xtFile, num + " extension(s) unregisterered.");
			}
			log.info("Unregistered  " + num + " extensions from file: " + xtFile);
		}
		catch (Throwable ex)
		{
			wFile.setError(true);
			numError++;
			if (results != null)
			{
				results.addErrorMessage(xtFile, "Could not unregister extensions: " + ex.getClass().getName() + "; " + ex.getMessage());
			}
			log.error("Could not unregister extensions in file: " + xtFile, ex);
		}
		finally
		{
			wFile.close();
		}
	}

	@Override
	public void done(ExtensionsManager manager)
	{
		log.info(numFiles + " extensions has been deleted");
		if (numError > 0)
		{
			log.warn(numError + " extensions could not be deleted");
		}
	}
	
	@Override
	public void done(ExtensionsManager manager, Throwable t)
	{}
	// ----------------------------------------------------
	
	/**
		Unregister the given extension including factory metadata etc.
	*/
	private void unregisterExtension(WriteableExtensionsFile wFile, Extension<?> ext, Registry registry)
	{
		unregisterFactory(wFile, ext.getRendererFactory());
		unregisterFactory(wFile, ext.getActionFactory());
		ExtensionKey key = new ExtensionKey(ext);
		wFile.unregisterObject(key);
		registry.unregisterExtension(ext.getId());
	}

	/**
		Unregister the given extension point including factory metadata etc.
	*/
	private void unregisterExtensionPoint(WriteableExtensionsFile wFile, ExtensionPoint<?> ep, Registry registry)
	{
		unregisterFactory(wFile, ep.getRendererFactory());
		unregisterFactory(wFile, ep.getErrorHandlerFactory());
		ExtensionPointKey key = new ExtensionPointKey(ep);
		wFile.unregisterObject(key);
		registry.unregisterExtensionPoint(ep.getId());
	}
	
	private void unregisterFactory(WriteableExtensionsFile wFile, Object factory)
	{
		if (factory == null) return;
		wFile.unregisterMetadata(new FactoryParametersKey(factory));
	}

	
	/**
		Get the number of files that was successfully processed.
	*/
	public int getNumFiles()
	{
		return numFiles;
	}
	
	/**
		Get the number of files that was had an error.
	*/
	public int getNumError()
	{
		return numError;
	}
	
	/**
		Get the number of extensions + extension points that
		was unregistered.
	*/
	public int getNumUnregistered()
	{
		return numUnregistered;
	}
	
}
