/**
	$Id: ValidationAction.java 5818 2011-10-21 11:15:41Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.fileset;

import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.File;
import net.sf.basedb.core.FileSet;
import net.sf.basedb.core.FileSetMember;
import net.sf.basedb.core.FileStoreEnabled;
import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.core.InvalidRelationException;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.util.extensions.Action;
import net.sf.basedb.util.extensions.ClientContext;

/**
	Action for validating and metadata extraction from files in a file set. 
	This action is used in two different contexts:
	
	<ul>
	<li>For validating the files when {@link FileSet#validate(DbControl)}.
		This action is supposed to validated the entire file set as a whole. The
		{@link ClientContext} parameter passed to the factory has the owner
		of the file set (eg. the {@link FileStoreEnabled}) item as the
		current item.
		
	<li>For removing metadata when a file in a file set is removed or replaced
		with another file. For example, when the {@link FileSet#removeMember(FileSetMember)},
		{@link FileSet#setMember(File, DataFileType)} or any other method that removs
		a file from the fileset is called. The {@link ClientContext} in this case also
		has the owner of the file set as the current item, but the action will only be 
		asked to accept the file(s) that are actually removed/modified.
	</ul>
	<p>
	
	Tip! For validators that works on a single file at a time the 
	{@link SingleFileValidationAction} can be used as a base class.

	@author Nicklas
	@since 3.0
	@base.modified $Date: 2011-10-21 13:15:41 +0200 (fre, 21 okt. 2011) $
*/
public interface ValidationAction
	extends Action
{

	/**
		Check if the given file is handled by this validator implementation 
		or not? No validation of the actual file data should take place in this
		method. This method is called by the core for all members of the
		file set that are still considered as valid. If another validator
		has marked a file as invalid it will not be offered to the remaining
		validators.
		<p>
		The file can either be validated immediately or later when this method
		has been called for all files in the file set. If the file
		is accepted for immediate validation the {@link #validateAndExtractMetadata()}
		is called before the next file is processed. When all files
		have been processed the {@link #validateAndExtractMetadata()} method
		is called again if at least one was accepted for later validation.
		
		@param member A member of the current file set
		@return A value from the Accept enumeration to accept the file
			for validation or null to not accept the file for validatio
	*/
	public Accept acceptFile(FileSetMember member);
	
	/**
		Validate and extract metadata from the file(s) that has been added to 
		this validator. If the {@link #acceptFile(FileSetMember)} method
		returns {@link Accept#VALIDATE_IMMEDIATELY} this method is called
		and the validator should work on the last accepted file. 
		<p>
		This method is also called after all files has been processed if at
		least one file returned {@link Accept#VALIDATE_LATER}.
		<p>
		
		This method is recommended to throw an {@link InvalidDataException} or a 
		subclass if the file is not a valid file. {@link PermissionDeniedException}:s 
		are ignored by the core and will not change the validation status. An 
		exception of type {@link InvalidRelationException} can be thrown if the 
		file itself is valid, but a related file that it depends on is not. The
		core will still mark the file as invalid and store the error message,
		but extracted metadata will not be reset.
		
		@throws InvalidDataException If the validator detects an invalid
			file
		@throws InvalidRelationException If the validator detects that the
			file is valid but another file that it depends on is not, metadata
			extract will still happen
	*/
	public void validateAndExtractMetadata()
		throws InvalidDataException, InvalidRelationException;

	/**
		Reset all metadata that this class normally can set. This method is called
		if the validation of a file failed or if a file is removed from the 
		file set. Any previous metadata that may signal that a valid file is
		present should be reset.
	*/
	public void resetMetadata();

	
}
