/**
	$Id: DateFormatter.java 7646 2019-03-13 12:17:17Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.formatter;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import net.sf.basedb.core.DateFormatException;
import net.sf.basedb.util.excel.ExcelFormatter;
import net.sf.basedb.util.excel.ExcelValue;


/**
	Format a date for output in a client application. This implementation
	uses the {@link SimpleDateFormat} standard formatting routines.
	
	@author nicklas
	@version 2.0
	@base.modified $Date: 2019-03-13 13:17:17 +0100 (ons, 13 mars 2019) $
	@see java.text.SimpleDateFormat
*/
public class DateFormatter
	implements Formatter<Date>, ExcelFormatter<Date, Date>
{

	private final DateFormat dateFormat;
	private final String format;
	
	/**
		Create a new date formatter.
		@param dateFormat The date format to use when formatting
	*/
	public DateFormatter(DateFormat dateFormat)
	{
		this.dateFormat = dateFormat;
		if (dateFormat instanceof SimpleDateFormat)
		{
			this.format = ((SimpleDateFormat)dateFormat).toPattern();
		}
		else
		{
			this.format = null;
		}
	}
	
	/**
		Create a new date formatter.
		@param format The date format to use when formatting
		@see SimpleDateFormat
	*/
	public DateFormatter(String format)
	{
		this.dateFormat = new SimpleDateFormat(format);
		this.dateFormat.setLenient(true);
		this.format = format;
	}
	
	/*
		From the Formatter interface
		-------------------------------------------
	*/
	@Override
	public String format(Date value)
	{
		return value == null ? "" : dateFormat.format(value);
	}
	@Override
	public Date parseString(String value)
	{
		if (value == null) return null;
		try
		{
			return dateFormat.parse(value);
		}
		catch (ParseException ex)
		{
			try
			{
				return new Date(Long.parseLong(value));
			}
			catch (Exception ex2)
			{
				throw new DateFormatException(value + " is not a valid date for format: " + 
					format, ex);
			}
		}
	}
	// -------------------------------------------

	/**
		Get access to the underlying {@link DateFormat} object
		used for formatting and parsing dates.
		@since 2.8
	*/
	public DateFormat getDateFormat()
	{
		return dateFormat;
	}

	/**
		@since 3.15
	*/
	@Override
	public ExcelValue<Date> toExcelValue(Date value) 
	{
		return ExcelValue.asDate(value, format);
	}
	
}
