/*
	$Id: NotInReporterListFunction.java 6875 2015-04-20 09:08:18Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.jep;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DynamicQuery;
import net.sf.basedb.core.ReporterList;
import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.Restriction;
import net.sf.basedb.core.query.SqlResult;
import net.sf.basedb.util.BioAssaySetUtil;

import java.util.Stack;

import org.nfunk.jep.Node;
import org.nfunk.jep.ParseException;

/**
	A JEP function class that adds a <code>notInList(reporterListId)</code> function to a 
	JEP expression parser. The function will check if a reporter doesn't exists in the
	reporter list with the ID given as an argument to this function.
	<p>
	To be able to use this function it must be registered with the JEP
	parser and, before the expression is evaluated, a {@link SqlResult} object
	must be set.
	
	@author Nicklas
	@version 2.0
	@base.modified $Date: 2015-04-20 11:08:18 +0200 (mån, 20 apr. 2015) $
	@see Jep
	@see BioAssaySetUtil#createJepRestriction(DbControl, String, DynamicQuery)
*/
public class NotInReporterListFunction
	implements JepRestrictionFunction
{

	private DbControl dc;
	private int numParameters;

	/**
		Create a new instance of this function. The new instance cannot be used
		to dynamically evaluate expressions. It should only be used for converting
		JEP formulas to {@link Restriction}:s.
	 	@param dc Open DbControl to use when accessing database and checking permissions. 
		@see Jep#formulaToRestriction(String, JepFunction[])	
	*/
	public NotInReporterListFunction(DbControl dc)
	{
		this.dc = dc;
	}
	
	/*
		From the JepFunction interface
		-------------------------------------------
	*/
	/**
		@return The string "notInList"
	*/
	@Override
	public String getFunctionName()
	{
		return "notInList";
	}
	// -------------------------------------------
	/*
		From the JepRestrictionFunction interface
		-------------------------------------------
	*/
	/**
		Use the {@link Dynamic#isNotPartOf(ReporterList)} method to create a
		restriction checking that a reporter isn't found in the reporter list.
	*/
	@Override
	public Restriction toRestriction(Node node)
	{
		int numChildren = node.jjtGetNumChildren();
		if (numChildren != 1)
		{
			throw new BaseException("Invalid number of expressions for 'notInList' function: " + numChildren);
		}
		int reporterListId = Jep.nodeToInt(node.jjtGetChild(0));
		ReporterList list = ReporterList.getById(dc, reporterListId);
		return Dynamic.isNotPartOf(list);
	}
	// -------------------------------------------
	/*
		From the PostfixMathCommandI interface
		-------------------------------------------
	*/
	/**
		@return Always 1
	*/
	@Override
	public int getNumberOfParameters()
	{
		return 1;
	}
	@Override
	public void setCurNumberOfParameters(int n)
	{
		this.numParameters = n;
	}
	@Override
	public boolean checkNumberOfParameters(int n)
	{
		return n == 1;
	}
	@Override
	@SuppressWarnings({"rawtypes"})
	public void run(Stack stack)
		throws ParseException
	{
		if (stack == null || stack.empty()) 
		{
			throw new ParseException("Stack is empty");
		}
		Object reporterListId = stack.pop();
		throw new ParseException("'notInList(" + reporterListId + ")' function doesn't support evaluation");
	}
	// -------------------------------------------

}
