/**
	$Id: AutoChildNodeLoader.java 6875 2015-04-20 09:08:18Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.loader;

import java.util.LinkedList;
import java.util.List;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.util.overview.OverviewContext;
import net.sf.basedb.util.overview.Node;

/**
	Node loader implementation that automatically loads child nodes.
	When any of the <code>create...</code> methods are called the actual
	node loading is delegated to a parent node loader. If this loader returns
	a non-null node, {@link #loadChildNodes(DbControl, OverviewContext, Node)}
	is automatically invoked on it.
	<p>
	The automatic loading can of course only be done one step down, since
	the new children probably needs a different parent node loader. It is 
	possible to make the loading fully recursive by if the {@link
	NodeLoaderFactory} returned from {@link OverviewContext#getNodeLoaderFactory()}
	wrap all node loaders it create by an instance of this class.

	@author Nicklas
	@version 2.10
	@base.modified $Date: 2015-04-20 11:08:18 +0200 (mån, 20 apr. 2015) $
*/
public class AutoChildNodeLoader<I>
	implements NodeLoader<I>
{

	private final NodeLoader<I> parent;
	public AutoChildNodeLoader(NodeLoader<I> parent)
	{
		this.parent = parent;
	}
	/*
		From the NodeLoader interface
		-----------------------------
	*/
	/**
		Call the same method on the 'parent' node loader. If one or more
		child nodes are added to the parent node, the 
		{@link #loadChildNodes(DbControl, OverviewContext, Node)} is called
		for each of the new nodes.
		@return The node that the parent node loader creates (may be null)
	*/
	@Override
	public Node createForwardNode(DbControl dc, OverviewContext context, Node parentNode)
	{
		Node result = parent.createForwardNode(dc, context, parentNode);
		List<Node> newChildren = parentNode.getNewChildren();
		if (result != null) 
		{
			loadChildNodes(dc, context, result);
		}
		else if (newChildren != null)
		{
			for (Node n : newChildren)
			{
				loadChildNodes(dc, context, n);
			}
		}
		return result;
	}

	/**
		Call the same method on the 'parent' node loader. If one or more
		child nodes are added to the parent node, the 
		{@link #loadChildNodes(DbControl, OverviewContext, Node)} is called
		for each of the new nodes.
		@return The node that the parent node loader creates (may be null)
	*/
	@Override
	public Node createPropertyNode(DbControl dc, OverviewContext context, Node parentNode)
	{
		Node result = parent.createPropertyNode(dc, context, parentNode);
		List<Node> newChildren = parentNode.getNewChildren();
		if (result != null) 
		{
			loadChildNodes(dc, context, result);
		}
		else if (newChildren != null)
		{
			for (Node n : newChildren)
			{
				loadChildNodes(dc, context, n);
			}
		}
		return result;
	}

	/**
		Call the same method on the 'parent' node loader. If one or more
		child nodes are added to the parent node, the 
		{@link #loadChildNodes(DbControl, OverviewContext, Node)} is called
		for each of the new nodes.
		@return The node that the parent node loader creates (may be null)
	*/
	@Override
	public Node createReverseNode(DbControl dc, OverviewContext context, Node childNode)
	{
		Node result = parent.createReverseNode(dc, context, childNode);
		List<Node> newChildren = childNode.getNewChildren();
		if (result != null)
		{
			loadChildNodes(dc, context, result);
		}
		else if (newChildren != null)
		{
			for (Node n : newChildren)
			{
				loadChildNodes(dc, context, n);
			}
		}
		return result;
	}

	/**
		Call the same method on the 'parent' node loader. If a node is created
		{@link #loadChildNodes(DbControl, OverviewContext, Node)} is called
		on the new node.
		@return The node that the parent node loader creates (may be null)
	*/
	@Override
	public Node createRootNode(DbControl dc, OverviewContext context, I item)
	{
		Node result = parent.createRootNode(dc, context, item);
		if (result != null) loadChildNodes(dc, context, result);
		return result;
	}

	/**
		Call the same method on the 'parent' node loader.
		@return The result of the parent call
	*/
	@SuppressWarnings({ "unchecked", "rawtypes" })
	@Override
	public boolean loadChildNodes(DbControl dc, OverviewContext context, Node node)
	{
		if (node.isChildrenLoaded()) return false;
		boolean nodesLoaded = parent.loadChildNodes(dc, context, node);
		
		if (nodesLoaded)
		{
			// Check if all child nodes have had a chance to load it's children
			NodeLoaderFactory factory = context.getNodeLoaderFactory();
			
			List<Node> nodesToCheck = new LinkedList<Node>(node.getChildren());
			while (nodesToCheck.size() > 0)
			{
				Node child = nodesToCheck.remove(0);
				
				if (child.getNodeType() == Node.Type.FOLDER)
				{
					List<Node> grandChildren = child.getChildren();
					if (grandChildren != null && grandChildren.size() > 0) nodesToCheck.addAll(grandChildren);
				}
				else if (!child.isChildrenLoaded())
				{
					NodeLoader loader = factory.createNodeLoader(child.getItemType());
					loader.loadChildNodes(dc, context, child);
				}
			}
		}
		node.setChildrenLoaded();
		return nodesLoaded;
	}
	// --------------------------------

	@Override
	public String toString()
	{
		return super.toString() + "-->" + parent.toString();
	}
	
}
