/**
	$Id: ExtractLoader.java 7605 2019-02-26 10:10:15Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.loader;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DerivedBioAssay;
import net.sf.basedb.core.Extract;
import net.sf.basedb.core.ItemSubtype;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.core.PhysicalBioAssay;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.ItemResultIterator;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.RootRawBioAssay;
import net.sf.basedb.core.Sample;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.util.listable.ExtractToChildExtractTransformer;
import net.sf.basedb.util.listable.SourceItemTransformer;
import net.sf.basedb.util.listable.TransformContext;
import net.sf.basedb.util.overview.Fix;
import net.sf.basedb.util.overview.NodeAttribute;
import net.sf.basedb.util.overview.OverviewContext;
import net.sf.basedb.util.overview.OverviewUtil;
import net.sf.basedb.util.overview.Validator;
import net.sf.basedb.util.overview.filter.HasAttributeFilter;
import net.sf.basedb.util.overview.node.ChildNodeDirection;
import net.sf.basedb.util.overview.node.ExtractNameGenerator;
import net.sf.basedb.util.overview.Node;
import net.sf.basedb.util.overview.node.NodeFactory;

/**
	Node loader implementation for extracts. The forward-loading
	direction goes from {@link Sample}:s -&gt; {@link Extract}:s -&gt; 
	{@link PhysicalBioAssay}:s. The reverse-loading direction is the opposite 
	direction. In both cases we also need to consider pooled extracts.

	@author Nicklas
	@version 2.10
	@base.modified $Date: 2019-02-26 11:10:15 +0100 (tis, 26 feb. 2019) $
*/
public class ExtractLoader
	extends BasicItemNodeLoader<Extract>
{

	public ExtractLoader()
	{
		super(Item.EXTRACT, ALLOW_ROOT_NODE, 
				new ExtractNameGenerator("extract", "Extract"));
	}
	
	/*
		From the NodeLoader interface
		------------------------------
	*/
	/**
		Create forward-loading extract nodes from a given parent sample
		or extract. In both cases the returned node is a folder-type node
		that contains the child extracts. It can also create all-loading
		node from a biomaterial list for the member samples.
		@return The folder node for the extracts (or null if used with a list)
	*/
	@Override
	public Node createForwardNode(DbControl dc, OverviewContext context, Node parentNode)
	{
		Node returnNode = null;
		Item parentType = parentNode.getItemType();
		if (parentType == Item.SAMPLE)
		{
			returnNode = createForwardNode((Sample)parentNode.getItem(dc), dc, context, parentNode, false);
		}
		else if (parentType == Item.EXTRACT)
		{
			returnNode = createForwardNode((Extract)parentNode.getItem(dc), dc, context, parentNode, false);
		}
		else if (parentType == Item.ITEMLIST)
		{
			ItemListLoader.loadMemberNodes(this, dc, context, parentNode);
		}
		return returnNode;
	}
	
	/**
		The parent node can be one of:
		<ul>
		<li>Physical bioassay: Load the source extract(s) and return a folder-type node
		<li>Extract: Load the parent extracts and return either a folder-type or
			item-type node depending on the number of parents
		<li>Raw bioassay: Load the parent extract and return an item-type node
		<li>Derived bioassay: Load the parent extract and return an item-type node
		</ul>
		@return A folder-type or item-type node
	*/
	@Override
	public Node createReverseNode(DbControl dc, OverviewContext context, Node parentNode)
	{
		Node returnNode = null;
		Item parentItem = parentNode.getItemType();
		if (parentItem == Item.PHYSICALBIOASSAY)
		{
			returnNode = createReverseNode((PhysicalBioAssay)parentNode.getItem(dc), dc, context, parentNode);
		}
		else if (parentItem == Item.EXTRACT)
		{
			Extract e = (Extract)parentNode.getItem(dc);
			if (e.hasSingleParent())
			{
				// We already know that we have an extract as parent, since otherwise we would be in SampleLoader
				returnNode = createSingleParentReverseNode(e, dc, context, parentNode);
			}
			else
			{
				returnNode = createPooledReverseNode(e, dc, context, parentNode);
			}
		}
		else if (parentItem == Item.DERIVEDBIOASSAY)
		{
			returnNode = createReverseNode((DerivedBioAssay)parentNode.getItem(dc), dc, context, parentNode);
		}
		else if (parentItem == Item.RAWBIOASSAY)
		{
			returnNode = createReverseNode((RawBioAssay)parentNode.getItem(dc), dc, context, parentNode);
		}
		else if (parentItem == Item.ROOTRAWBIOASSAY)
		{
			returnNode = createReverseNode(((RootRawBioAssay)parentNode.getItem(dc)).getRawBioAssay(), dc, context, parentNode);
		}
		return returnNode;
	}
	
	/**
		A property node is created when moving in the reverse direction so that we
		can load child extracts+physical bioassays with the {@link ItemSubtype#getPushAnnotations()} 
		flag set. The parent node should be a sample or extract node.
		@since 3.1
	*/
	@Override
	public Node createPropertyNode(DbControl dc, OverviewContext context, Node parentNode) 
	{
		Node returnNode = null;
		Item parentType = parentNode.getItemType();
		if (parentType == Item.SAMPLE)
		{
			returnNode = createForwardNode((Sample)parentNode.getItem(dc), dc, context, parentNode, true);
		}
		else if (parentType == Item.EXTRACT)
		{
			returnNode = createForwardNode((Extract)parentNode.getItem(dc), dc, context, parentNode, true);
		}
		return returnNode;
	}
	// --------------------------------
	/*
		From the AbstractNodeLoader class
		----------------------------------
	*/
	/**
		Loads property nodes of an extract. 
		<ul>
		<li>Annotations: {@link AnnotationLoader#createPropertyNode(DbControl, OverviewContext, Node)}
		<li>Bioplate/well: {@link BioPlateLoader#createPropertyNode(DbControl, OverviewContext, Node)}
		<li>Protocol: {@link ProtocolLoader#createPropertyNode(DbControl, OverviewContext, Node)}
		<li>Kit: {@link KitLoader#createPropertyNode(DbControl, OverviewContext, Node)}
		<li>Hardware: {@link HardwareLoader#createPropertyNode(DbControl, OverviewContext, Node)}
		<li>Tag: {@link TagLoader#createPropertyNode(DbControl, OverviewContext, Node)}
		</ul>
	*/
	@Override
	protected void loadPropertyChildNodes(DbControl dc, OverviewContext context, Node extractNode)
	{
		Node parentNode = extractNode.getParent();
		getNodeLoader(context, Item.ANNOTATION).createPropertyNode(dc, context, extractNode);
		getNodeLoader(context, Item.TAG).createPropertyNode(dc, context, extractNode);
		if (parentNode == null || parentNode.getItemType() != Item.BIOPLATE)
		{
			// Do not load the bioplate if we just started from it
			getNodeLoader(context, Item.BIOPLATE).createPropertyNode(dc, context, extractNode);
		}
		getNodeLoader(context, Item.PROTOCOL).createPropertyNode(dc, context, extractNode);
		getNodeLoader(context, Item.KIT).createPropertyNode(dc, context, extractNode);
		getNodeLoader(context, Item.HARDWARE).createPropertyNode(dc, context, extractNode);
		getNodeLoader(context, Item.ANYTOANY).createPropertyNode(dc, context, extractNode);
	}

	/**
		Loads the child extracts and physical bioassays that has been created from
		this extract. Derived and raw bioassays linking to this parent extract and doesn't have
		a parent derived bioassay are also loaded.
		@see ExtractLoader#createForwardNode(DbControl, OverviewContext, Node)
		@see PhysicalBioAssayLoader#createForwardNode(DbControl, OverviewContext, Node)
		@see DerivedBioAssayLoader#createForwardNode(DbControl, OverviewContext, Node)
		@see RawBioAssayLoader#createForwardNode(DbControl, OverviewContext, Node)
	*/
	@Override
	protected void loadForwardChildNodes(DbControl dc, OverviewContext context, Node extractNode)
	{
		getNodeLoader(context, Item.PHYSICALBIOASSAY).createForwardNode(dc, context, extractNode);
		getNodeLoader(context, Item.EXTRACT).createForwardNode(dc, context, extractNode);
		getNodeLoader(context, Item.DERIVEDBIOASSAY).createForwardNode(dc, context, extractNode);
		getNodeLoader(context, Item.RAWBIOASSAY).createForwardNode(dc, context, extractNode);
	}
	
	/**
		Load either the parent sample or extract(s) and child extracts+physical bioassays
		with a subtype that has {@link ItemSubtype#getPushAnnotations()} flag set.
		@see ExtractLoader#createReverseNode(DbControl, OverviewContext, Node)
		@see SampleLoader#createReverseNode(DbControl, OverviewContext, Node)
	*/
	@Override
	protected void loadReverseChildNodes(DbControl dc, OverviewContext context, Node extractNode)
	{
		Extract extract = (Extract)extractNode.getItem(dc);
		if (extract.getParentType() == Item.EXTRACT)
		{
			getNodeLoader(context, Item.EXTRACT).createReverseNode(dc, context, extractNode);
		}
		else
		{
			getNodeLoader(context, Item.SAMPLE).createReverseNode(dc, context, extractNode);
		}
		if (extractNode.getChildNodeDirection() == ChildNodeDirection.REVERSE)
		{
			// Only if we are moving in the reverse direction (eg. not when the extract is a root node)
			getNodeLoader(context, Item.EXTRACT).createPropertyNode(dc, context, extractNode);
			getNodeLoader(context, Item.PHYSICALBIOASSAY).createPropertyNode(dc, context, extractNode);
		}
	}
	// ---------------------------------------
	/**
		Create reverse-loading extract nodes of a pooled extract. Eg. the child nodes
		are the source extracts that was pooled to create the given product.
	*/
	private Node createPooledReverseNode(Extract product, DbControl dc, OverviewContext context, Node parentNode)
	{
		NodeFactory<Extract> nf = getNodeFactory(dc, context);
		Node folderNode = null;
		ItemQuery<Extract> query = context.initQuery(
				product.getCreationEvent().getSources(), "name");
		
		// Check if we are follwing a specific paths of extracts due to DerivedBioAssay.getExtract
		Node extractPath = parentNode.getFirstParent(new HasAttributeFilter(NodeAttribute.EXTRACT_PATH));
		if (extractPath != null)
		{
			// Yes, we are following a specific paths of extracts...
			// Is the parent item still on the path?
			if (extractPath.getAttribute(NodeAttribute.EXTRACT_PATH).contains(product.getId()))
			{
				// Yes, but is the parent item one of the specified extract on a DerivedBioassay?
				if (parentNode.getFirstParent(new HasAttributeFilter(NodeAttribute.EXTRACT, product)) == null)
				{
					// No, so we have not found the 'exit' extract
					// and we must only load extracts that are part of the path
					query.restrict(Restrictions.in(Hql.property("id"), Expressions.parameter("extracts")));
					query.setParameter("extracts", extractPath.getAttribute(NodeAttribute.EXTRACT_PATH), Type.INT);
				}
			}
		}
		
		ItemResultIterator<Extract> it = query.iterate(dc);
		int numExtracts = 0;
		while (it.hasNext())
		{
			numExtracts++;
			Extract source = it.next();
			// We need to handle circular references inline because
			// we can never require that a validator implementation handles this
			Node circularNode = OverviewUtil.getCircularRef(parentNode, source);
			if (circularNode != null)
			{
				context.createFailure(Validator.CIRCULAR_REFERENCE, parentNode, 
						"Circular reference to '" + source.getName() + "'", 
						new Fix("Modify parent extracts of '" + product.getName() + "'", product),
						new Fix("Modify parent extracts of '" + source.getName() + "'", source)
					);
			}
			else
			{
				if (folderNode == null)
				{
					folderNode = new Node("pooled", "Parent extracts", parentNode, ChildNodeDirection.REVERSE);
				}
				createItemNode(nf, source, source, false, folderNode, ChildNodeDirection.REVERSE);
			}
		}
		postValidateFolder(nf, folderNode, parentNode, numExtracts == 0);
		return folderNode;
	}

	/**
		Create a reverse-loading extract node from an extract. Eg. the child node
		is the extract that is the parent of the (non-pooled) extract.
	*/
	private Node createSingleParentReverseNode(Extract bioMaterial, DbControl dc, OverviewContext context, Node parentNode)
	{
		NodeFactory<Extract> nf = getNodeFactory(dc, context);
		Extract extract = null;
		boolean denied = false;
		try
		{
			extract = (Extract)bioMaterial.getParent();
		}
		catch (PermissionDeniedException ex)
		{
			denied = true;
		}
		Node extractNode = createItemNode(nf, extract, extract, denied, 
				parentNode, ChildNodeDirection.REVERSE);
		return extractNode;
	}

	/**
		Create reverse-loading extract nodes from a physical bioassay. Eg. the child nodes
		are the extracts that has been used on a bioassay.
	*/
	private Node createReverseNode(PhysicalBioAssay bioAssay, DbControl dc, OverviewContext context, Node bioAssayNode)
	{
		Node folderNode = null;
		NodeFactory<Extract> nf = getNodeFactory(dc, context);

		ItemQuery<Extract> query = context.initQuery(bioAssay.getExtracts(0), "name");
		// If we have followed this path from derived bioassays that has specified an 
		// extract we should only load the same extracts when following the path upstreams
		Set<Integer> extracts = getExtractChain(bioAssayNode);
		if (extracts.size() > 0) 
		{
			// Since we don't know exactly how far upstreams the given extrcts are we
			// must first find all child items (recursively)
			Set<Integer> allChildren = getAllChildren(dc, extracts);
			// We keep the set of all child extracts since we need that later
			// when moving up the parent chain (see createPooledReverseNode)
			bioAssayNode.setAttribute(NodeAttribute.EXTRACT_PATH, allChildren);
			
			// An we must of course limit the extracts on this level also
			query.restrict(Restrictions.in(Hql.property("id"), Expressions.parameter("extracts")));
			query.setParameter("extracts", allChildren, Type.INT);
		}
		Iterator<Extract> it = query.iterate(dc);
		while (it.hasNext())
		{
			Extract extract = it.next();
			if (folderNode == null)
			{
				folderNode = new Node("extracts", "Extracts", bioAssayNode, ChildNodeDirection.REVERSE);
			}
			createItemNode(nf, extract, extract, false, folderNode, ChildNodeDirection.REVERSE);
		}
		postValidateFolder(nf, folderNode, bioAssayNode, extracts.size() > 0);
		return folderNode;
	}

	private Node createReverseNode(DerivedBioAssay bioAssay, DbControl dc, OverviewContext context, Node bioAssayNode)
	{
		NodeFactory<Extract> nf = getNodeFactory(dc, context);
		Extract extract = null;
		boolean denied = false;
		try
		{
			extract = bioAssay.getExtract();
		}
		catch (PermissionDeniedException ex)
		{
			denied = true;
		}
		Node extractNode = createItemNode(nf, extract, extract, denied, 
				bioAssayNode, ChildNodeDirection.REVERSE);
		return extractNode;
	}
	
	private Node createReverseNode(RawBioAssay raw, DbControl dc, OverviewContext context, Node rawNode)
	{
		NodeFactory<Extract> nf = getNodeFactory(dc, context);
		Extract extract = null;
		boolean denied = false;
		try
		{
			extract = raw.getParentExtract();
		}
		catch (PermissionDeniedException ex)
		{
			denied = true;
		}
		Node extractNode = createItemNode(nf, extract, extract, denied, 
				rawNode, ChildNodeDirection.REVERSE);
		return extractNode;
	}
	
	/**
		Create forward-loading extract nodes for the extracts that has a given
		sample as their parent.
	*/
	private Node createForwardNode(Sample sample, DbControl dc, OverviewContext context, Node parentNode,
		boolean onlyPushAnnotationTypes)
	{
		NodeFactory<Extract> nf = getNodeFactory(dc, context);
		Node folderNode = null;
		ItemQuery<Extract> query = context.initQuery(sample.getExtracts(), "name");
		if (onlyPushAnnotationTypes) OverviewUtil.restrictToPushAnnotationSubtypes(query);
		ItemResultIterator<Extract> it = query.iterate(dc);
		while (it.hasNext())
		{
			Extract extract = it.next();
			if (folderNode == null)
			{
				folderNode = new Node("extracts", "Child extracts", parentNode, ChildNodeDirection.FORWARD);
			}
			createItemNode(nf, extract, extract, false, folderNode, ChildNodeDirection.FORWARD);
		}
		postValidateFolder(nf, folderNode, parentNode, false);
		return folderNode;
	}
	
	/**
		Create forward-loading extract nodes for the (pooled) extract that has a given
		extract among their parents. If the parent node is a forward-loading node all child 
		nodes are loaded, otherwise only child nodes that have an item with a subtype that
		has {@link ItemSubtype#getPushAnnotations()} set.
	*/
	private Node createForwardNode(Extract parentExtract, DbControl dc, OverviewContext context, Node parentNode, 
		boolean onlyPushAnnotationTypes)
	{
		NodeFactory<Extract> nf = getNodeFactory(dc, context);
		Node folderNode = null;
		ItemQuery<Extract> query = context.initQuery(parentExtract.getChildExtracts(), "name");
		if (onlyPushAnnotationTypes) OverviewUtil.restrictToPushAnnotationSubtypes(query);
		ItemResultIterator<Extract> it = query.iterate(dc);
		while (it.hasNext())
		{
			Extract childExtract = it.next();
			// We need to handle circular references inline because
			// we can never require that a validator implementation handles this
			Node circularNode = OverviewUtil.getCircularRef(parentNode, childExtract);
			if (circularNode != null)
			{
				context.createFailure(Validator.CIRCULAR_REFERENCE, parentNode, 
						"Circular reference to '" + childExtract.getName() + "'", 
						new Fix("Modify parent extracts of '" + childExtract.getName() + "'", childExtract),
						new Fix("Modify parent extracts of '" + parentExtract.getName() + "'", parentExtract)
						);
			}
			else
			{
				if (folderNode == null)
				{
					folderNode = new Node("extracts", "Child extracts", parentNode, ChildNodeDirection.FORWARD);
				}
				createItemNode(nf, childExtract, childExtract, false, folderNode, ChildNodeDirection.FORWARD);
			}
		}
		postValidateFolder(nf, folderNode, parentNode, false);
		return folderNode;
	}

	private Set<Integer> getExtractChain(Node node)
	{
		List<Node> nodesWithExtract = node.findAll(new HasAttributeFilter(NodeAttribute.EXTRACT));
		Set<Integer> extracts = new HashSet<Integer>(nodesWithExtract.size());
		for (Node n : nodesWithExtract)
		{
			extracts.add(n.getAttribute(NodeAttribute.EXTRACT).getId());
		}
		return extracts;
	}
	
	private Set<Integer> getAllChildren(DbControl dc, Set<Integer> parents)
	{
		SourceItemTransformer transformer = new ExtractToChildExtractTransformer(true);
		return transformer.transform(new TransformContext(dc), parents);
	}
	
}
