/**
	$Id: PlatformValidator.java 5814 2011-10-18 11:46:59Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.validator;

import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.FileStoreEnabled;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.PlatformVariant;
import net.sf.basedb.core.Project;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.util.overview.Fix;
import net.sf.basedb.util.overview.OverviewContext;
import net.sf.basedb.util.overview.Validator;
import net.sf.basedb.util.overview.Node;

/**
	Validator implementation for platforms/variants. Validation rules:
	<ul>
	<li>Access denied: {@link Validator#DENIED_PLATFORM}
	<li>Non-default platform: {@link Validator#NONDEFAULT_PLATFORM}
	</ul>

	@author Nicklas
	@version 2.10
	@base.modified $Date: 2011-10-18 13:46:59 +0200 (tis, 18 okt. 2011) $
*/
public class PlatformValidator
	extends NameableNodeValidator<Platform>
{
	
	public PlatformValidator()
	{
		super(null, Validator.DENIED_PLATFORM);
	}
	
	/* 
		From BasicNodeValidator class
		-----------------------------
	*/
	/**
		Checks if the platform is the same as
		the default for the project.
	*/
	@Override
	public void postValidate(DbControl dc, OverviewContext context, Node node, Node parentNode)
	{
		super.postValidate(dc, context, node, parentNode);
		Platform platform = (Platform)node.getItem(dc);
		PlatformVariant variant = null;
		boolean failedVariant = false;
		BasicItem parentItem = parentNode.getItem(dc);
		try
		{
			variant = ((FileStoreEnabled)parentItem).getVariant();
		}
		catch (Throwable t)
		{
			// Failure to load the variant is detected by other validators
			failedVariant = true;
		}
		String parentTypeName = parentNode.getItemType().toString().toLowerCase();
		boolean canChangePlatform = false;
		if (parentNode.getItemType() == Item.RAWBIOASSAY)
		{
			RawBioAssay rba = (RawBioAssay)parentItem;
			canChangePlatform = rba.getNumDbSpots() == 0;
		}
		
		
		Project project = context.getProject();
		if (project != null)
		{
			if (variant != null)
			{
				// The item has a platform variant, so we check that
				if (!project.isDefaultItem(variant) && !project.isDefaultItem(platform) &&
					(project.hasDefaultForItem(variant) || project.hasDefaultForItem(platform)))
				{
					context.createFailure(Validator.NONDEFAULT_PLATFORM, parentNode, null,
						canChangePlatform ? new Fix("Change platform/variant of " + parentTypeName, parentItem) : null,
						new Fix("Add '" + variant.getName() + "' as default platform variant to project", project),
						new Fix("Add '" + platform.getName() + "' as default platform to project", project)
					);
				}
			}
			else if (!failedVariant)
			{
				// The item only has a platform, so we check that
				if (!project.isDefaultItem(platform) && project.hasDefaultForItem(platform))
				{
					context.createFailure(Validator.NONDEFAULT_PLATFORM, parentNode, null,
						canChangePlatform ? new Fix("Change platform of " + parentTypeName, parentItem) : null,
						new Fix("Add '" + platform.getName() + "' as default platform to project", project)
					);
				}
			}
		}
	}
	
	/**
		@return Always null
	*/
	@Override
	protected Fix getMissingItemFix(DbControl dc, Node parentNode)
	{
		return null;
	}
	// --------------------------------
}
