/**
	$Id: ProtocolParameterValidator.java 6939 2015-08-21 12:32:11Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.validator;

import java.util.List;

import net.sf.basedb.core.Annotatable;
import net.sf.basedb.core.Annotation;
import net.sf.basedb.core.AnnotationSet;
import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.util.overview.Fix;
import net.sf.basedb.util.overview.OverviewContext;
import net.sf.basedb.util.overview.Validator;
import net.sf.basedb.util.overview.Node;

/**
	Validator implementation for protocol parameters. The node containing
	the parameter is expected to live in a folder-type node that
	has a protocol as it's parent. The protocol node must have a 
	parent node that contains an {@link Annotatable} item since this
	is were the	parameter values are stored. Validation rules:
	<ul>
	<li>No value for parameter: {@link Validator#MISSING_PARAMETER}
	<li>Invalid parameter value: {@link Validator#ANNOTATION_INVALID_VALUE}
	</ul>
	
	@author Nicklas
	@version 2.10
	@base.modified $Date: 2015-08-21 14:32:11 +0200 (fre, 21 aug. 2015) $
*/
public class ProtocolParameterValidator
	extends NameableNodeValidator<AnnotationType>
{
	
	private Node lastProtocolNode;
	private Node parentItemNode;
	private BasicItem parentItem;
	private AnnotationSet annotationSet;
	
	public ProtocolParameterValidator()
	{
		super(null, null);
	}

	
	/* 
		From BasicNodeValidator class
		-----------------------------
	*/
	@Override
	public void postValidate(DbControl dc, OverviewContext context, Node node, Node protocolNode)
	{
		super.postValidate(dc, context, node, protocolNode);
		getParentInformation(dc, protocolNode);
		
		// Safety measure if a parent item is not found -- shouldn't happen
		if (parentItem == null) return;

		AnnotationType parameter = (AnnotationType)node.getItem(dc);
		// Do we have a value for the protocol parameter?
		if (annotationSet == null || !annotationSet.hasAnnotation(parameter, Annotation.Source.PRIMARY))
		{
			// NO :(
			context.createFailure(Validator.MISSING_PARAMETER, parentItemNode, 
				"Missing value for parameter: " + parameter.getName(), 
				new Fix("Add value for: " + parameter.getName(), parentItem, parameter, false));
		}
		else
		{
			// YES, check that the value(s) are valid
			Annotation a = annotationSet.getAnnotation(parameter);
			try
			{
				List<?> values = a.getValues();
				if (values != null)
				{
					for (Object value : values)
					{
						parameter.validateAnnotationValue(value);
					}
				}
			}
			catch (InvalidDataException ex)
			{
				context.createFailure(Validator.ANNOTATION_INVALID_VALUE, parentItemNode, ex.getMessage(),
					new Fix("Change parameter value", parentItem, a),
					new Fix("Change parameter restrictions", parameter)
				);
			}
		}
	}
	
	/**
		@return Always "null"
	*/
	@Override
	protected Fix getMissingItemFix(DbControl dc, Node parentNode)
	{
		return null;
	}
	// ----------------------------
	
	/**
		Load parent item information from the current protocol node.
		We expect: protocolNode -&gt; annotatableNode
	*/
	private void getParentInformation(DbControl dc, Node protocolNode)
	{
		// No need to reload if we are still in the same folder
		if (lastProtocolNode == protocolNode) return;
		
		// Reset existing
		lastProtocolNode = protocolNode;
		parentItemNode = null;
		parentItem = null;
		annotationSet = null;
		
		// Get parent item of the protocol
		parentItemNode = protocolNode.getParent();
		if (parentItemNode == null) return;
		parentItem = parentItemNode.getItem(dc);
		if (!(parentItem instanceof Annotatable)) return;
		
		Annotatable annotatable = (Annotatable)parentItem;
		if (annotatable.isAnnotated()) annotationSet = annotatable.getAnnotationSet();
	}

}
