/**
	$Id: TagValidator.java 5807 2011-10-14 13:08:23Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.validator;

import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemSubtype;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.core.Subtypable;
import net.sf.basedb.core.Tag;
import net.sf.basedb.util.overview.Fix;
import net.sf.basedb.util.overview.OverviewContext;
import net.sf.basedb.util.overview.Validator;
import net.sf.basedb.util.overview.Node;

/**
	Validator implementation for tags. Validation rules:
	<ul>
	<li>Access denied: {@link Validator#DENIED_TAG}
	<li>Incorrect tag type: {@link Validator#INCORRECT_TAGTYPE} 
	</ul>

	@author Nicklas
	@since 3.0
	@base.modified $Date: 2011-10-14 15:08:23 +0200 (fre, 14 okt. 2011) $
*/
public class TagValidator
	extends NameableNodeValidator<Tag>
{
	
	public TagValidator()
	{
		super(null, Validator.DENIED_TAG);
	}

	
	/* 
		From BasicValidator class
		-------------------------
	*/
	/**
		If the parent item has a subtype that is related to a TAG subtype, report
		the missing item as a {@link Validator#MISSING_TAG} failure. Otherwise,
		ignore the missing tag.
		@return Always false
	*/
	@Override
	public boolean preMissingItem(DbControl dc, OverviewContext context, Node parentNode)
	{
		BasicItem parentItem = parentNode.getItem(dc);
		ItemSubtype expectedSubtype = getExpectedTagSubtype(dc, parentItem);
		if (expectedSubtype != null)
		{
			context.createFailure(Validator.MISSING_TAG, parentNode, 
					"Missing " + expectedSubtype.getName().toLowerCase(), getMissingItemFix(dc, parentNode));
		}
		return false;
	}

	/**
		Checks if the tag is of the correct subtype.
	*/
	@Override
	public void postValidate(DbControl dc, OverviewContext context, Node node, Node parentNode)
	{
		super.postValidate(dc, context, node, parentNode);
		Tag tag = (Tag)node.getItem(dc);
		
		BasicItem parentItem = parentNode.getItem(dc);
		ItemSubtype expectedSubtype = getExpectedTagSubtype(dc, parentItem);
		try
		{
			ItemSubtype subtype = tag.getItemSubtype();
			if (expectedSubtype != null && !expectedSubtype.equals(subtype))
			{
				ItemSubtype mySubtype = ((Subtypable)parentItem).getItemSubtype();
				Fix subtypeFix = subtype == null ? null : 
					new Fix("Change related tag subtype of '" + mySubtype.getName() + "' to '" + subtype.getName() + "'", mySubtype);

				context.createFailure(Validator.INCORRECT_TAGTYPE, parentNode, 
					"Expected a tag with subtype: " + expectedSubtype.getName(),
					new Fix("Change tag of '" + parentNode.getTitle() + "'", parentItem),
					new Fix("Change tag type of '" + tag.getName() + "' to '" + expectedSubtype.getName() + "'", tag),
					subtypeFix
				);
			}
		}
		catch (PermissionDeniedException ex)
		{}
	}

	/**
		@return Suggested fix is to add a tag to the parent item
	*/
	@Override
	protected Fix getMissingItemFix(DbControl dc, Node parentNode)
	{
		BasicItem parentItem = parentNode.getItem();		
		return new Fix("Add tag to '" + parentNode.getTitle() + "'", parentItem);
	}
	// --------------------------------
	
	/**
		Get the subtype of the associated tag that we expect for the given
		parent item.
	*/
	protected ItemSubtype getExpectedTagSubtype(DbControl dc, BasicItem parentItem)
	{
		ItemSubtype expectedSubtype = null;
		try
		{
			if (parentItem instanceof Subtypable)
			{
				expectedSubtype = ItemSubtype.getRelatedSubtype(dc, 
						(Subtypable)parentItem, Item.TAG, 0);
			}
		}
		catch (PermissionDeniedException ex)
		{}
		return expectedSubtype;
	}
	
}
