/*
	$Id: ColumnMapper.java 7690 2019-04-05 12:19:47Z nicklas $
	
	Copyright (C) 2006, 2007 Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.
	
	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.parser;

import java.text.NumberFormat;
import java.util.Date;

import net.sf.basedb.util.formatter.Formatter;
import net.sf.basedb.util.parser.FlatFileParser.Data;

/**
	Returns the string found in the column given by the index
	set in the constructor. If the index is larger than the number
	of columns, an empty string is returned.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2019-04-05 14:19:47 +0200 (fre, 05 apr. 2019) $
*/
public class ColumnMapper
	implements Mapper
{
	private final int index;
	private final String name;
	private final NumberFormat parser;
	private final Formatter<Date> dateParser;
	private final boolean nullIfException;

	/**
		Create a new column mapper.
		@param index The index of the data column to use, starting at 0
		@param name An optional name of the column, use in the <code>toString</code>
			method
	 */
	public ColumnMapper(int index, String name)
	{
		this(index, name, null, null, false);
	}
	
	/**
		Create a new column mapper using a specific number format.
		@param index The index of the data column to use, starting at 0
		@param name An optional name of the column, use in the <code>toString</code>
			method
		@param nullIfException If TRUE, the mapper returns null for unparsable numeric
			values, otherwise an excption is thrown
		@param parser The parser to use or null to use Double.valueOf()
		@since 2.4
		@deprecated In 3.15, use {@link ColumnMapper#ColumnMapper(int, String, NumberFormat, Formatter, boolean)} instead
	*/
	@Deprecated
	public ColumnMapper(int index, String name, NumberFormat parser, boolean nullIfException)
	{
		this(index, name, parser, null, nullIfException);
	}
	
	/**
		Create a new column mapper using a specific number or date format.
		@param index The index of the data column to use, starting at 0
		@param name An optional name of the column, use in the <code>toString</code>
			method
		@param nullIfException If TRUE, the mapper returns null for unparsable numeric
			values, otherwise an excption is thrown
		@param parser The parser to use for numeric values or null to use Double.valueOf()
		@param dateParser The parser to use for date values
		@since 3.15
	*/
	public ColumnMapper(int index, String name, NumberFormat parser, Formatter<Date> dateParser, boolean nullIfException)
	{
		this.index = index;
		this.name = name;
		this.parser = parser;
		this.dateParser = dateParser;
		this.nullIfException = nullIfException;
	}

	/*
		From the Mapper interface
		-------------------------------------------
	*/
	@Override
	@Deprecated
	public String getValue(Data data)
	{
		return getString(data);
	}
	@Override
	public String getString(Data data)
	{
		return index < data.columns() ? data.getString(index) : null;
	}
	@Override
	public Integer getInt(Data data)
	{
		return index < data.columns() ? data.getInt(index, parser, nullIfException) : null;
	}
	@Override
	public Long getLong(Data data)
	{
		return index < data.columns() ? data.getLong(index, parser, nullIfException) : null;
	}
	@Override
	public Float getFloat(Data data)
	{
		return index < data.columns() ? data.getFloat(index, parser, nullIfException) : null;
	}
	@Override
	public Double getDouble(Data data)
	{
		return index < data.columns() ? data.getDouble(index, parser, nullIfException) : null;
	}
	@Override
	public Date getDate(Data data)
	{
		return index < data.columns() ? data.getDate(index, dateParser, nullIfException) : null;
	}
	// -------------------------------------------
	
	/*
		From the Object class
		-------------------------------------------
	*/
	@Override
	public String toString()
	{
		return name == null ? "\\" + index + "\\" : "\\" + name + "\\";
	}
	// -------------------------------------------

}

