/**
	$Id: StaticKeyManager.java 5362 2010-06-16 12:28:36Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.ssl;

import java.net.Socket;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.Principal;
import java.security.PrivateKey;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;

import javax.net.ssl.X509KeyManager;

import net.sf.basedb.core.FileServer;

/**
	A key manager implementation that always selects a specified keystore entry.
	We use it to make sure that a client certificate that has been registered
	with a {@link FileServer} is always selected for authoriation independent of 
	other settings.

	@author Nicklas
	@since 2.16
	@base.modified $Date: 2010-06-16 14:28:36 +0200 (ons, 16 juni 2010) $
*/
public class StaticKeyManager
	implements X509KeyManager
{

	private final KeyStore keyStore;
	private final String alias;
	private final char[] password;
	
	/**
		Create a new key manager
		@param keyStore The key-store containing the client certificate
		@param alias The alias of the key-store entry to use, or null to use
			the first entry in the key-store
		@param password The password that is required to unlock the entry
	*/
	public StaticKeyManager(KeyStore keyStore, String alias, String password) 
		throws KeyStoreException
	{
		this.keyStore = keyStore;
		this.alias = alias == null ? keyStore.aliases().nextElement() : alias;
		this.password = password == null ? null : password.toCharArray();
	}

	/**
		@return Always the alias specified in the constructor
	 */
	@Override
	public String chooseClientAlias(String[] keyType, Principal[] issuers, Socket socket)
	{
		return alias;
	}

	/**
		@return Always null
	*/
	@Override
	public String chooseServerAlias(String keyType, Principal[] issuers, Socket socket)
	{
		return null;
	}

	@Override
	public X509Certificate[] getCertificateChain(String alias)
	{
		X509Certificate[] certificates = null;
		try
		{
			Certificate[] certs = keyStore.getCertificateChain(alias);
			if (certs != null)
			{
				certificates = new X509Certificate[certs.length];
				System.arraycopy(certs, 0, certificates, 0, certs.length);
			}
		}
		catch (KeyStoreException ex)
		{}
		return certificates;
	}

	/**
		@return An array with the alias given in the constructor as the only element
	*/
	@Override
	public String[] getClientAliases(String keyType, Principal[] issuers)
	{
		return new String[] { alias };
	}

	@Override
	public PrivateKey getPrivateKey(String alias)
	{
		PrivateKey key = null;
		try
		{
			key = (PrivateKey)keyStore.getKey(alias, password);
		}
		catch (Exception ex)
		{}
		return key;
	}

	/**
		@return Always null
	*/
	@Override
	public String[] getServerAliases(String keyType, Principal[] issuers)
	{
		return null;
	}

}
