/**
	$Id: JepIntensityTransformer.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.plugins;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BioAssay;
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DynamicSpotQuery;
import net.sf.basedb.core.IntensityTransform;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.ProgressReporter;
import net.sf.basedb.core.RawDataType;
import net.sf.basedb.core.RequestInformation;
import net.sf.basedb.core.SpotBatcher;
import net.sf.basedb.core.StringParameterType;
import net.sf.basedb.core.StringUtil;
import net.sf.basedb.core.Transformation;
import net.sf.basedb.core.VirtualColumn;
import net.sf.basedb.core.plugin.AbstractAnalysisPlugin;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.core.plugin.InteractivePlugin;
import net.sf.basedb.core.plugin.Permissions;
import net.sf.basedb.core.plugin.Request;
import net.sf.basedb.core.plugin.Response;
import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.Expression;
import net.sf.basedb.core.query.Selects;
import net.sf.basedb.core.signal.SignalHandler;
import net.sf.basedb.core.signal.SignalTarget;
import net.sf.basedb.core.signal.ThreadSignalHandler;
import net.sf.basedb.util.BioAssaySetUtil;
import net.sf.basedb.util.Enumeration;
import net.sf.basedb.util.Values;

/**
	This plugin transform channel intensities using mathematical formulas
	entered by the user.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2012-09-14 09:28:35 +0200 (fre, 14 sep. 2012) $
*/
public class JepIntensityTransformer
	extends AbstractAnalysisPlugin
	implements InteractivePlugin, SignalTarget
{

	private static final Set<Permissions> permissions = new HashSet<Permissions>();
	private static final Set<GuiContext> guiContexts = Collections.unmodifiableSet(
			new HashSet<GuiContext>(Arrays.asList(CONTEXT_BIOASSAYSET, CONTEXT_BIOASSAYS)));

	private ThreadSignalHandler signalHandler;
	private RequestInformation configureJob;
	private List<PluginParameter<String>> channelParameters;

	/**
	   Constructor should be empty. 
	*/
	public JepIntensityTransformer()
	{}

	/*
		From the Plugin interface
		-------------------------------------------
	*/
	@Override
	public boolean supportsConfigurations()
	{
		return false;
	}
	@Override
	public boolean requiresConfiguration()
	{
		return false;
	}
	/**
	 	Request use access to Experiment:s and read access to Job:s.
	*/
	@Override
	public Collection<Permissions> getPermissions()
	{
		if (permissions.size() == 0)
		{
			permissions.add(new Permissions(Item.EXPERIMENT, null, EnumSet.of(Permission.USE)));
			permissions.add(new Permissions(Item.JOB, null, EnumSet.of(Permission.READ)));
		}
		return permissions;
	}
	@Override
	public void run(Request request, Response response, ProgressReporter progress)
	{
		if (signalHandler != null) signalHandler.setWorkerThread(null);
		DbControl dc = sc.newDbControl();
		try
		{
			if (progress != null) progress.display(5, "Initialising transformation expressions...");

			// Get all parameters
			BioAssaySet source = getSourceBioAssaySet(dc);
			List<BioAssay> bioAssays = getSourceBioAssays(dc);
			String childName = Values.getString((String)job.getValue(CHILD_NAME), source.getName());
			String childDescription = (String)job.getValue(CHILD_DESCRIPTION);
			String transformationName = (String)job.getValue(TRANSFORMATION_NAME);
			String resultTransform = (String)job.getValue("resultTransform");
			RawDataType rdt = source.getRawDataType();
			
			// and, the current job
			Job currentJob = getCurrentJob(dc);
			
			// Create Transformation
			Transformation t = source.newTransformation(currentJob);
			t.setName(StringUtil.trimString(transformationName, Transformation.MAX_NAME_LENGTH));
			dc.saveItem(t);

			// Create the filtered BioAssaySet in the same datacube but a new layer
			BioAssaySet child = t.newProduct(null, "new", true);
			child.setName(childName);
			child.setDescription(childDescription);
			IntensityTransform result = source.getIntensityTransform();
			if (resultTransform != null) result = IntensityTransform.valueOf(resultTransform);
			child.setIntensityTransform(result);
			dc.saveItem(child);

			// Create the query
			DynamicSpotQuery query = source.getSpotData();
			query.setDisableRawJoinOnMergedData(true);
			query.select(Dynamic.select(VirtualColumn.COLUMN));
			query.select(Dynamic.select(VirtualColumn.POSITION));
			
			// Restrict to selected bioassays
			restrictSource(query, bioAssays);

			// Transformation expressions
			for (int ch = 1; ch <= rdt.getChannels(); ++ch)
			{
				String expression = (String)job.getValue("ch"+ch+".expression");
				Expression chExpression = null;
				if (expression != null)
				{
					chExpression = BioAssaySetUtil.createJepExpression(dc, expression, query);
				}
				else
				{
					chExpression = Dynamic.column(VirtualColumn.channelIntensity(ch));
				}
				query.select(Selects.expression(chExpression, "ch" + ch));
			}

			// Create the batchers we need
			if (progress != null) progress.display(10, "Transforming intensities ("+source.getNumSpots()+" total)...");
			ThreadSignalHandler.checkInterrupted();
			SpotBatcher spotBatcher = child.getSpotBatcher();
			spotBatcher.insert(query);

			if (progress != null) progress.display(90, "Committing transaction ("+source.getNumSpots()+" done)...");
			dc.commit();

			if (progress != null) progress.display(100, "Done.");

			int childSpots = child.getNumSpots();
			response.setDone(childSpots +" spots transformed");
		}
		catch (Throwable t)
		{
			response.setError(t.getMessage(), Arrays.asList(t));
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	// -------------------------------------------

	/*
		From the InteractivePlugin interface
		-------------------------------------------
	*/
	@Override
	public Set<GuiContext> getGuiContexts()
	{
		return guiContexts;
	}
	@Override
	public String isInContext(GuiContext context, Object item)
	{
		String message = super.isInContext(context, item);
		if (message == null)
		{
			BioAssaySet bas = null;
			if (context.equals(CONTEXT_BIOASSAYSET))
			{
				bas = (BioAssaySet)item;
			}
			else
			{
				DbControl dc = sc.newDbControl();
				try
				{
					bas = getCurrentBioAssaySet(dc);
				}
				finally
				{
					if (dc != null) dc.close();
				}
			}
			if (bas.getNumSpots() <= 0)
			{
				message = "This plug-in requires spot data in the database";
			}
		}
		return message;
	}
	@Override
	public RequestInformation getRequestInformation(GuiContext context, String command) 
		throws BaseException
	{
		RequestInformation requestInformation = null;
		if (command.equals(Request.COMMAND_CONFIGURE_JOB))
		{
			requestInformation = getConfigureJobParameters(context);
		}
		return requestInformation;
	}
	@Override
	public void configure(GuiContext context, Request request, Response response)
	{
		String command = request.getCommand();
		try
		{
			if (command.equals(Request.COMMAND_CONFIGURE_JOB))
			{
				RequestInformation ri = getConfigureJobParameters(context);
				List<Throwable> errors = validateRequestParameters(ri.getParameters(), request);
				if (errors != null)
				{
					response.setError(errors.size() + " invalid parameter(s) were found in the request", errors);
					return;
				}
				
				// Source bioassay set
				storeValue(job, request, ri.getParameter(SOURCE_BIOASSAYSET));
				
				// Selected bioassays
				storeValue(job, request, ri.getParameter(SOURCE_BIOASSAYS));
				
				// Child name, description and transformation
				storeValue(job, request, ri.getParameter(CHILD_NAME));
				storeValue(job, request, ri.getParameter(CHILD_DESCRIPTION));
				storeValue(job, request, ri.getParameter(TRANSFORMATION_NAME));
				
				// Channel transformations
				storeValue(job, request, ri.getParameter("resultTransform"));
				for (int ch = 1; ch <= channelParameters.size(); ++ch)
				{
					storeValue(job, request, channelParameters.get(ch-1));
				}
				
				BioAssaySet bioAssaySet = (BioAssaySet)job.getValue(SOURCE_BIOASSAYSET);
				response.setSuggestedJobName("Intensity transform on bioassay set '" + bioAssaySet.getName() + "'");
				
				response.setDone("Job configuration complete", Job.ExecutionTime.SHORT);
			}
		}
		catch (Throwable ex)
		{
			response.setError(ex.getMessage(), Arrays.asList(ex));
		}
	}
	// -------------------------------------------
	/*
		From the SignalTarget interface
		-------------------------------------------
	*/
	@Override
	public SignalHandler getSignalHandler()
	{
		signalHandler = new ThreadSignalHandler();
		return signalHandler;
	}
	// -------------------------------------------

	private RequestInformation getConfigureJobParameters(GuiContext context)
	{
		DbControl dc = null;
		try
		{
			if (configureJob == null)
			{
				// Get current raw data type
				dc = sc.newDbControl();
				BioAssaySet bas = getCurrentBioAssaySet(dc);
				RawDataType rdt = bas.getRawDataType();
				
				// Parameters for CONFIGURE_JOB
				List<PluginParameter<?>> parameters = new ArrayList<PluginParameter<?>>();

				// Source and child bioassay set parameters
				parameters.add(getSourceBioAssaySetParameter(null, null));
				parameters.add(getSourceBioAssaysParameter(null, null));
				parameters.add(getChildNameParameter(null, null, bas.getName()));
				parameters.add(getChildDescriptionParameter(null, null, null));
				parameters.add(getTransformationNameParameter(null, null, null));
				
				// Intensity transformation parameters
				channelParameters = new ArrayList<PluginParameter<String>>(rdt.getChannels());
				for (int ch = 1; ch <= rdt.getChannels(); ++ch)
				{
					String parameterName = "ch" + ch + ".expression";
					String defaultExpression = "rawCh(" + ch + ")";
					PluginParameter<String> chParameter = new PluginParameter<String>(
						parameterName, "Ch " + ch + " transformation",
						"The expression to use for calculating new intensities in channel " + ch,
						new StringParameterType(255, defaultExpression, true));
					parameters.add(chParameter);
					channelParameters.add(chParameter);
				}
				Enumeration<String, String> transforms = new Enumeration<String, String>();
				for (IntensityTransform t : IntensityTransform.values())
				{
					transforms.add(t.name(), t.toString());
				}
				parameters.add(
					new PluginParameter<String>(
						"resultTransform",
						"Resulting transform",
						"The type of spot intensity values this transformation generates. If not " +
						"specified it is expected to generate the same type of values as are found " +
						"in the source bioassay set",
						new StringParameterType(255, null, false, 1, 0, 0, transforms)
					)
				);
				
				configureJob = new RequestInformation
				(
					Request.COMMAND_CONFIGURE_JOB,
					"Specify transformation formulas",
					"You should enter formulas for the transformation for each channel.",
					parameters, "jep_intensity_transformer.jsp"
				);
			}
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return configureJob;
	}
}
