/*
	$Id $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.ClonableProperty;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DynamicReporterQuery;
import net.sf.basedb.core.DynamicResultIterator;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.ExtendedProperties;
import net.sf.basedb.core.ExtendedProperty;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemProxy;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.Reporter;
import net.sf.basedb.core.ReporterCloneBatcher;
import net.sf.basedb.core.ReporterCloneTemplate;
import net.sf.basedb.core.ReporterCloneBatcher.CloneSource;
import net.sf.basedb.core.data.ReporterData;
import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.SqlResult;
import net.sf.basedb.util.ConsoleProgressReporter;

import java.util.Date;
import java.util.List;
import java.util.Set;

public class TestReporterCloneTemplate
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing reporter clone templates");
		write_header();
		// Standard tests: create, load, list
		
		List<ExtendedProperty> properties = ExtendedProperties.getProperties("ReporterData");
		
		ClonableProperty symbol = ClonableProperty.createFromRegular(ClonableProperty.SYMBOL);
		ClonableProperty ex1 = ClonableProperty.createFromExtended(properties.get(0).getName());
		
		int id = test_create("Test reporter clone template", symbol, ex1);
		test_load(id);
		test_list(-1);

		boolean wasSetup = TestAnalyzeUtil.setupExperiment();
		try
		{
	
			int experimentId = TestAnalyzeUtil.getItemId("experiment.genepix");
			if (experimentId == 0)
			{
				ok = false;
				write("Test experiment was not setup correctly.");
			}

			test_clone_reporters(id, experimentId, CloneSource.VIRTUAL, 572);
			test_list_cloned_reporters(experimentId);

			test_externalId("B-actin_1/4", experimentId);
			
			test_clone_reporters(id, experimentId, CloneSource.RAW, 572);
			test_list_cloned_reporters(experimentId);

			// Standard test: Delete
			if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
			test_remove_cloned_reporters(experimentId);
			test_delete(id);
		}
		finally
		{
			if (wasSetup) TestAnalyzeUtil.cleanupExperiment();
			write("++Testing reporter clone templates "+(ok ? "OK" : "Failed")+"\n");
		}
		return ok;
	}

	static int test_create(String name, ClonableProperty... properties)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.REPORTERCLONETEMPLATE)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterCloneTemplate rct = ReporterCloneTemplate.getNew(dc);
			rct.setName(name);
			rct.setDescription("Added at "+new Date());
			if (properties != null)
			{
				for (ClonableProperty cp : properties)
				{
					rct.addClonableProperty(cp);
				}
			}
			dc.saveItem(rct);
			dc.commit();
			id = rct.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(rct, false);
			write_item(0, rct);
			write("--Create reporter clone template OK");
		}
		catch (Throwable ex)
		{
			write("--Create reporter clone template FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterCloneTemplate rct = ReporterCloneTemplate.getById(dc, id);
			write_item(0, rct);
			write("--Load reporter clone template OK");
		}
		catch (Throwable ex)
		{
			write("--Load reporter clone template FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<ReporterCloneTemplate> l = ReporterCloneTemplate.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List reporter clone templates OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List reporter clone templates FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterCloneTemplate rct = ReporterCloneTemplate.getById(dc, id);
			dc.deleteItem(rct);
			Set<ItemProxy> using = rct.getUsingItems();
			if (using.size() > 0) 
			{
			   throw new BaseException(rct + " is used by " + using);
			}
			dc.commit();
			write("--Delete reporter clone template OK");
		}
		catch (Throwable ex)
		{
			write("--Delete reporter clone template FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tLocked\tProperties");
			write("-- \t-- \t--------- \t-----------\t------\t----------");
		}
	}
	static void write_item(int i, ReporterCloneTemplate rct)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.println(i+":\t"+rct.getId()+"\t"+rct.getName()+"\t"+rct.getDescription()+
					"\t"+rct.isLocked() + "\t" + rct.getClonableProperties());
		}
	}
	static void write(String message)
	{
		System.out.println(message);
	}

	static void test_clone_reporters(int id, int experimentId, CloneSource source, int numClonedExpected)
	{
		if (id == 0 || experimentId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterCloneTemplate template = ReporterCloneTemplate.getById(dc, id);
			Experiment exp = Experiment.getById(dc, experimentId);
			
			ReporterCloneBatcher batcher = exp.getReporterCloneBatcher(dc);
			int numCloned = batcher.cloneReporters(source, template, 
				TestUtil.getSilent() ? null : new ConsoleProgressReporter());
			if (numCloned != numClonedExpected && numClonedExpected >= 0)
			{
				throw new BaseException("Expected "+numClonedExpected+" reporters, not "+numCloned);
			}
			
			dc.commit();
			if (!TestUtil.getSilent()) write("");
			write("--Clone reporters OK");
		}
		catch (Throwable ex)
		{
			write("--Clone reporters FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_remove_cloned_reporters(int experimentId)
	{
		if (experimentId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment exp = Experiment.getById(dc, experimentId);
			
			ReporterCloneBatcher batcher = exp.getReporterCloneBatcher(dc);
			batcher.dropClonedReporters();
			
			dc.commit();
			write("--Remove cloned reporters OK");
		}
		catch (Throwable ex)
		{
			write("--Remove cloned reporters FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list_cloned_reporters(int experimentId)
	{
		if (experimentId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment exp = Experiment.getById(dc, experimentId);
			
			DynamicReporterQuery query = Reporter.getDynamicQuery(exp);
			ReporterCloneTemplate template = exp.getVirtualDb().getReporterCloneTemplate();
			
			StringBuilder header = new StringBuilder("  ");
			int numProperties = 0;
			for (ClonableProperty cp : template.getClonableProperties())
			{
				header.append("\t").append(cp.getTitle());
				query.select(Dynamic.selectReporter(cp.getName()));
				numProperties++;
			}
			if (!TestUtil.getSilent()) 
			{
				write(header.toString());
				write(header.toString().replaceAll("[^\t]", "-"));
			}
			
			DynamicResultIterator it = query.iterate(dc);
			int num = 1;
			while (it.hasNext() && num < 50)
			{
				SqlResult row = it.next();
				StringBuilder data = new StringBuilder();
				data.append(num);
				for (int index = 1; index <= numProperties; index++)
				{
					data.append("\t").append(row.getString(index));
				}
				num++;
				if (!TestUtil.getSilent()) write(data.toString());
			}
			dc.commit();
			write("--List cloned reporters OK");
		}
		catch (Throwable ex)
		{
			write("--List cloned FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	
	static void test_externalId(String externalId, int experimentId)
	{
		if (experimentId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			
			Experiment exp = Experiment.getById(dc, experimentId);
			
			ReporterCloneTemplate template = exp.getVirtualDb().getReporterCloneTemplate();
			StringBuilder header = new StringBuilder("  ");
			int numProperties = 0;
			for (ClonableProperty cp : template.getClonableProperties())
			{
				header.append("\t").append(cp.getTitle());
				numProperties++;
			}
			if (!TestUtil.getSilent()) 
			{
				write(header.toString());
				write(header.toString().replaceAll("[^\t]", "-"));
			}
			
			ReporterData rd = Reporter.getByExternalId(dc, externalId, exp);
			StringBuilder data = new StringBuilder();
			data.append(1);
			for (ClonableProperty cp : template.getClonableProperties())
			{
				Object value = null;
				String propertyName = cp.getName();
				if (cp.isExtendedProperty())
				{
					value = rd.getExtended(propertyName);
				}
				else if (propertyName.equals(ClonableProperty.EXTERNAL_ID))
				{
					value = rd.getExternalId();
				}
				else if (propertyName.equals(ClonableProperty.NAME))
				{
					value = rd.getName();
				}
				else if (propertyName.equals(ClonableProperty.ID))
				{
					value = rd.getId();
				}
				data.append("\t").append(value);
			}
			if (!TestUtil.getSilent()) write(data.toString());

			write("--Get cloned by external id OK");
		}
		catch (Throwable ex)
		{
			write("--Get cloned by external id FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	
}
