/**
	$Id: FilterTest.java 5827 2011-10-26 10:51:52Z nicklas $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.test.performance;


import java.util.ArrayList;
import java.util.List;

import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.ItemNotFoundException;
import net.sf.basedb.core.ItemParameterType;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.StringParameterType;
import net.sf.basedb.test.PluginUtil;
import net.sf.basedb.test.TestUtil;
import net.sf.basedb.util.Values;

/**
	Create a filtered bioassayset.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2011-10-26 12:51:52 +0200 (ons, 26 okt. 2011) $
*/
public class FilterTest
{

	public static boolean ok = true;

	public static final String DEFAULT_SOURCE_PREFIX = "Root";
	public static final String DEFAULT_CHILD_PREFIX = "Filtered";
	
	/**
		Run the tests in this class.
	*/
	public static boolean run(String user, String password, 
		String sourcePrefix, String childPrefix, boolean execute)
	{
		DbControl dc = null;
		TestUtil.write("++Entering filter test\n");
		try
		{
			TestUtil.login(user, password);
			
			// Configuration options
			String expression = Config.getFilterExpression();
			if (sourcePrefix == null) sourcePrefix = DEFAULT_SOURCE_PREFIX;
			if (childPrefix == null) childPrefix = DEFAULT_CHILD_PREFIX;

			TestUtil.write("---------------------------------------------\n");
			TestUtil.write("   Expression=" + expression + "\n");
			TestUtil.write("Source prefix=" + sourcePrefix + "\n");
			TestUtil.write(" Child prefix=" + childPrefix + "\n");
			TestUtil.write("      Execute=" + execute + "\n");
			TestUtil.write("---------------------------------------------\n");

			// Load source bioassay sets
			dc = TestUtil.getDbControl();
			Experiment experiment = PrepareTest.getExperiment(dc);
			List<BioAssaySet> sources = new ArrayList<BioAssaySet>(
					Util.findAllBioAssaySets(dc, experiment, sourcePrefix + "%"));
			int numSources = sources.size();
			if (numSources == 0)
			{
				throw new ItemNotFoundException("BioAssaySet[name like " + sourcePrefix + "]");
			}
			dc.close();
			
			// Run filter tests
			long minTime = Long.MAX_VALUE;
			long maxTime = 0;
			long totalTime = 0;
			for (int i = 0; i < numSources; ++i)
			{
				BioAssaySet source = sources.get(i);
				String filteredName = source.getName().replace(sourcePrefix, childPrefix);
				long time = createFilteredBioAssaySet(source, filteredName, expression, execute) / 1000;
				if (execute)
				{
					totalTime += time;
					if (time < minTime) minTime = time;
					if (time > maxTime) maxTime = time;
					TestUtil.write(filteredName + "\t" + Values.formatTime(time) + "\n");
				}
				else
				{
					TestUtil.write("--Filter job created for: " + source.getName() + "-->" + 
							filteredName + "\n");
				}
			}
			
			if (execute)
			{
				TestUtil.write("---------------------------------------------\n");
				TestUtil.write("Filtered " + numSources + " bioassay sets\n");
				TestUtil.write("Max time: " + Values.formatTime(maxTime) + "\n");
				TestUtil.write("Min time: " + Values.formatTime(minTime) + "\n");
				TestUtil.write("Mean time: " + Values.formatTime(totalTime / numSources) + "\n");
			}		
			TestUtil.write("---------------------------------------------\n");
			TestUtil.logout();
		}
		catch (Throwable t)
		{
			ok = false;
			t.printStackTrace();
		}
		finally
		{
			if (dc != null) dc.close();
		}
		TestUtil.write("++Leaving filter test (" + (ok ? "OK" : "FAILED") + ")\n");
		return ok;
	}

	/**
		Create a filtered bioassay set.
	*/
	public static long createFilteredBioAssaySet(BioAssaySet bioAssaySet, 
		String name, String expression, boolean execute)
		throws Exception
	{
		if (execute) TestUtil.write("--Create filtered bioassay set: " + name + "\n");
		DbControl dc = TestUtil.getDbControl();
		try
		{
			PluginDefinition plugin = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.FormulaFilter");
			bioAssaySet = BioAssaySet.getById(dc, bioAssaySet.getId());
			Job job = Job.getNew(dc, plugin, null, bioAssaySet.getExperiment());
			job.setName("PerformanceTest->Filter: " + name);
			job.setSendMessage(false);
			job.setRemoveJobWhenFinished(true);
			job.setParameterValue("source", 
					new ItemParameterType<BioAssaySet>(BioAssaySet.class, null), bioAssaySet);
			job.setParameterValue("childName", 
					new StringParameterType(255, null, true), name);
			job.setParameterValue("expression", 
					new StringParameterType(255, null, true), expression);
			dc.saveItem(job);
			dc.commit();
			long time = 0;
			if (execute)
			{
				time = System.currentTimeMillis();
				PluginUtil.executeJob(job);
				time = System.currentTimeMillis() - time;
			}
			return time;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	
}
